Package: MIMSunit
Type: Package
Title: Algorithm to Compute Monitor Independent Movement Summary Unit
        (MIMS-Unit)
Version: 0.11.3
Date: 2026-05-13
Authors@R: c(person("Qu", "Tang", email = "tang.q@northeastern.edu",
                  role = c("aut"), comment = c(ORCID = "0000-0001-5415-0205")),
             person("Dinesh", "John", email = "d.john@northeastern.edu",
                  role = c("aut")),
             person("Stephen", "Intille", email = "s.intille@northeastern.edu",
                  role = c("aut")),
             person("Umberto", "Mazzucchelli", email = "mazzucchelli.u@northeastern.edu",
                  role = c("cre"), comment = c(ORCID = "0009-0000-1071-6027")),
             person("mHealth Research Group", role = c("cph"), 
             comment = "https://www.mhealthgroup.org")
            )
Description: The MIMS-unit algorithm is developed to compute Monitor Independent
    Movement Summary Unit, a measurement to summarize raw accelerometer data 
    while ensuring harmonized results across different devices. It also includes
    scripts to reproduce results in the related publication 
    (John, D., Tang. Q., Albinali, F. and Intille, S. (2019) <doi:10.1123/jmpb.2018-0068>).
License: MIT + file LICENSE
Language: en-US
Depends: R (>= 3.6.0)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.0
URL: https://mhealthgroup.github.io/MIMSunit/,
        https://github.com/mhealthgroup/MIMSunit/tree/master
BugReports: https://github.com/mhealthgroup/MIMSunit/issues/
Imports: caTools (>= 1.17.1.1), tibble (>= 3.0.4), dplyr (>= 0.7.7),
        lubridate (>= 1.7.4), magrittr (>= 1.5), plyr (>= 1.8.4), readr
        (>= 1.1.1), R.utils (>= 2.7.0), stringr (>= 1.3.1), xts (>=
        0.11-2), signal (>= 0.7-7), dygraphs (>= 1.1.1.6), shiny (>=
        1.4.0.2), RColorBrewer (>= 1.1-2), utils (>= 3.6.1), ggplot2
        (>= 3.2.1)
Suggests: testthat, pkgdown, gridExtra, remotes
SystemRequirements: memory (>= 4GB) Ubuntu: build-essential,
        libxml2-dev, libssl-dev, libcurl4-openssl-dev Windows: Rtools
        (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-13 21:00:09 UTC; umberto
Author: Qu Tang [aut] (ORCID: <https://orcid.org/0000-0001-5415-0205>),
  Dinesh John [aut],
  Stephen Intille [aut],
  Umberto Mazzucchelli [cre] (ORCID:
    <https://orcid.org/0009-0000-1071-6027>),
  mHealth Research Group [cph] (https://www.mhealthgroup.org)
Maintainer: Umberto Mazzucchelli <mazzucchelli.u@northeastern.edu>
Repository: CRAN
Date/Publication: 2026-05-14 14:00:11 UTC
Built: R 4.5.2; ; 2026-05-14 14:22:03 UTC; unix

Package: MIMSunit
Type: Package
Title: Algorithm to Compute Monitor Independent Movement Summary Unit
        (MIMS-Unit)
Version: 0.11.3
Date: 2026-05-13
Authors@R: c(person("Qu", "Tang", email = "tang.q@northeastern.edu",
                  role = c("aut"), comment = c(ORCID = "0000-0001-5415-0205")),
             person("Dinesh", "John", email = "d.john@northeastern.edu",
                  role = c("aut")),
             person("Stephen", "Intille", email = "s.intille@northeastern.edu",
                  role = c("aut")),
             person("Umberto", "Mazzucchelli", email = "mazzucchelli.u@northeastern.edu",
                  role = c("cre"), comment = c(ORCID = "0009-0000-1071-6027")),
             person("mHealth Research Group", role = c("cph"), 
             comment = "https://www.mhealthgroup.org")
            )
Description: The MIMS-unit algorithm is developed to compute Monitor Independent
    Movement Summary Unit, a measurement to summarize raw accelerometer data 
    while ensuring harmonized results across different devices. It also includes
    scripts to reproduce results in the related publication 
    (John, D., Tang. Q., Albinali, F. and Intille, S. (2019) <doi:10.1123/jmpb.2018-0068>).
License: MIT + file LICENSE
Language: en-US
Depends: R (>= 3.6.0)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.0
URL: https://mhealthgroup.github.io/MIMSunit/,
        https://github.com/mhealthgroup/MIMSunit/tree/master
BugReports: https://github.com/mhealthgroup/MIMSunit/issues/
Imports: caTools (>= 1.17.1.1), tibble (>= 3.0.4), dplyr (>= 0.7.7),
        lubridate (>= 1.7.4), magrittr (>= 1.5), plyr (>= 1.8.4), readr
        (>= 1.1.1), R.utils (>= 2.7.0), stringr (>= 1.3.1), xts (>=
        0.11-2), signal (>= 0.7-7), dygraphs (>= 1.1.1.6), shiny (>=
        1.4.0.2), RColorBrewer (>= 1.1-2), utils (>= 3.6.1), ggplot2
        (>= 3.2.1)
Suggests: testthat, pkgdown, gridExtra, remotes
SystemRequirements: memory (>= 4GB) Ubuntu: build-essential,
        libxml2-dev, libssl-dev, libcurl4-openssl-dev Windows: Rtools
        (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-13 21:00:09 UTC; umberto
Author: Qu Tang [aut] (ORCID: <https://orcid.org/0000-0001-5415-0205>),
  Dinesh John [aut],
  Stephen Intille [aut],
  Umberto Mazzucchelli [cre] (ORCID:
    <https://orcid.org/0009-0000-1071-6027>),
  mHealth Research Group [cph] (https://www.mhealthgroup.org)
Maintainer: Umberto Mazzucchelli <mazzucchelli.u@northeastern.edu>
Repository: CRAN
Date/Publication: 2026-05-14 14:00:11 UTC
Built: R 4.6.0; ; 2026-05-14 15:46:25 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aggregate_for_mims)
export(aggregate_for_orientation)
export(bandlimited_interp)
export(clip_data)
export(compute_orientation)
export(custom_mims_unit)
export(cut_off_signal)
export(export_to_actilife)
export(extrapolate)
export(extrapolate_rate)
export(extrapolate_single_col)
export(generate_interactive_plot)
export(iir)
export(illustrate_extrapolation)
export(illustrate_signal)
export(import_actigraph_count_csv)
export(import_actigraph_csv)
export(import_actigraph_csv_chunked)
export(import_actigraph_meta)
export(import_activpal3_csv)
export(import_enmo_csv)
export(import_mhealth_csv)
export(import_mhealth_csv_chunked)
export(interpolate_signal)
export(mims_unit)
export(mims_unit_from_files)
export(parse_epoch_string)
export(sampling_rate)
export(segment_data)
export(sensor_orientations)
export(shiny_app)
export(simulate_new_data)
export(sum_up)
export(vector_magnitude)
importFrom(magrittr,"%>%")

<!-- README.md is generated from README.Rmd. Please don't edit that file -->

# MIMS-unit algorithm

<!-- badges: start -->

[![CRAN
Status](https://www.r-pkg.org/badges/version/MIMSunit)](https://cran.r-project.org/package=MIMSunit)
[![Pkgdown](https://github.com/mHealthGroup/MIMSunit/workflows/Pkgdown/badge.svg?branch=master)](https://github.com/mHealthGroup/MIMSunit/)
[![.github/workflows/test.yml](https://github.com/mHealthGroup/MIMSunit/workflows/.github/workflows/test.yml/badge.svg)](https://github.com/mHealthGroup/MIMSunit/)
[![AppVeyor build
status](https://ci.appveyor.com/api/projects/status/github/muschellij2/MIMSunit?branch=master&svg=true)](https://ci.appveyor.com/project/muschellij2/MIMSunit)
[![](https://cranlogs.r-pkg.org/badges/MIMSunit)](https://cran.r-project.org/package=MIMSunit)
<!-- badges: end -->

**Please create github
[issues](https://github.com/mhealthgroup/MIMSunit/issues/) if you have
any question related to the package. Please click the following button
to subscribe to any update on the package (Very important to receive
notifications about bugs and bug fixes).**

<!-- Place this tag where you want the button to render. -->

<a class="github-button" href="https://github.com/mhealthgroup/MIMSunit/subscription" data-color-scheme="no-preference: dark; light: light; dark: dark;" data-size="large" data-show-count="true" aria-label="Watch mhealthgroup/MIMSunit on GitHub">Watch</a>

## Short introduction

MIMS-unit is abbreviated for *Monitor Independent Movement Summary*
unit. This measurement is developed to harmonize the processing of
accelerometer data from different devices. You may refer to the
[manuscript](https://doi.org/10.1123/jmpb.2018-0068) for the detail
description of the algorithm.

## Copyright and citation

The copyright of the work belongs to Northeastern University, [mHealth
Research Group](https://www.mhealthgroup.org/). Please kindly [cite the
manuscript](https://mhealthgroup.github.io/MIMSunit/authors.html) if you
have used the package or referred to the algorithm in your work.

## Shiny Demo App

You may try to compute MIMS-unit values using our shiny demo app
<https://qutang.shinyapps.io/MIMSunit/>. Note that the upload file size
limit is 50 MB. The usage quote for the server is limited, so we do not
guarantee the web app is always available to you.

## Datasets

All datasets used in the manuscript are available at
<https://mhealthgroup.github.io/MIMSunit/articles/datasets.html>.

## System Requirements

1.  R (&gt;= 3.6.0)
2.  memory (&gt; 4GB)

### For Windows

Rtools &gt;=3.5 (see: <https://cran.r-project.org/bin/windows/Rtools/>)

### For Linux (use ubuntu as an example)

Install dependency system packages for `devtools`: `build-essential`,
`libcurl4-gnutls-dev`, `libxml2-dev`, `libssl-dev`, `libcurl4-ssl-dev`.

## Installation

### Stable version

CRAN

``` r
install.packages('MIMSunit')
```

### Development version

Github

``` r
install.packages("devtools")
devtools::install_github("mhealthgroup/MIMSunit")
```

*Note: It is recommended to use Rstudio when installing the package,
because `devtools` has some compatible issues with R command line
interface.*

## Usage

``` r
MIMSunit::mims_unit(input_dataframe, dynamic_range=c(-3,3), epoch='1 min')
```

Assume the input dataframe is in following format, with the first column
(timestamp) in `POSXlct` objects and the device used to collect this
data has dynamic range being -3g to 3g. You may set the epoch length to
be `1 min`, `1 sec`, `5 sec`, `10 sec` and so on.

    HEADER_TIME_STAMP,X,Y,Z
    2016-10-03 14:51:14.236,0.007,-0.005,0.984
    2016-10-03 14:51:14.256,0.008,-0.007,0.981
    2016-10-03 14:51:14.276,0.009,-0.006,0.978
    2016-10-03 14:51:14.297,0.009,-0.007,0.984
    2016-10-03 14:51:14.317,0.010,-0.010,0.982
    2016-10-03 14:51:14.337,0.011,-0.010,0.982

#' This is an example Shiny application
#' It is the same as example_02 in the RStudio Shiny Tutorial

library(shiny)
library(dygraphs)
library(xts)

options(shiny.maxRequestSize=50*1024^2) # Limit to upload data size to 50 MB

server <- function(input, output) {

  mims_unit_values <- eventReactive(input$compute, {
    file_paths = input$input_files$datapath
    results = c()
    if (length(file_paths) == 1){
      input_files = c(file_paths)
      results = MIMSunit::mims_unit_from_files(
        c(input_files),
        epoch = paste(input$epoch, 'sec'),
        dynamic_range = c(-input$dynamic_range, input$dynamic_range),
        file_type = input$file_type,
        output_mims_per_axis = TRUE, header=input$header, has_ts=input$timestamps)
    } else if (length(file_paths) > 1 && !input$independent) {
      input_files = file_paths
      results = MIMSunit::mims_unit_from_files(
        c(input_files),
        epoch = paste(input$epoch, 'sec'),
        dynamic_range = c(-input$dynamic_range, input$dynamic_range),
        file_type = input$file_type,
        output_mims_per_axis = TRUE, header=input$header, has_ts=input$timestamps)
    } else if (length(file_paths) > 1 && input$independent) {
      for (file_path in file_paths) {
        result = MIMSunit::mims_unit_from_files(
          c(file_path),
          epoch = paste(input$epoch, 'sec'),
          dynamic_range = c(-input$dynamic_range, input$dynamic_range),
          file_type = input$file_type,
          output_mims_per_axis = TRUE, header=input$header, has_ts=input$timestamps)
      }
      results = append(results, result)
    }
    return(results)
  })

  output$dygraph <- renderDygraph({
    results = mims_unit_values()
    mims_xts = xts(results[,2:5], results[,1])
    dygraph(mims_xts) %>% dyRangeSelector() %>% dyOptions(colors = RColorBrewer::brewer.pal(4, "Set2")) %>% dyLegend(width = 400) %>% dyAxis("y", valueRange = c(0, 2 * max(mims_xts)))
  })

  output$download_data <- downloadHandler(
    filename = function() {
      paste('mims_unit_values_', Sys.Date(), '.csv', sep='')
    },
    content = function(con) {
      write.csv(mims_unit_values(), con, row.names = FALSE, quote = FALSE)
    }
  )
}

#' This is an example shiny application
#' It is the same as example_02 in the RStudio Shiny Tutorial

library(shiny)
library(dygraphs)

ui <- fluidPage(

  # Application title
  titlePanel("Compute MIMS-unit values"),

  # Sidebar with a slider input for number of bins
  sidebarLayout(
    sidebarPanel(
      fileInput("input_files", "Choose CSV Files",
                multiple = TRUE,
                accept = c(
                  "text/csv",
                  "text/comma-separated-values,text/plain",
                  ".csv")
      ),
      selectInput("file_type", "File type:",
                  c("mHealth" = "mhealth",
                    "Actigraph" = "actigraph"
                  )
      ),
      conditionalPanel(
        condition = "input.input_files && input.input_files.length > 1",
        checkboxInput("independent", "Process each file independently", TRUE)
      ),
      conditionalPanel(
        condition = "input.file_type == 'actigraph'",
        checkboxInput("header", "Header on the first row", TRUE),
        checkboxInput("timestamps", "Timestamps on the first column", TRUE)
      ),
      sliderInput("epoch",
                  "Epoch (secs)",
                  min = 1,
                  max = 300,
                  value = 5,
                  step = 1
      ),
      sliderInput("dynamic_range",
                  "Dynamic range (g):",
                  min = 1,
                  max = 20,
                  value = 8,
                  step = 1),
      actionButton("compute", "Compute MIMS-unit values")
    ),

    # Show a plot of the generated distribution
    mainPanel(
      dygraphOutput("dygraph"),
      conditionalPanel(
        condition = "output.dygraph",
        downloadButton("download_data", "Download")
      )
    )
  )
)

#' Get extrapolation rate.
#'
#' \code{extrapolate_rate} computes the extrapolation rate given the test signal
#' (maxed out), the true complete signal (no maxed out) and the extrapolated
#' signal.
#'
#' All three input dataframes will have the same format, with the first column
#' being timestamps in POSXlct format, and the following columns being
#' acceleration values in g.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is used to
#'   compute extrapolation rate during extrapolation parameter optimization. You
#'   may see results in Figure 2 of the manuscript.
#'
#' @param test_df dataframe. See details for the input format.
#' @param true_df dataframe. See details for the input format.
#' @param extrap_df dataframe. See details for the input format.
#' @return number. The extrapolation rate value in double format. If
#'   extrapolation rate is 1, it means the extrapolated signal recovers as the
#'   true signal. If extrapolation rate is between 0 and 1, it means the
#'   extrapolation helps reducing the errors caused by signal maxing out. If
#'   extrapolation rate is smaller than 0, it means the extrapolation increases
#'   the errors caused by signal maxing out (during over extrapolation).
#' @family extrapolation related functions
#' @export
#' @examples
#'   # Prepare data for test, ground truth
#'   test_df = conceptual_diagram_data[
#'                 conceptual_diagram_data['GRANGE'] == 4,
#'                 c("HEADER_TIME_STAMP", "X")]
#'   true_df = conceptual_diagram_data[
#'                 conceptual_diagram_data['GRANGE'] == 8,
#'                 c("HEADER_TIME_STAMP", "X")]
#'
#'   # Do extrapolation
#'   extrap_df = extrapolate(test_df, range=c(-4, 4))
#'
#'   # Compute extrapolation rate
#'   extrapolate_rate(test_df, true_df, extrap_df)
extrapolate_rate <- function(test_df, true_df, extrap_df) {
  true_auc <- caTools::trapz(true_df[[1]], abs(true_df[[2]]))
  test_auc <- caTools::trapz(test_df[[1]], abs(test_df[[2]]))
  extrapolated_auc <-
    caTools::trapz(extrap_df[[1]], abs(extrap_df[[2]]))
  test_err <- (true_auc - test_auc) / true_auc
  extrapolated_err <- (true_auc - extrapolated_auc) / true_auc
  extrapolated_rate <- (test_err - abs(extrapolated_err)) / test_err
  return(extrapolated_rate)
}

#' Plot given raw signal in illustration diagram style.
#'
#' \code{illustrate_signal} plots the given uniaxial signal in illustration diagram style.
#'   Illustration diagram style hides axes markers, unnecessary guidelines.
#'
#' @param data data.frame. The input uniaxial signal. First column should be timestamp.
#' @param point_size number. The size of the plotted data point.
#' @param plot_point Bool. Plot signal as points if TRUE.
#' @param line_size number. The line width of the plotted signal curve.
#' @param plot_line Bool. Plot signal with curve if TRUE.
#' @param range vector. Dynamic range of the signal.
#' @param plot_maxed_out_line Bool. Plot dynamic range lines if TRUE. Dynamic range is set by `range`.
#' @param plot_origin Bool. Plot the 0 horizontal line if TRUE.
#' @param title Char. The title of the plot.
#' @param plot_title Bool. Plot title if TRUE.
#'
#' @return ggplot2 graph object. The graph to be shown.
#' @family visualization functions.
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = sample_raw_accel_data
#'
#'   # Plot it with default settings
#'   illustrate_signal(df)
#'
#'   # Plot with a different style
#'   illustrate_signal(df, point_size=1, line_size=1)
#'
#'   # Turn off annotation lines
#'   illustrate_signal(df, plot_maxed_out_line = FALSE, plot_origin = FALSE)
#'
#'   # Use title
#'   illustrate_signal(df, plot_title=TRUE, title = "This is a title")
illustrate_signal <- function(data,
                              point_size = 0.3,
                              plot_point = TRUE,
                              line_size = 0.3,
                              plot_line = TRUE,
                              range = c(-2, 2),
                              plot_maxed_out_line = TRUE,
                              plot_origin = TRUE,
                              title = NULL,
                              plot_title = TRUE) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package \"ggplot2\" needed for this function to work. Please install it.",
      call. = FALSE
    )
  }
  colnames(data) <- c("HEADER_TIME_STAMP", "value")
  p <- ggplot2::ggplot(
    data = data,
    ggplot2::aes_string(x = 'HEADER_TIME_STAMP', y = 'value')
  )
  if (plot_line) {
    p <- p + ggplot2::geom_line(size = line_size)
  }
  if (plot_point) {
    p <- p + ggplot2::geom_point(size = point_size)
  }
  if (plot_maxed_out_line) {
    p <- p + ggplot2::geom_hline(
      yintercept = range[1],
      size = 0.3,
      linetype = "dashed"
    )
    p <- p + ggplot2::geom_hline(
      yintercept = range[2],
      size = 0.3,
      linetype = "dashed"
    )
  }
  if (plot_origin) {
    p <- p + ggplot2::geom_hline(yintercept = 0, size = 0.1)
  }
  p <- p + ggplot2::theme_bw(base_size = 12) + ggplot2::theme(
    panel.grid.major = ggplot2::element_blank(),
    panel.grid.minor = ggplot2::element_blank(),
    axis.title.x = ggplot2::element_blank(),
    axis.text.x = ggplot2::element_blank(),
    axis.ticks.x = ggplot2::element_blank(),
    axis.title.y = ggplot2::element_blank(),
    axis.text.y = ggplot2::element_blank(),
    axis.ticks.y = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(0.1, 0.1, -0.32, -0.32), "line")
  ) + ggplot2::ylim(-8.1, 8.1) + ggplot2::labs(x = NULL, y = NULL, title = title)
  return(p)
}


#' Plot illustrations about extrapolation in illustration style.
#'
#' \code{illustrate_extrapolation} plots elements of extrapolations
#' (e.g., marked points, reference lines) in the same style as
#' \code{\link{illustrate_signal}}.
#'
#' @param df data.frame.The original data before extrapolation.
#' @param dynamic_range numerical vector. The dynamic ranges of the input
#'   signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach.
#' @param title Char. The title of the plot.
#' @param show_neighbors bool. Show the points used for extrapolation if TRUE.
#' @param show_extrapolated_points_and_lines bool. Show the extrapolated points
#'  and curves used for extrapolation.
#' @param ... Parameters that can be used to tune extrapolation, including
#'   \code{spar}, \code{k}, and \code{noise_level}. See \code{\link{extrapolate}}
#'   for explanations.
#'
#' @return ggplot2 graph object. The graph to be shown.
#' @family visualization functions.
#' @export
#' @examples
#'   # Use the maxed-out data for the conceptual diagram
#'   df = conceptual_diagram_data[
#'          conceptual_diagram_data['GRANGE'] == 2,
#'          c("HEADER_TIME_STAMP", "X")]
#'
#'   # Plot extrapolation illustration using default settings
#'   illustrate_extrapolation(df, dynamic_range=c(-2,2))
#'
#'   # Do not show neighbor points
#'   illustrate_extrapolation(df, dynamic_range=c(-2,2), show_neighbors=FALSE)
#'
#'   # Do not show extrapolated points and lines
#'   illustrate_extrapolation(df,
#'                            dynamic_range=c(-2,2),
#'                            show_extrapolated_points_and_lines=FALSE)
illustrate_extrapolation <-
  function(df,
           dynamic_range,
           title = NULL,
           show_neighbors = TRUE,
           show_extrapolated_points_and_lines = TRUE, ...) {
    results = .prepare_for_extrapolation_illustration(df, dynamic_range = dynamic_range, ...)
    if(is.null(results$between_neighbor_df)) {
      p <- illustrate_signal(
        df,
        point_size = 1,
        plot_line = FALSE,
        title = title,
        range = dynamic_range,
        plot_title = TRUE
      )
      return(p)
    } else {
      p <- illustrate_signal(
        results$between_neighbor_df[, 1:2],
        point_size = 1,
        plot_line = FALSE,
        title = title,
        range = dynamic_range,
        plot_title = TRUE
      )
      if (show_neighbors) {
        if (!is.null(results$left_neighbors_df)) {
          p <- p +
            ggplot2::geom_point(
              data = results$left_neighbors_df,
              ggplot2::aes_string(x = 'HEADER_TIME_STAMP', y = 'VALUE'),
              shape = 1,
              size = 1
            )
        }
        if (!is.null(results$right_neighbors_df)) {
          p <- p +
            ggplot2::geom_point(
              data = results$right_neighbors_df,
              ggplot2::aes_string(x = 'HEADER_TIME_STAMP', y = 'VALUE'),
              shape = 1,
              size = 1
            )
        }
      }
      if (show_extrapolated_points_and_lines) {
        if (!is.null(results$points_ex_df)) {
          p <- p +
            ggplot2::geom_point(
              data = results$points_ex_df,
              ggplot2::aes_string(x = 'HEADER_TIME_STAMP', y = 'VALUE'),
              shape = 17,
              size = 2
            ) +
            ggplot2::geom_vline(
              xintercept = as.numeric(results$points_ex_df[, 1]),
              size = 0.3,
              linetype = "dotted"
            )
        }

        if (!is.null(results$fitted_line_df)) {
          indices <- unique(results$fitted_line_df$index)
          for (i in indices) {
            left_fit_line_df <- results$fitted_line_df[results$fitted_line_df$index == i &
                                                         results$fitted_line_df$type == "left_line", c(1, 2)]
            right_fit_line_df <- results$fitted_line_df[results$fitted_line_df$index == i &
                                                          results$fitted_line_df$type == "right_line", c(1, 2)]
            colnames(left_fit_line_df) <- c("HEADER_TIME_STAMP", "value")
            colnames(right_fit_line_df) <- c("HEADER_TIME_STAMP", "value")
            p <- p + ggplot2::geom_line(
              data = left_fit_line_df,
              ggplot2::aes_string(x = "HEADER_TIME_STAMP", y = "value"),
              size = 0.5
            ) +
              ggplot2::geom_line(
                data = right_fit_line_df,
                ggplot2::aes_string(x = "HEADER_TIME_STAMP", y = "value"),
                size = 0.5
              )
          }
        }
      }
      return(p)
    }
  }

.prepare_for_extrapolation_illustration = function(df, dynamic_range, noise_level=0.03, k=0.05, spar=0.6) {
  df = df[,c(1,2)]
  df = data.frame(.extrapolate_oversampling(df[[1]], df[[2]]))
  colnames(df) = c('HEADER_TIME_STAMP', 'VALUE')
  markers = .extrapolate_mark("gamma")(df$HEADER_TIME_STAMP,
                                       df$VALUE,
                                       dynamic_range[1],
                                       dynamic_range[2],
                                       noise_level)
  markers_df = data.frame(HEADER_TIME_STAMP = df$HEADER_TIME_STAMP,
                          VALUE = abs(markers))
  neighbors = .extrapolate_neighbor(markers, 100, k)
  if (nrow(neighbors) > 0) {
    left_indices = c()
    right_indices = c()
    mo_indices = c()
    for (i in 1:nrow(neighbors)) {
      left_indices = c(left_indices,
                       neighbors$left_start[i]:neighbors$left_end[i])
      right_indices = c(right_indices,
                        neighbors$right_start[i]:neighbors$right_end[i])
      mo_indices = c(mo_indices,
                     neighbors$left_end[i]:neighbors$right_start[i])
    }
    fitted_line_df = .extrapolate_fitline(
      df[[1]],
      df[[2]],
      neighbors,
      markers,
      spar,
      100,
      k
    )
    colnames(fitted_line_df) = c('HEADER_TIME_STAMP', 'VALUE', 'type', 'index')
    left_neighbors_df = df[left_indices, ]
    right_neighbors_df = df[right_indices, ]
    between_neighbor_df = df[c(-left_indices, -right_indices), ]
    between_neighbor_without_maxed_out_df = df[c(-left_indices, -right_indices, -mo_indices), ]
    maxed_out_df = df[mo_indices, ]
    maxed_out_df$weight = markers_df[mo_indices, 'value']
    points_ex_df = fitted_line_df[fitted_line_df$type == 'point', c(1, 2)]
  } else {
    marks_df = NULL
    left_neighbors_df = NULL
    right_neighbors_df = NULL
    maxed_out_df = NULL
    between_neighbor_df = NULL
    between_neighbor_without_maxed_out_df = NULL
    fitted_line_df = NULL
    points_ex_df = NULL
  }
  return(
    list(
      df = df,
      between_neighbor_df = between_neighbor_df,
      left_neighbors_df = left_neighbors_df,
      right_neighbors_df = right_neighbors_df,
      maxed_out_df = maxed_out_df,
      between_neighbor_without_maxed_out_df = between_neighbor_without_maxed_out_df,
      markers_df = markers_df,
      fitted_line_df = fitted_line_df,
      points_ex_df = points_ex_df
    )
  )
}


#' Plot MIMS unit values or raw signal using dygraphs interactive plotting library.
#'
#' \code{generate_interactive_plot} plots MIMS unit values or raw signal
#'   using dygraphs interactive plotting library.
#'
#' @param df data.frame.The dataframe storing MIMS unit values or raw
#'   accelerometer signal. The first column should be timestamps.
#' @param y_label str. The label name to be put on the y axis.
#' @param value_cols numerical vector. The indices of columns storing values,
#'   typically starting from the second column. The default is `c(2,3,4)`.
#'
#' @importFrom magrittr %>%
#' @return A dygraphs graph object. When showing, the graph will be plotted in
#'   a html widgets in an opened browser.
#' @family visualization functions.
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = sample_raw_accel_data
#'
#'   # Plot using default settings, due to pkgdown limitation, no interactive
#'   # plots will be shown on the website page.
#'   generate_interactive_plot(df,
#'                             y_label="Acceleration (g)")
#'
#'   # The function can be used to plot MIMS unit values as well
#'   mims = mims_unit(df, dynamic_range=c(-8, 8))
#'   generate_interactive_plot(mims,
#'                             y_label="MIMS-unit values",
#'                             value_cols=c(2))
generate_interactive_plot = function(df, y_label, value_cols=c(2,3,4)) {
  xts_values = xts::xts(df[,value_cols], df[,1])
  range = max(xts_values) - min(xts_values)
  min_y = min(xts_values) - range * 1/2
  max_y = max(xts_values) + range * 1/2
  g = dygraphs::dygraph(xts_values) %>%
    dygraphs::dyRangeSelector() %>%
    dygraphs::dyOptions(colors = RColorBrewer::brewer.pal(4, "Set2")) %>%
    dygraphs::dyLegend(width = 400) %>%
    dygraphs::dyAxis("y", valueRange = c(min_y, max_y), label = y_label) %>%
    dygraphs::dyAxis("x", label = 'Time')
  return(g)
}

#' Import raw multi-channel accelerometer data stored in mHealth Specification
#'
#' \code{import_mhealth_csv} imports the raw multi-channel accelerometer data
#'   stored in mHealth Specification. Note that this function will fail when
#'   loading data that have size too big to fit in the memory. For large data
#'   file, please use \code{\link{import_mhealth_csv_chunked}} to load.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import data stored in mHealth Specification during
#'   algorithm validation.
#'
#' @param filepath string. The filepath of the input data.
#' @return dataframe. The imported multi-channel accelerometer signal, with the
#'   first column being the timestamps in POSXlct format, and the rest columns
#'   being accelerometer values in \eqn{g} unit.
#'
#' @family File I/O functions
#'
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'   # Use the sample mhealth csv file provided by the package
#'   filepath = system.file('extdata', 'mhealth.csv', package='MIMSunit')
#'   filepath
#'
#'   # Load the file
#'   df = import_mhealth_csv(filepath)
#'
#'   # Check loaded file
#'   head(df)
#'
#'   # Check more
#'   summary(df)
#'
#'   # Restore default options
#'   options(default_ops)
import_mhealth_csv <- function(filepath) {
  ncols <-
    readr::count_fields(filepath,
                        readr::tokenizer_csv(),
                        skip = 0,
                        n_max = 1L)
  date_format <- readr::col_datetime(format = "%Y-%m-%d %H:%M:%OS")
  coltypes <- list(date_format)
  if (ncols == 4) {
    colheaders <- c("HEADER_TIME_STAMP", "X", "Y", "Z")
  } else if (ncols == 2) {
    colheaders <- c("HEADER_TIME_STAMP", "VALUE")
  } else {
    stop('The file does not contain the correct columns for mhealth raw accelerometer data.')
  }
  for (i in 2:ncols) {
    coltypes <- append(coltypes, list(readr::col_double()))
  }

  # Preserve legacy readr parsing semantics while supporting readr >= 2.
  df <- readr::with_edition(
    1,
    readr::read_csv(file = filepath,
                    quoted_na = TRUE,
                    col_types = coltypes)
  )
  # convert factors back to characters
  col_classes <- sapply(1:ncols, function(i) {
    return(class(df[1, i]))
  })
  factor_cols <- which(col_classes == "factor")
  if (length(factor_cols) > 0) {
    df[, factor_cols] <- as.character(df[, factor_cols])
  }

  # enhance column headers
  colnames(df)[1:length(colheaders)] <- colheaders
  colnames(df) <- toupper(colnames(df))
  df <- data.frame(df, stringsAsFactors = FALSE)

  return(df)
}

#' Import large raw multi-channel accelerometer data stored in mHealth Specification
#' in chunks.
#'
#' \code{import_mhealth_csv_chunked} imports the raw multi-channel accelerometer
#'   data stored in mHealth Specification in chunks.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import data stored in mHealth Specification during
#'   algorithm validation.
#'
#' @param filepath string. The filepath of the input data.
#' @param chunk_samples number. The number of samples in each chunk. Default is
#'   180000, which is half hour data for 100 Hz sampling rate.
#' @return list. The list contains two items. The first item is a generator
#'   function that each time it is called, it will
#'   return a dataframe with at most \code{chunk_samples} samples of imported data.
#'   The third item is a \code{close_connection} function which you can call at
#'   any moment to close the file loading.
#' @family File I/O functions
#'
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'
#'   # Use the mhealth csv file shipped with the package
#'   filepath = system.file('extdata', 'mhealth.csv', package='MIMSunit')
#'
#'   # Example 1
#'   # Load chunks every 1000 samples
#'   results = import_mhealth_csv_chunked(filepath, chunk_samples=100)
#'   next_chunk = results[[1]]
#'   close_connection = results[[2]]
#'   # Check data as chunks, you can see chunk time is shifting forward at each iteration.
#'   n = 1
#'   repeat {
#'     df = next_chunk()
#'     if (nrow(df) > 0) {
#'       print(paste('chunk', n))
#'       print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
#'       n = n + 1
#'     } else {
#'       break
#'     }
#'   }
#'
#'   # Close connection after reading all the data
#'   close_connection()
#'
#'   # Example 2: close loading early
#'   results = import_mhealth_csv_chunked(filepath, chunk_samples=1000)
#'   next_chunk = results[[1]]
#'   close_connection = results[[2]]
#'   # Check data as chunks, you can see chunk time is shifting forward at each iteration.
#'   n = 1
#'   repeat {
#'     df = next_chunk()
#'     if (nrow(df) > 0) {
#'       print(paste('chunk', n))
#'       print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
#'       n = n + 1
#'       close_connection()
#'     }
#'     else {
#'       break
#'     }
#'   }
#'
#'  # Restore default options
#'  options(default_ops)
import_mhealth_csv_chunked <-
  function(filepath, chunk_samples = 180000) {
    chunk_size <- chunk_samples
    col_types <- c("character", "numeric", "numeric", "numeric")
    col_names <- c("HEADER_TIME_STAMP", "X", "Y", "Z")

    con <- file(filepath, open = "r")

    df = NULL

    next_chunk <- function() {
      is_open <- tryCatch({
        isOpen(con)
      },
      error =
        function(cond) {
          return(FALSE)
        })
      if (!is_open) {
        return(data.frame())
      }
      df <- utils::read.csv(
        file = con,
        header = TRUE,
        skip = 0,
        nrows = chunk_size,
        colClasses = col_types,
        col.names = col_names
      )
      df <- .convert_mhealth_timestamps(df)
      df <- cbind(as.data.frame(df))
      return(df)
    }

    close_connection <- function() {
      close(con)
    }
    return(list(next_chunk, close_connection))
  }

.convert_mhealth_timestamps <- function(dat) {
  time_format <- "%Y-%m-%d %H:%M:%OS"
  dat[["HEADER_TIME_STAMP"]] <-
    strptime(x = dat[["HEADER_TIME_STAMP"]], format = time_format) + 5e-04
  return(dat)
}



#' Import raw multi-channel accelerometer data stored in ActivPal3 csv format
#'
#' \code{import_activpal3_csv} imports the raw multi-channel accelerometer data
#' stored in ActivPal3 csv format by converting the accelerometer values (in
#' digital voltage values) to \eqn{g} unit.
#'
#' ActivPal 3 sensors have known dynamic range to be \eqn{(-2g, +2g)}. And the
#' sensor stores values using 8-bit memory storage. So, the digital voltage
#' values may be converted to \eqn{g} unit using following equation.
#'
#' \deqn{x_g = \frac{x_{voltage} - 127}{2^8} * 4}
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import data from ActivPal3 devices during
#'   algorithm validation.
#'
#' @param filepath string. The filepath of the input data.
#' @param header boolean. If TRUE, the input csv file will have column names in
#'   the first row.
#' @return dataframe. The imported multi-channel accelerometer signal, with the
#'   first column being the timestamps in POSXlct format, and the rest columns
#'   being accelerometer values in \eqn{g} unit.
#'
#' @family File I/O functions
#'
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'   # Use the sample activpal3 csv file provided by the package
#'   filepath = system.file('extdata', 'activpal3.csv', package='MIMSunit')
#'
#'   # Check the csv format
#'   readLines(filepath)[1:5]
#'
#'   # Load the file, in our case without header
#'   df = import_activpal3_csv(filepath, header=FALSE)
#'
#'   # Check loaded file
#'   head(df)
#'
#'   # Check more
#'   summary(df)
#'
#'   # Restore default options
#'   options(default_ops)
import_activpal3_csv <- function(filepath, header = FALSE) {
  ncols <-
    readr::count_fields(filepath, readr::tokenizer_csv(), n_max = 1)
  col_types <- paste(rep("d", ncols), collapse = "")
  if (header) {
    dat <-
      readr::read_csv(
        filepath,
        col_names = TRUE,
        trim_ws = TRUE,
        col_types = col_types
      )
  } else {
    dat <-
      readr::read_csv(
        filepath,
        col_names = FALSE,
        trim_ws = TRUE,
        col_types = col_types
      )
  }
  dat <- dat[1:4]
  colnames(dat) <- c("HEADER_TIME_STAMP", "X", "Y", "Z")
  dat["HEADER_TIME_STAMP"] <-
    as.POSIXct(dat[["HEADER_TIME_STAMP"]] * 60 * 60 * 24, origin = "1899-12-30", tz = "GMT")
  dat["HEADER_TIME_STAMP"] <-
    lubridate::force_tz(dat["HEADER_TIME_STAMP"], tzone = Sys.timezone())
  dat[2:4] <- (dat[2:4] - 127) / 2 ^ 8 * 4
  dat <- as.data.frame(dat)
  return(dat)
}

#' Import large raw multi-channel accelerometer data stored in Actigraph raw csv
#' format in chunks
#'
#' \code{import_actigraph_csv_chunked} imports the raw multi-channel accelerometer data
#' stored in Actigraph raw csv format. It supports files from the following
#' devices: GT3X, GT3X+, GT3X+BT, GT9X, and GT9X-IMU.
#'
#' For old device (GT3X) that stores accelerometer values as digital voltage.
#' The function will convert the values to \eqn{g} unit using the following
#' equation.
#'
#' \deqn{x_g = \frac{x_{voltage}r}{(2 ^ r) - \frac{v}{2}}}
#'
#' Where \eqn{v} is the max voltage corresponding to the max accelerometer value
#' that can be found in the meta section in the csv file; \eqn{r} is the
#' resolution level which is the number of bits used to store the voltage
#' values. \eqn{r} can also be found in the meta section in the csv file.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import data from Actigraph devices during
#'   algorithm validation.
#'
#' @param filepath string. The filepath of the input data.The first column of
#' the input data should always include timestamps.
#' @param in_voltage set as TRUE only when the input Actigraph csv file is in
#'   analog quantized format and need to be converted into g value
#' @param header boolean. If TRUE, the input csv file will have column names in
#'   the first row.
#' @param has_ts boolean. If TRUE, the input csv file should have a timestamp
#'   column at first.
#' @param chunk_samples number. The number of samples in each chunk. Default is
#'   180000.
#' @return list. The list contains two items. The first item is a generator
#'   function that each time it is called, it will return a data.frame of the
#'   imported chunk. The second item is a \code{close} function which you can
#'   call at any moment to close the file loading.
#'
#' @family File I/O functions
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'
#'   # Use the actigraph csv file shipped with the package
#'   filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')
#'
#'   # Check original file format
#'   readLines(filepath)[1:15]
#'
#'   # Example 1: Load chunks every 2000 samples
#'   results = import_actigraph_csv_chunked(filepath, chunk_samples=2000)
#'   next_chunk = results[[1]]
#'   close_connection = results[[2]]
#'   # Check data as chunks, you can see chunks are shifted at each iteration.
#'   n = 1
#'   repeat {
#'     df = next_chunk()
#'     if (nrow(df) > 0) {
#'       print(paste('chunk', n))
#'       print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
#'       n = n + 1
#'     }
#'     else {
#'       break
#'     }
#'   }
#'
#'   # Close connection after reading all the data
#'   close_connection()
#'
#'   # Example 2: Close loading early
#'   results = import_actigraph_csv_chunked(filepath, chunk_samples=2000)
#'   next_chunk = results[[1]]
#'   close_connection = results[[2]]
#'   # Check data as chunks, you can see chunk time is shifting forward at each iteration.
#'   n = 1
#'   repeat {
#'     df = next_chunk()
#'     if (nrow(df) > 0) {
#'       print(paste('chunk', n))
#'       print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
#'       n = n + 1
#'       close_connection()
#'     }
#'     else {
#'       break
#'     }
#'   }
#'
#'   # Restore default options
#'   options(default_ops)
import_actigraph_csv_chunked <- function(filepath,
                                         in_voltage = FALSE,
                                         header = TRUE,
                                         has_ts = TRUE,
                                         chunk_samples = 180000) {
  chunk_size <- chunk_samples
  actigraph_meta <- import_actigraph_meta(filepath)

  if (has_ts) {
    ncols <- 4
    col_types <- c("character", "numeric", "numeric", "numeric")
    col_names <- c("HEADER_TIME_STAMP", "X", "Y", "Z")
  } else {
    ncols <- 3
    col_types <- c("numeric", "numeric", "numeric")
    col_names <- c("X", "Y", "Z")
  }
  if (header) {
    skip <- 11
  } else {
    skip <- 10
  }
  con <- file(filepath, "r")

  env <- new.env()
  env$df = NULL

  next_chunk <- function() {
    is_open <- tryCatch({
      isOpen(con)
    },
    error =
      function(cond) {
        return(FALSE)
      })
    if (!is_open) {
      return(data.frame())
    }
    dat <- utils::read.csv(
      con,
      header = FALSE,
      skip = skip,
      nrows = chunk_size,
      colClasses = col_types,
      col.names = col_names
    )
    if (!has_ts) {
      if (!is.null(env$df)) {
        st = env$df[nrow(env$df), 1]
      } else {
        st = actigraph_meta$st
      }
      dat <-
        .append_actigraph_timestamps(dat, actigraph_meta, st = st)
    } else {
      dat <- .convert_actigraph_timestamps(dat, actigraph_meta)
    }
    colnames(dat) <- c("HEADER_TIME_STAMP", "X", "Y", "Z")

    if (in_voltage) {
      dat <- .convert_voltage_to_g(dat, actigraph_meta)
    }

    env$df <- as.data.frame(dat)
    return(env$df)
  }

  close_connection <- function() {
    close(con)
  }
  return(list(next_chunk, close_connection))
}

.append_actigraph_timestamps <- function(dat, actigraph_meta, st = NULL) {
  if (is.null(st)) {
    st = actigraph_meta$st
  } else {
    st = st + 1 /  actigraph_meta$sr
  }
  ts_col <-
    seq(
      from = st,
      by = 1 / actigraph_meta$sr,
      length.out = nrow(dat)
    )
  dat <- cbind(ts_col, dat)
  return(dat)
}

.convert_actigraph_timestamps <- function(dat, actigraph_meta) {
  time_format <-
    ifelse(test = actigraph_meta$imu,
           yes = "%Y-%m-%dT%H:%M:%OS",
           no = "%m/%d/%Y %H:%M:%OS")
  dat[["HEADER_TIME_STAMP"]] <-
    strptime(x = dat[["HEADER_TIME_STAMP"]], format = time_format) + 5e-04
  return(dat)
}

.convert_voltage_to_g <- function(dat, actigraph_meta) {
  vs <- actigraph_meta$vs
  res <- actigraph_meta$res

  dat[, 2:ncol(dat)] <-
    (dat[, 2:ncol(dat)] * vs / (2 ^ res) - vs / 2) / (vs / res)
  dat[, 2:ncol(dat)] <-
    as.data.frame(apply(dat[, 2:ncol(dat)], 2, function(col) {
      col[col == -5] <- 0
      return(as.numeric(col))
    }))
  return(dat)
}

#' Import raw multi-channel accelerometer data stored in Actigraph raw csv
#' format
#'
#' \code{import_actigraph_csv} imports the raw multi-channel accelerometer data
#' stored in Actigraph raw csv format. It supports files from the following
#' devices: GT3X, GT3X+, GT3X+BT, GT9X, and GT9X-IMU.
#'
#' For old device (GT3X) that stores accelerometer values as digital voltage.
#' The function will convert the values to \eqn{g} unit using the following
#' equation.
#'
#' \deqn{x_g = \frac{x_{voltage}r}{(2 ^ r) - \frac{v}{2}}}
#'
#' Where \eqn{v} is the max voltage corresponding to the max accelerometer value
#' that can be found in the meta section in the csv file; \eqn{r} is the
#' resolution level which is the number of bits used to store the voltage
#' values. \eqn{r} can also be found in the meta section in the csv file.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import data from Actigraph devices during
#'   algorithm validation.
#'
#' @param filepath string. The filepath of the input data. The first column of
#' the input data should always include timestamps.
#' @param in_voltage set as TRUE only when the input Actigraph csv file is in
#'   analog quantized format and need to be converted into g value
#' @param header boolean. If TRUE, the input csv file will have column names in
#'   the first row.
#' @param has_ts boolean. If TRUE, the input csv file will have a timestamp column.
#' @return dataframe. The imported multi-channel accelerometer signal, with the
#'   first column being the timestamps in POSXlct format, and the rest columns
#'   being accelerometer values in \eqn{g} unit.
#'
#' @family File I/O functions
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'
#'   # Use the sample actigraph csv file provided by the package
#'   filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')
#'
#'   # Check file format
#'   readLines(filepath)[1:15]
#'
#'   # Load the file with timestamp column
#'   df = import_actigraph_csv(filepath)
#'
#'   # Check loaded file
#'   head(df)
#'
#'   # Check more
#'   summary(df)
#'
#'   # Use the sample actigraph csv file without timestamp
#'   filepath = system.file('extdata', 'actigraph_no_timestamp.csv', package='MIMSunit')
#'
#'   # Check file format
#'   readLines(filepath)[1:15]
#'
#'   # Load the file without timestamp column
#'   df = import_actigraph_csv(filepath, has_ts = FALSE)
#'
#'   # Check loaded file
#'   head(df)
#'
#'   # Check more
#'   summary(df)
#'
#'   # Restore default options
#'   options(default_ops)
import_actigraph_csv <-
  function(filepath,
           in_voltage = FALSE,
           has_ts = TRUE,
           header = TRUE) {
    actigraph_meta <- import_actigraph_meta(filepath)
    if (has_ts) {
      ncols <- 4
      col_types <- paste(c("c", rep("d", ncols - 1)), collapse = "")
    } else {
      ncols <- 3
      col_types <- paste(rep("d", ncols), collapse = "")
    }

    if (header) {
      dat <-
        readr::read_csv(
          filepath,
          col_names = FALSE,
          skip = 11,
          trim_ws = TRUE,
          col_types = col_types
        )
    } else {
      dat <-
        readr::read_csv(
          filepath,
          col_names = FALSE,
          skip = 10,
          trim_ws = TRUE,
          col_types = col_types
        )
    }

    if (has_ts && ncol(dat) == 3) {
      stop(
        "The input data only has 3 column
s, there should be 4 columns with the first column being the timestamps."
      )
    }

    if (!has_ts) {
      dat = .append_actigraph_timestamps(dat = dat,
                                         actigraph_meta = actigraph_meta)
    }

    dat <- dat[, 1:4]

    names(dat) <- c("HEADER_TIME_STAMP", "X", "Y", "Z")

    if (has_ts) {
      time_format <-
        ifelse(test = actigraph_meta$imu,
               yes = "%Y-%m-%dT%H:%M:%OS",
               no = "%m/%d/%Y %H:%M:%OS")
      dat[["HEADER_TIME_STAMP"]] <-
        strptime(x = dat[["HEADER_TIME_STAMP"]], format = time_format) + 5e-04
    }

    dat <- as.data.frame(dat)

    if (in_voltage) {
      vs <- actigraph_meta$vs
      res <- actigraph_meta$res

      dat[, 2:ncol(dat)] <-
        (dat[, 2:ncol(dat)] * vs / (2 ^ res) - vs / 2) / (vs / res)
      dat[, 2:ncol(dat)] <-
        as.data.frame(apply(dat[, 2:ncol(dat)], 2, function(col) {
          col[col == -5] <- 0
          return(as.numeric(col))
        }))
    }
    return(dat)
  }

#' Import Actigraph count data stored in Actigraph summary csv format
#'
#' \code{import_actigraph_count_csv} imports Actigraph count data stored in
#' Actigraph summary csv format, which was exported by Actilife.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import Actigraph count data from Actigraph devices
#'   during algorithm validation.
#'
#' @note If both \code{count_col} and \code{count_per_axis_cols} are
#'   \code{NULL}, the function will raise an error.
#'
#' @param filepath string. The filepath of the input data.
#' @param count_col number. The index of column of Actigraph count (combined
#'   axes). If it is \code{NULL}, the function will use
#'   \code{count_per_axis_cols} to get the combined Actigraph count values.
#' @param count_per_axis_cols numerical vector. The indices of columns of
#'   Actigraph count values per axis. If \code{count_col} is not \code{NULL},
#'   the argument will be ignored. If it is \code{NULL}, the output dataframe
#'   will only have two columns without Actigraph count values per axis.
#' @return dataframe. The imported actigraph count data, with the first column
#'   being the timestamps in POSIXct format, and the second column being the
#'   combined Actigraph count values, and the rest of columns being the
#'   Actigraph cont values per axis if available. Column names:
#'   \code{HEADER_TIME_STAMP}, \code{ACTIGRAPH_COUNT},
#'   \code{ACTIGRAPH_COUNT_X}....
#'
#' @family File I/O functions
#' @export
#' @examples
#'   # Use the actigraph count csv file shipped with the package
#'   filepath = system.file('extdata', 'actigraph_count.csv', package='MIMSunit')
#'
#'   # Check original data format
#'   readLines(filepath)[1:5]
#'
#'   # Load file, default column for actigraph count values are 2, this file does not have
#'   # axial count values
#'   output = import_actigraph_count_csv(filepath, count_col=2)
#'
#'   # Check output
#'   head(output)
import_actigraph_count_csv <-
  function(filepath,
           count_col = 2,
           count_per_axis_cols = c(2, 3, 4)) {
    dat <-
      utils::read.csv(file = filepath,
                      header = TRUE,
                      stringsAsFactors = FALSE)
    dat[, 1] <-
      as.POSIXct(dat[, 1],
                 format = "%Y-%m-%d %H:%M:%OS",
                 tz = "UTC")

    if (!is.null(count_col)) {
      result <- dat[, c(1, count_col)]
      colnames(result) = c("HEADER_TIME_STAMP", "ACTIGRAPH_COUNT")
    } else if (is.null(count_col) & !is.null(count_per_axis_cols)) {
      result <-
        cbind(dat[, 1], sqrt(rowSums(dat[, count_per_axis_cols] ^ 2)), dat[, count_per_axis_cols])
      colnames(result) = c(
        "HEADER_TIME_STAMP",
        "ACTIGRAPH_COUNT",
        "ACTIGRAPH_COUNT_X",
        "ACTIGRAPH_COUNT_Y",
        "ACTIGRAPH_COUNT_Z"
      )
    } else {
      stop("You must set count_col or count_per_axis_cols")
    }
    return(result)
  }

#' Import ENMO data stored in csv csv
#'
#' \code{import_enmo_csv} imports ENMO data stored in a summary csv format,
#' which was exported by the
#' \href{https://github.com/OxWearables/biobankAccelerometerAnalysis}{biobank
#' data analysis tools}.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to import ENMO data from activity monitor devices
#'   during algorithm validation.
#'
#' @param filepath string. The filepath of the input data.
#' @param enmo_col number. The index of column of ENMO values in the csv file.
#' @return dataframe. The imported ENMO data, with the first column being the
#'   timestamps in POSIXct format, and the second column being the ENMO values.
#'   Column names: \code{HEADER_TIME_STAMP}, \code{ENMO}.
#'
#' @family File I/O functions
#' @export
#' @examples
#'   # Use the enmo csv file shipped with the package
#'   filepath = system.file('extdata', 'enmo.csv', package='MIMSunit')
#'
#'   # Check original data format
#'   readLines(filepath)[1:5]
#'
#'   # Load file, default column for enmo values are 2
#'   output = import_enmo_csv(filepath, enmo_col=2)
#'
#'   # Check output
#'   head(output)
import_enmo_csv <- function(filepath, enmo_col = 2) {
  dat <- readr::read_csv(filepath, col_names = TRUE)
  dat <- data.frame(dat)
  dat <- dat[, c(1, enmo_col)]
  dat[, 1] <-
    as.POSIXct(dat[, 1],
               format = "%Y-%m-%d %H:%M:%OS",
               tz = Sys.timezone())
  result <- dat
  colnames(result) <- c("HEADER_TIME_STAMP", "ENMO")
  return(result)
}

#' Import The meta information stored in Actigraph RAW or summary csv file.
#'
#' \code{import_actigraph_meta} imports meta information stored in the Actigraph
#' summary csv file.
#'
#' The returned meta information includes following fields.
#'
#' \itemize{ \item sr: Sampling rate in Hz. \item fw: Firmware version. For
#' example "1.7.0". \item sw: Software version of Actilife. For example "6.13.0".
#' \item sn: Serial number of the device. \item st: Start time of the data, in
#' POSIXct format. \item dt: Download time of the data, in POSIXct format. \item
#' at: Type of the device. Could be "MAT","CLE", "MOS" or "TAS", corresponding
#' to different Actigraph devices. \item imu: Whether the file is about
#' Actigraph GT9X IMU data. \item gr: The dynamic range in \eqn{g} unit. \item
#' vs: The voltage level of the device, may be used in AD conversion. See
#' \code{\link{import_actigraph_csv}}. \item res: The resolution or the number of
#' bits used to store quantized voltage values of the device, may be used in AD
#' conversion. See \code{\link{import_actigraph_csv}}. }
#'
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a File IO
#'   function that is used to get related meta information such as sampling
#'   rate, firmware version from Actigraph devices.
#'
#' @param filepath string. The filepath of the input data.
#' @param header logical. Whether the Actigraph RAW or summary csv file includes
#'   column names. Default is TRUE.
#' @return list. A list of Actigraph device meta information.
#' @family File I/O functions
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'
#'   # Use the sample actigraph csv file provided by the package
#'   filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')
#'
#'   # Check file format
#'   readLines(filepath)[1:15]
#'
#'   # Load the meta headers of input file
#'   import_actigraph_meta(filepath, header=TRUE)
#'
#'   # Restore default options
#'   options(default_ops)
import_actigraph_meta <- function(filepath, header = TRUE) {
  ACTIGRAPH_HEADER_SR_PATTERN <- "([0-9]+) Hz"
  ACTIGRAPH_FIRMWARE_PATTERN <- "Firmware v([0-9]+.[0-9]+.[0-9]+)"
  ACTIGRAPH_SOFTWARE_PATTERN <- "ActiLife v([0-9]+.[0-9]+.[0-9]+)"
  ACTIGRAPH_SERIALNUM_PATTERN <- "Serial Number: ([A-Za-z0-9]+)"
  ACTIGRAPH_TIMESTAMP <- "%m/%d/%Y %H:%M:%OS"

  headlines <- readLines(filepath, n = 10, encoding = "UTF-8")

  # Sampling rate
  sr_pattern <- ACTIGRAPH_HEADER_SR_PATTERN
  sr <- headlines[[1]]
  sr <- stringr::str_match(sr, sr_pattern)
  sr <- as.numeric(sr[2])

  # Firmware code
  fw_pattern <- ACTIGRAPH_FIRMWARE_PATTERN
  fw <- headlines[[1]]
  fw <- stringr::str_match(fw, fw_pattern)
  fw <- fw[2]

  # Software code
  sw_pattern <- ACTIGRAPH_SOFTWARE_PATTERN
  sw <- headlines[[1]]
  sw <- stringr::str_match(sw, sw_pattern)
  sw <- sw[2]

  # Serial number
  sn_pattern <- ACTIGRAPH_SERIALNUM_PATTERN
  sn <- headlines[[2]]
  sn <- stringr::str_match(sn, sn_pattern)
  sn <- sn[2]

  # actigraph type
  at <- substr(sn, 1, 3)

  # g range
  gr <- switch(
    at,
    MAT = "3",
    CLE = "6",
    MOS = "8",
    TAS = "8"
  )

  # IMU or not
  if (stringr::str_detect(headlines[[1]], "IMU")) {
    imu <- TRUE
  } else {
    imu <- FALSE
  }

  if (imu) {
    gr <- "16"
  }

  # Session start time
  st <- headlines[[3]]
  sd <- headlines[[4]]
  time_reg <- "[0-9]{2}(:[0-9]{2}){1,2}+"
  date_reg <- "[0-9]+/[0-9]+/[0-9]{4}"
  st <- regmatches(st, regexpr(time_reg, st, perl = TRUE))
  sd <- regmatches(sd, regexpr(date_reg, sd, perl = TRUE))
  st <- paste(sd, st, sep = " ")
  time_format <- ACTIGRAPH_TIMESTAMP
  st <- strptime(st, time_format) + 5e-04

  # Session download time
  dt <- headlines[[6]]
  dd <- headlines[[7]]
  time_reg <- "[0-9]{2}(:[0-9]{2}){1,2}+"
  date_reg <- "[0-9]+/[0-9]+/[0-9]{4}"
  dt <- regmatches(dt, regexpr(time_reg, dt, perl = TRUE))
  dd <- regmatches(dd, regexpr(date_reg, dd, perl = TRUE))
  dt <- paste(dd, dt, sep = " ")
  time_format <- ACTIGRAPH_TIMESTAMP
  dt <- strptime(dt, time_format) + 5e-04
  if (is.na(sr)) {
    # determine sr by start and download time options(digits = 13)
    duration <- as.numeric(dt - st, units = "secs")
    if (header) {
      nlines <- R.utils::countLines(filepath) - 11
    } else {
      nlines <- R.utils::countLines(filepath) - 10
    }
    sr <- as.numeric(ceiling(nlines / duration))
  }

  # input voltage
  vs <- headlines[[9]]
  vs_reg <- ": ([0-9](\\.[0-9]+)*)"
  vs <- as.numeric(stringr::str_match(vs, vs_reg)[2])

  # input resolution
  resolution <- headlines[[9]]
  res_reg <- "= ([0-9]+)"
  resolution <-
    as.numeric(stringr::str_match(resolution, res_reg)[2])

  # header object as output
  meta <- {

  }
  meta$sr <- sr
  meta$fw <- fw
  meta$sw <- sw
  meta$sn <- sn
  meta$st <- st
  meta$dt <- dt
  meta$at <- at
  meta$imu <- imu
  meta$gr <- gr
  meta$vs <- vs
  meta$res <- resolution

  return(meta)
}

test_that("import_mhealth_csv matches legacy readr parsing for package examples", {
  for (filename in c("mhealth.csv", "mhealth1.csv")) {
    filepath <- system.file("extdata", filename, package = "MIMSunit")
    coltypes <- list(
      readr::col_datetime(format = "%Y-%m-%d %H:%M:%OS"),
      readr::col_double(),
      readr::col_double(),
      readr::col_double()
    )
    expected <- readr::with_edition(
      1,
      readr::read_csv(file = filepath,
                      quoted_na = TRUE,
                      col_types = coltypes)
    )
    colnames(expected) <- c("HEADER_TIME_STAMP", "X", "Y", "Z")
    expected <- data.frame(expected, stringsAsFactors = FALSE)

    result <- import_mhealth_csv(filepath)

    expect_equal(dim(result), c(480L, 4L))
    expect_equal(colnames(result), c("HEADER_TIME_STAMP", "X", "Y", "Z"))
    expect_s3_class(result$HEADER_TIME_STAMP, "POSIXct")
    expect_true(all(vapply(result[c("X", "Y", "Z")], is.numeric, logical(1))))
    expect_equal(result, expected)
  }
})

test_that("import_mhealth_csv preserves quoted NA behavior", {
  filepath <- tempfile(fileext = ".csv")
  writeLines(
    c(
      "HEADER_TIME_STAMP,VALUE",
      "2017-03-16 12:25:50.000,\"NA\"",
      "2017-03-16 12:25:50.013,NA",
      "2017-03-16 12:25:50.025,1.5"
    ),
    filepath
  )

  expect_silent(result <- import_mhealth_csv(filepath))

  expect_equal(colnames(result), c("HEADER_TIME_STAMP", "VALUE"))
  expect_s3_class(result$HEADER_TIME_STAMP, "POSIXct")
  expect_true(is.numeric(result$VALUE))
  expect_true(is.na(result$VALUE[1]))
  expect_true(is.na(result$VALUE[2]))
  expect_equal(result$VALUE[3], 1.5)
})

#' Export accelerometer data in Actilife RAW CSV format
#'
#' \code{export_to_actilife} exports the input dataframe as a csv file that is
#' compatible with Actilife.
#'
#' This function takes an input accelerometer dataframe and exports it in
#' Actilife RAW CSV format with a prepended a madeup header. The exported file
#' csv file has compatible header, column names, timestamp format with Actilife
#' and can be imported directly into Actilife software.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is an utility
#' function that was used to convert validation data into Actilife RAW CSV
#' format so that we can use Actilife to compute Actigraph counts values for
#' these data.
#'
#' @param df dataframe. Input accelerometer data. The first column is timestamp
#'   in POSXlct format, and the rest columns are accelerometer values in g
#'   (\eqn{9.81m/s^2}).
#' @param filepath string. The output filepath.
#' @param actilife_version string. The Actilife version number to be added to
#'   the header. Default is "6.13.3", that was used by the algorithm during
#'   development.
#' @param firmware_version string. The firmware version number to be added to
#'   the header. This is supposed to be the firmware version of the Actigraph
#'   devices. We did not see any usage of the number during the computation of
#'   Actigraph counts by Actilife, so it may be set with an arbitrary version
#'   code seen in any Actigraph devices. We use default version code "1.6.0".
#' @return No return value.
#' @family File I/O functions
#' @export
#' @examples
#'   # Use the first 5 rows from sample data
#'   df = sample_raw_accel_data[1:5,]
#'   head(df)
#'
#'   # Save to current path with default mocked actilife and firmware versions
#'   filepath = tempfile()
#'   export_to_actilife(df, filepath)
#'
#'   # The saved file will have the same format as Actigraph csv files
#'   readLines(filepath)
#'
#'   # Cleanup
#'   file.remove(filepath)
export_to_actilife <-
  function(df,
           filepath,
           actilife_version = "6.13.3",
           firmware_version = "1.6.0") {
    sr <- sampling_rate(df)
    start_time <-
      strftime(df[1, 1], format = "%H:%M:%S", tz = "UTC")
    start_date <-
      strftime(df[1, 1], format = "%m/%d/%Y", tz = "UTC")
    stop_time <-
      strftime(df[nrow(df), 1], format = "%H:%M:%S", tz = "UTC")
    stop_date <-
      strftime(df[nrow(df), 1], format = "%m/%d/%Y", tz = "UTC")
    actigraph_meta <-
      c(
        paste0(
          "------------ Data File Created By Actigraph GT3X+ ActiLife v",
          actilife_version,
          " Firmware v",
          firmware_version,
          " date format M/d/yyyy at ",
          sr,
          " Hz  Filter Nomral -----------"
        )
      )
    actigraph_meta <-
      c(actigraph_meta, "Serial Number: TAS1E00000000")
    actigraph_meta <-
      c(actigraph_meta, paste0("Start Time ", start_time))
    actigraph_meta <-
      c(actigraph_meta, paste0("Start Date ", start_date))
    actigraph_meta <-
      c(actigraph_meta, paste0("Epoch Period (hh:mm:ss) 00:00:00"))
    actigraph_meta <-
      c(actigraph_meta, paste0("Download Time ", stop_time))
    actigraph_meta <-
      c(actigraph_meta, paste0("Download Date ", stop_date))
    actigraph_meta <-
      c(actigraph_meta, paste0("Current Memory Address 0"))
    actigraph_meta <-
      c(
        actigraph_meta,
        paste0("Current Battery Voltage: 4.19     Mode = 12")
      )
    actigraph_meta <-
      c(
        actigraph_meta,
        paste0("--------------------------------------------------")
      )
    actigraph_meta <-
      c(
        actigraph_meta,
        paste0("Timestamp,Accelerometer X,Accelerometer Y,Accelerometer Z")
      )
    df[[1]] <-
      strftime(df[[1]], format = "%m/%d/%Y %H:%M:%OS3", tz = "UTC")
    df[c(2, 3, 4)] <- round(df[c(2, 3, 4)], digits = 3)
    readr::write_lines(x = actigraph_meta, file = filepath, append = FALSE)
    readr::write_csv(
      x = df,
      file = filepath,
      col_names = FALSE,
      append = TRUE
    )
  }

#' Parse epoch string to the corresponding number of samples it represents.
#'
#' \code{parse_epoch_string} parses the epoch string (e.g. "1 min"), and outputs
#' the corresponding number of samples it represents.
#'
#' This function parses the given epoch string (e.g. "5 secs") and outputs the
#' corresponding number of samples represented by the epoch string.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is used in
#'   \code{\link{aggregate_for_mims}} function and \code{\link{mims_unit}} function.
#'
#' @param epoch_str string. The input epoch str as accepted by \code{breaks}
#'   argument of \code{\link[base]{cut.POSIXt}}.
#' @param sr number. The sampling rate in Hz used to parse the epoch string.
#' @return number. The number of samples represented by the epoch string.
#' @family utility functions
#' @export
#' @examples
#' # 1 min with 80 Hz = 4800 samples
#' parse_epoch_string('1 min', sr=80)
#'
#' # 30 sec with 30 Hz = 900 samples
#' parse_epoch_string('30 sec', sr=30)
#'
#' # 1 hour with 1 Hz = 3600 samples
#' parse_epoch_string('1 hour', sr=1)
#'
#' # 1 day with 10 Hz = 864000 samples
#' parse_epoch_string('1 day', sr=10)
parse_epoch_string <- function(epoch_str, sr) {
  if (stringr::str_detect(epoch_str, "sec")) {
    tokens <- stringr::str_split(epoch_str, pattern = " ")[[1]]
    n <- as.numeric(tokens[1]) * sr
  } else if (stringr::str_detect(epoch_str, "min")) {
    tokens <- stringr::str_split(epoch_str, pattern = " ")[[1]]
    n <- as.numeric(tokens[1]) * sr * 60
  } else if (stringr::str_detect(epoch_str, "hour")) {
    tokens <- stringr::str_split(epoch_str, pattern = " ")[[1]]
    n <- as.numeric(tokens[1]) * sr * 3600
  } else if (stringr::str_detect(epoch_str, "day")) {
    tokens <- stringr::str_split(epoch_str, pattern = " ")[[1]]
    n <- as.numeric(tokens[1]) * sr * 3600 * 24
  }
  return(n)
}

#' Estimate sampling rate for multi-channel signal
#'
#' \code{sampling_rate} estimates the sampling rate based on the average time
#' intervals between adjacent samples for the input multi-channel signal.
#'
#' This function accepts a dataframe of multi-channel signal, computes the
#' duration of the sequence, and gets the sampling rate by dividing the number
#' of samples by it.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that was used in various part in the algorithm whenever we need to
#'   know the sampling rate.
#'
#' @param df dataframe. Input dataframe of the multi-channel signal. The first
#'   column is the timestamps in POSXlct format and the following columns are
#'   accelerometer values.
#' @return number. The estimated sampling rate in Hz.
#' @family utility functions
#' @export
#' @examples
#' # Get the test data
#' df = sample_raw_accel_data
#'
#' # Default sampling rate is 80Hz
#' sampling_rate(df)
#'
#' # Downsample to 30Hz
#' output = bandlimited_interp(df, 80, 30)
#' sampling_rate(output)
#'
#' # Upsampling to 100Hz
#' output = bandlimited_interp(df, 80, 100)
#' sampling_rate(output)
sampling_rate <- function(df) {
  duration <-
    as.numeric(dplyr::last(df[, 1]) - dplyr::first(df[, 1]), units = "secs")
  sr <- round(nrow(df) / duration / 10) * 10
  return(sr)
}

#' Clip dataframe to the given start and stop time
#'
#' \code{clip_data} clips the input sensor dataframe according to the given
#' start and stop time
#'
#' This function accepts a dataframe of multi-channel signal, clips it
#' according to the start_time and stop_time.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that was used in various part in the algorithm whenever we need to
#'   clip a dataframe.
#'
#' @param df dataframe. Input dataframe of the multi-channel signal. The first
#'   column is the timestamps in POSXlct format and the following columns are
#'   accelerometer values.
#' @param start_time POSXlct format or character. Start time for clipping.
#' If it is a character, it should be recognizable by as.POSXlct function.
#' @param stop_time POSXlct format or character. Stop time for clipping.
#' If it is a character, it should be recognizable by as.POSXlct function.
#' @return dataframe. The same format as the input dataframe.
#' @family utility functions
#' @export
#' @examples
#'   default_ops = options()
#'   options(digits.secs=3)
#'   # Use the provided sample data
#'   df = sample_raw_accel_data
#'
#'   # Check the start time and stop time of the dataset
#'   summary(df)
#'
#'   # Use timestamp string to clip 1 second data
#'   start_time = "2016-01-15 11:01:00"
#'   stop_time = "2016-01-15 11:01:01"
#'   output = clip_data(df, start_time, stop_time)
#'   summary(output)
#'
#'   # Use POSIXct timestamp to clip data
#'   start_time = as.POSIXct("2016-01-15 11:01:00")
#'   stop_time = as.POSIXct("2016-01-15 11:01:01")
#'   output = clip_data(df, start_time, stop_time)
#'   summary(output)
#'
#'   # If start and stop time is not in the range of the input data
#'   # return empty data.frame
#'   start_time = "2016-01-15 12:01:00"
#'   stop_time = "2016-01-15 12:01:01"
#'   output = clip_data(df, start_time, stop_time)
#'   output
#'
#'   # Restore original options
#'   options(default_ops)
clip_data <- function(df, start_time, stop_time) {
  df = as.data.frame(df)
  tzone <- lubridate::tz(df[[1, 1]])
  if (is.character(start_time)) {
    start_time <- as.POSIXct(start_time, tz = tzone)
  }
  start_time <- lubridate::force_tz(start_time, tzone = tzone)
  if (is.character(stop_time)) {
    stop_time <- as.POSIXct(stop_time, tz = tzone)
  }
  stop_time <- lubridate::force_tz(stop_time, tzone = tzone)

  mask <- df[, 1] >= start_time &
    df[, 1] <= stop_time
  sub_df <- df[mask, ]

  return(sub_df)
}

#' Segment input dataframe into windows as specified by breaks.
#' \code{segment_data} segments the input sensor dataframe into
#'   epoch windows with length specified in breaks.
#'
#'   This function accepts a dataframe of multi-channel signal, segments it
#'   into epoch windows with length specified in breaks.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that was used in various part in the algorithm whenever we need to
#'   segment a dataframe, e.g., before aggregating values over epoch windows.
#'
#' @param df dataframe. Input dataframe of the multi-channel signal. The first
#'   column is the timestamps in POSXlct format and the following columns are
#'   accelerometer values.
#' @param breaks character. An epoch length character that can be accepted by
#'   cut.breaks function.
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the breaks generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate breaks. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#'
#' @return dataframe. The same format as the input dataframe, but with an extra
#'   column "SEGMENT" in the end specifies the epoch window a sample belongs to.
#' @family utility functions
#' @export
#' @examples
#'   # Use sample data
#'   df = sample_raw_accel_data
#'
#'   # segment data into 1 minute segments
#'   output = segment_data(df, "1 min")
#'
#'   # check the 3rd segment, each segment would have 1 minute data
#'   summary(output[output['SEGMENT'] == 3,])
#'
#'   # segment data into 15 second segments
#'   output = segment_data(df, "15 sec")
#'
#'   # check the 1st segment, each segment would have 15 second data
#'   summary(output[output['SEGMENT'] == 1,])
#'
#'   # segment data into 1 hour segments
#'   output = segment_data(df, "1 hour")
#'
#'   # because the input data has only 15 minute data
#'   # there will be only 1 segment in the output
#'   unique(output['SEGMENT'])
#'   summary(output)
#'
#'   # use manually set start time
#'   output = segment_data(df, "15 sec", st='2016-01-15 10:59:50.000')
#'
#'   # check the 1st segment, because the start time is 10 seconds before the
#'   # start time of the actual data, the first segment will only include 5 second
#'   # data.
#'   summary(output[output['SEGMENT'] == 1,])
segment_data <- function(df, breaks, st=NULL) {
  ts <- df[["HEADER_TIME_STAMP"]]
  tzone <- lubridate::tz(ts[1])
  if (!is.null(st)) {
    if (is.character(st)) {
      st <- as.POSIXct(st, tz = tzone)
    }
    ts = append(st, ts)
  }
  if (missing(breaks) || is.null(breaks)) {
    segments <- ts[1]
  } else {
    ts[1] <- .segment.floor_date(ts[1], breaks)
    segments <- cut(ts, breaks = breaks)
  }
  segments <- as.numeric(segments)
  if (!is.null(st)) {
    df["SEGMENT"] <- segments[2:length(segments)]
  } else {
    df["SEGMENT"] <- segments
  }
  return(df)
}

.segment.floor_date <- function(ts, breaks) {
  if (stringr::str_detect(breaks, "sec")) {
    ts <- lubridate::floor_date(ts, unit = c("second"))
  } else if (stringr::str_detect(breaks, "min")) {
    ts <- lubridate::floor_date(ts, unit = c("minute"))
  } else if (stringr::str_detect(breaks, "hour")) {
    ts <- lubridate::floor_date(ts, unit = c("hour"))
  } else if (stringr::str_detect(breaks, "day")) {
    ts <- lubridate::floor_date(ts, unit = c("day"))
  }
  return(ts)
}

.segment.ceil_date <- function(ts, breaks) {
  if (stringr::str_detect(breaks, "sec")) {
    ts <- lubridate::ceiling_date(ts, unit = c("second"))
  } else if (stringr::str_detect(breaks, "min")) {
    ts <- lubridate::ceiling_date(ts, unit = c("minute"))
  } else if (stringr::str_detect(breaks, "hour")) {
    ts <- lubridate::ceiling_date(ts, unit = c("hour"))
  } else if (stringr::str_detect(breaks, "day")) {
    ts <- lubridate::ceiling_date(ts, unit = c("day"))
  }
  return(ts)
}

test_that("parse_epoch_string works", {
  n1 <- parse_epoch_string("1 min", sr = 50)
  expect_equal(n1, 3000)
  n2 <- parse_epoch_string("5 secs", sr = 10)
  expect_equal(n2, 50)
  n3 <- parse_epoch_string("2 hours", sr = 10)
  expect_equal(n3, 72000)
})

#' Run shiny app to compute MIMSunit values from files
#'
#' \code{shiny_app} runs a local shiny app that provides a user friendly interface
#' to compute mims unit values from files stored in mhealth or actigraph format.
#'
#' @section How is it used in MIMS-unit algorithm?: This provides a user friendly
#' graphical interface to load local files, call \code{\link{mims_unit_from_files}}
#' and display the results as an interactive graph.
#'
#' @param options The options passed to \code{\link[shiny]{shinyApp}}.
#'
#' @family Top level API functions
#' @export
#' @examples
#'   \dontrun{
#'   shiny_app()
#'   }
shiny_app <- function(options = list()) {
  shiny::shinyAppDir(appDir = system.file("app", package='MIMSunit'), options = options)
}

#' Aggregate over epoch to get numerically integrated values.
#'
#' \code{aggregate_for_mims} returns a dataframe with integrated values by
#' trapzoidal method over each epoch for each column. The epoch start time will
#' be used as timestamp in the first column.
#'
#' This function accepts a dataframe (in mhealth accelerometer data format) and
#' computes its aggregated values over each fixed epoch using different
#' integration methods (default is trapzoidal method, other methods are not used
#' by mims unit algorithm) for each value columns. The returned dataframe will
#' have the same number of columns as input dataframe, and have the same
#' datetime format as input dataframe in the timestamp column. The trapzoidal
#' method used in the function is based on \code{\link[caTools]{trapz}}.
#'
#' @section How is it used in mims-unit algorithm?: This function is used in
#'   mims-unit algorithm after filtering (\code{\link{iir}}). The filtered
#'   signal will be rectified and integrated to get mims unit values for each
#'   axis using this function.
#'
#' @param df dataframe of accelerometer data in mhealth format. First column
#'   should be timestamps in POSIXt format.
#' @param epoch string. Any format that is acceptable by argument \code{breaks}
#'   in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
#'   secs", "10 mins".
#' @param method string. Integration methods. Supported strings include:
#'   "trapz", "power", "sum", "meanBySecond", "meanBySize". Default is "trapz".
#' @param rectify logical. If TRUE, input data will be rectified before
#'   integration. Default is TRUE.
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the epochs generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate epochs. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#' @return dataframe. The returned dataframe will have the same format as input
#'   dataframe.
#'
#' @family aggregate functions
#' @seealso \code{\link{aggregate_for_orientation}} for aggregating to get
#'   accelerometer orientation estimation for each epoch.
#'
#' @note If \code{epoch} argument is not provided or is \code{NULL}, the
#'   function will treat the input dataframe as a single epoch.
#'
#'   If the number of samples in one segment is less than 90% of the expected
#'   samples, the aggregation result will be -1 (marker of invalid value).
#' @export
#'
#' @examples
#'   # sample data
#'   df = sample_raw_accel_data
#'   head(df)
#'
#'   # epoch set to 5 seconds, and method set to "trapz"
#'   aggregate_for_mims(df, epoch = '5 sec', method='trapz')
#'
#'   # epoch set to 1 second, method set to "sum"
#'   aggregate_for_mims(df, epoch = '1 sec', method='sum')
#'
#'   # epoch set to 1 second, and st set to be 1 second before the start time of the data
#'   # so the first segment will only include data for 1 second, therefore the resulted
#'   # aggregated value for the first segment will be -1 (invalid) because the
#'   # samples are not enough. And the second segment starts from 11:00:01, instead
#'   # of 11:00:02 as shown in prior example,
#'   aggregate_for_mims(df, epoch = '1 sec', method='sum', st=df[1,1] - 1)
aggregate_for_mims <-
  function(df,
           epoch,
           method = c("trapz", "power", "mean_by_time", "mean_by_size", "sum"),
           rectify = TRUE, st=NULL) {
    method = match.arg(method)
    time_zone <- lubridate::tz(df[[1, 1]])
    n_cols <- ncol(df)

    # parse input argument epoch
    if (missing(epoch) || is.null(epoch)) {
      df$SEGMENT <- 1
    } else {
      df <- MIMSunit::segment_data(df, breaks = epoch, st = st)
    }

    # get the number of samples in each epoch
    n_threshold <-
      parse_epoch_string(epoch_str = epoch, sr = sampling_rate(df))

    # iterate over each value column
    result <- plyr::ddply(df, c("SEGMENT"), function(rows) {
      r1 = rows[1,1]
      r1 = .segment.floor_date(r1, breaks = epoch)
      r1 = as.numeric(r1)
      rows[, 1] <- as.numeric(rows[, 1])
      rows <- stats::na.omit(rows)

      # do integration if there are enough (90%) number of samples
      if (nrow(rows) >= 0.9 * n_threshold) {
        # do rectification if rectify is TRUE
        if (rectify) {
          for (icol in 2:n_cols) {
            col_data = rows[[icol]]
            turn_abs = col_data > -150
            col_data[turn_abs] <- abs(col_data[turn_abs])
            rm(turn_abs)
            if (any(col_data < 0)) {
              col_data <- rep(-200, length(col_data))
            }
            rows[[icol]] = col_data
          }
          rm(col_data)
          # rows[2:n_cols] <- (plyr::numcolwise(function(col_data) {
          #   turn_abs = col_data > -150
          #   col_data[turn_abs] <- abs(col_data[turn_abs])
          #   if (any(col_data < 0)) {
          #     col_data <- rep(-200, length(col_data))
          #   }
          #   return(col_data)
          # }))(rows[2:n_cols])
        }
        trapz_mat = function(x, y) {
          idx = 2:length(x)
          x = (x[idx] - x[idx - 1])
          cny = colnames(y)
          y = as.matrix(y)
          y = (y[idx,] + y[idx - 1,])
          out = as.double(x %*% y)/2
          out = matrix(out, ncol = ncol(y))
          colnames(out) = cny
          out = data.frame(out)
          return(out)
        }
        # select different methods for integration
        if (method == "trapz") {
          # auc_values <-
          #   (plyr::numcolwise(caTools::trapz, x = rows[, 1]))(rows[2:n_cols])
          auc_values <- trapz_mat(rows[,1,drop = TRUE], rows[2:n_cols])
          max_values <- 16 * n_threshold
        } else if (method == "power") {
          # auc_values <-
          #   (plyr::numcolwise(caTools::trapz,
          #     x = rows[, 1]
          #   ))(as.data.frame(rows[2:n_cols]^2))
          auc_values <- trapz_mat(rows[,1,drop = TRUE], rows[2:n_cols]^2)
          max_values <- 16^2 * n_threshold
        } else if (method == "mean_by_time") {
          r = range(rows[,1])
          auc_values <-
            colSums(rows[2:n_cols]) /
              (r[2] - r[1])
          max_values <- 16 * n_threshold / 32
        } else if (method == "mean_by_size") {
          auc_values <-
            colSums(rows[2:n_cols]) / length(rows[, 1])
          max_values <- 16
        } else if (method == "sum") {
          auc_values <- colSums(rows[2:n_cols])
          max_values <- 16 * nrow(rows)
        }
      } else {
        auc_values <- as.data.frame(lapply(rows, function(x) {
          rep.int(-1, 1)
        }))
        auc_values <- auc_values[2:n_cols]
        max_values <- 0
      }

      # flag extra large (abnormal) values
      auc_values[auc_values >= max_values] <- -1
      auc_values[auc_values < 0] <- -1
      return(data.frame(ts = r1,
                        auc_values))
    })

    # format output
    result$SEGMENT <- NULL
    names(result)[1] <- names(df)[1]
    for (i in 2:n_cols)
    {
      names(result)[i] <- paste("AGGREGATED", names(result)[i], sep = "_")
    }
    result[1] <-
      as.POSIXct(result[[1]], origin = "1970-01-01", tz = time_zone)
    return(result)
  }

#' Aggregate over epoch to get estimated accelerometer orientation.
#'
#' \code{aggregate_for_orientation} returns a dataframe with accelerometer
#' orientations estimated by
#' \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#' 2003} over each epoch (see \code{\link{compute_orientation}}). The epoch
#' start time will be used as timestamp in the first column.
#'
#' This function accepts a dataframe (in mhealth accelerometer data format) and
#' computes the estimated acclerometer orientations (in x, y, and z angles) over
#' each fixed epoch. The returned dataframe will have the same format as input
#' dataframe, including four columns, and have the same datetime format as input
#' dataframe in the timestamp column. The orientation estimation method used in
#' the function is based on
#' \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#' 2003}.
#'
#' @section How is it used in mims-unit algorithm?: This function is used in
#'   mims-unit algorithm after extrapolation (\code{\link{extrapolate}}). The
#'   extrapolated signal will be estimated to get orientation angles using this
#'   function.
#'
#' @param df dataframe. Input accelerometer data in mhealth format. First column
#'   should be timestamps in POSIXt format.
#' @param epoch string. Any format that is acceptable by argument \code{breaks}
#'   in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
#'   secs", "10 mins".
#' @param estimation_window number. Duration in seconds to be used to estimate
#'   orientation within each epoch. Default is 2 (seconds), as suggested by
#'   \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#'   2003}.
#' @param unit string. The unit of orientation angles. Can be "deg" (degree) or
#'   "rad" (radian). Default is "deg".
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the epochs generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate epochs. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#' @return dataframe. The returned dataframe will have the same format as input
#'   dataframe.
#'
#' @family aggregate functions
#' @seealso \code{\link{aggregate_for_mims}} for aggregating to get integrated
#'   values for each axis for each epoch.
#'
#' @note If \code{epoch} argument is not provided or is \code{NULL}, the
#'   function will treat the input dataframe as a single epoch.
#'
#'   If the number of samples in an epoch is less than 90%, the output
#'   would be NaN (invalid) for this epoch.
#' @export
#' @examples
#'   # Use sample input data
#'   df = sample_raw_accel_data
#'   head(df)
#'
#'   # set epoch to 1 second and unit to degree
#'   # last epoch does not have enough samples to estimate orientation angles.
#'   aggregate_for_orientation(df, epoch='1 sec', unit='deg')
#'
#'   # set epoch to 2 seconds and unit to radian
#'   # last epoch does not have enough samples to estimate orientation angles.
#'   aggregate_for_orientation(df, epoch='2 sec', unit='rad')
#'
#'   # epoch set to 2 seconds, and st set to be 1 second before the start time of the data
#'   # so the first segment will only include data for 1 second, therefore the resulted
#'   # aggregated value for the first segment will be -1 (invalid) because the
#'   # samples are not enough. And the second segment starts from 11:00:01, instead
#'   # of 11:00:01 as shown in prior example,
#'   aggregate_for_orientation(df, epoch = '1 sec', unit='rad', st=df[1,1] - 1)
#'
aggregate_for_orientation <-
  function(df,
           epoch,
           estimation_window = 2,
           unit = "deg", st=NULL) {
    time_zone <- lubridate::tz(df[[1, 1]])
    n_cols <- ncol(df)

    # parse input argument epoch
    if (missing(epoch) || is.null(epoch)) {
      df$SEGMENT <- 1
    } else {
      df <- MIMSunit::segment_data(df, breaks = epoch, st = st)
    }

    # get the desired number of samples in each epoch
    n_threshold <-
      parse_epoch_string(epoch_str = epoch, sr = sampling_rate(df))

    # iterate over each column
    result <- plyr::ddply(df, c("SEGMENT"), function(rows) {
      rows <- stats::na.omit(rows)

      # Do estimation if there are enough samples in the epoch
      if (nrow(rows) >= 0.9 * n_threshold) {
        ori_values <-
          compute_orientation(rows[, 1:n_cols],
            estimation_window = estimation_window,
            unit = unit
          )
      } else {
        ori_values <-
          data.frame(
            HEADER_TIME_STAMP = rows[1, 1],
            X_ANGLE = NaN,
            Y_ANGLE = NaN,
            Z_ANGLE = NaN
          )
      }
      return(ori_values)
    })

    # format output
    result$SEGMENT <- NULL
    names(result)[1] <- names(df)[1]
    result[1] <-
      as.POSIXct(result[[1]], origin = "1970-01-01", tz = time_zone)
    return(result)
  }

#' Interpolate missing points and unify sampling rate for multi-channel signal
#'
#' \code{interpolate_signal} applies different interpolation algorithms to the
#' input multi-channel signal to fill in the missing samples and harmonizes the
#' sampling rate.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that has been used in functions: \code{\link{extrapolate}}, and
#'   \code{\link{simulate_new_data}}.
#'
#' @param df dataframe. Input multi-channel accelerometer signal.
#' @param method string. Interpolation algorithms. Could be "spline_natural",
#'   "spline_improved" or "spline_fmm": see \code{\link[stats]{splinefun}};
#'   and "linear": see \code{\link[stats]{approxfun}}. Default is "spline_natural".
#' @param sr number. Sampling rate in Hz of the output signal. Default is 100.
#' @param st POSIXct date. The start time for interpolation. If it is
#'   \code{NULL}, it will use the start time of the input signal. Default is
#'   \code{NULL}.
#' @param et POSIXct date. The end time for interpolation. If it is \code{NULL},
#'   it will use the end time of the input signal. Default is \code{NULL}.
#' @return dataframe. Interpolated signal.
#' @family utility functions
#' @export
#' @examples
#'   # Use sample data
#'   df = sample_raw_accel_data
#'
#'   # Plot input
#'   illustrate_signal(df, plot_maxed_out_line=FALSE)
#'
#'   # Interpolate to 100 Hz
#'   sr = 100
#'
#'   # Interpolate the entire sequence of data
#'   output = interpolate_signal(df, sr=sr)
#'
#'   # Plot output
#'   illustrate_signal(output, plot_maxed_out_line=FALSE)
#'
#'   # Interpolate part of the sequence
#'   output = interpolate_signal(df, sr=sr, st=df[10,1], et=df[100,1])
#'
#'   # Plot output
#'   illustrate_signal(output, plot_maxed_out_line=FALSE)
interpolate_signal <-
  function(df,
           method = "spline_natural",
           sr = 100,
           st = NULL,
           et = NULL) {
    time_zone <- lubridate::tz(df[[1, 1]])
    n_rows <- nrow(df)
    if (is.null(st)) {
      st <- df[[1]][1]
    }
    if (is.null(et)) {
      et <- df[[1]][n_rows]
    }
    n_cols <- ncol(df)
    x_out <- seq(from = st, to = et, by = 1 / sr)
    ts <- df[[1]]
    values <- as.data.frame(df[2:n_cols])
    cn1 = colnames(df)[1]
    rm(df)
    result <- plyr::alply(values,
      .margins = 2, function(col_data) {
        col_name <- names(col_data)[1]
        col_data <- col_data[[1]]
        output <-
          switch(
            method,
            linear = stats::approx(x = ts, y = col_data, xout = x_out),
            spline_fmm = stats::spline(
              x = ts,
              y = col_data,
              xout = x_out,
              method = "fmm"
            ),
            spline_natural = stats::spline(
              x = ts,
              y = col_data,
              xout = x_out,
              method = "natural"
            ),
            spline_improved = stats::spline(
              x = ts,
              y = col_data,
              xout = x_out,
              method = "improved"
            )
          )
        output <- data.frame(output)
        colnames(output) <- c(cn1, col_name)
        return(output)
      }
    )
    rm(x_out)
    result <- Reduce(
      function(x, y) {
        return(dplyr::inner_join(x, y, by = colnames(x)[1]))
      },
      result
    )
    names(result[2:ncol(result)]) <-
      paste("INTERPOLATED", names(result[2:ncol(result)]), sep = "_")
    result[, 1] <-
      as.POSIXct(result[, 1], origin = "1970-01-01", tz = time_zone)
    result = as.data.frame(result)
    return(result)
  }

#' Extrapolate input multi-channel accelerometer data
#'
#' \code{extrapolate} applies the extrapolation algorithm to a multi-channel
#' accelerometer data, trying to reconstruct the true movement from the
#' maxed-out samples.
#'
#' This function first linearly interpolates the input signal to 100Hz, and then
#' applies the extrapolation algorithm (see the manuscript) to recover the
#' maxed-out samples. Maxed-out samples are samples that are cut off because the
#' intensity of the underlying movement exceeds the dynamic range of the device.
#'
#' \code{extrapolate} processes a dataframe of a multi-channel accelerometer
#' signal. \code{extrapolate_single_col} processes a single-channel signal with
#' its timestamps and values specified in the first and second arguments.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is the first
#'   step during MIMS-unit algorithm, applied before filtering.
#'
#' @param df dataframe. Input multi-channel accelerometer data. Used in
#'   \code{\link{extrapolate}}.  The first column should be the date/time
#' @param ... see following parameter list.
#' @param t POSIXct or numeric vector. Input index or timestamp sequence Used in
#'   \code{\link{extrapolate_single_col}}.
#' @param value numeric vector. Value vector used in
#'   \code{\link{extrapolate_single_col}}.
#' @param range numeric vector. The dynamic ranges of the input signal. Should
#'   be a 2-element numeric vector. \code{c(low, high)}, where \code{low} is
#'   the negative max value the device can reach and \code{high} is the positive
#'   max value the device can reach.
#' @param noise_level number. The tolerable noise level in \eqn{g} unit, should
#'   be between 0 and 1. Default is 0.03, which applies to most devices.
#' @param k number. Duration of neighborhood to be used in local spline
#'   regression for each side, in seconds. Default is 0.05, as optimized by
#'   MIMS-unit algorithm.
#' @param spar number. Between 0 and 1, to control how smooth we want to fit
#'   local spline regression, 0 is linear and 1 matches all local points.
#'   Default is 0.6, as optimized by MIMS-unit algorithm.
#' @return \code{extraplate} returns a dataframe with extrapolated multi-channel
#'   signal. \code{extrapolate_single_col} returns a dataframe with extrapolated
#'   single-channel signal, the timestamp col is in numeric values instead of
#'   POSIXct format.
#' @family extrapolation related functions
#' @name extrapolate
#' @examples
#'   # Use the maxed-out data for the conceptual diagram
#'   df = conceptual_diagram_data[
#'           conceptual_diagram_data['GRANGE'] == 4,
#'           c("HEADER_TIME_STAMP", "X")]
#'
#'   # Plot input
#'   illustrate_signal(df, range=c(-4, 4))
#'
#'   # Use the default parameter settings as in MIMunit algorithms
#'   # The dynamic range of the input data is -4g to 4g.
#'   output = extrapolate(df, range=c(-4, 4))
#'
#'   # Plot output
#'   illustrate_signal(output, range=c(-4, 4))
NULL

#' @rdname extrapolate
#' @export
extrapolate <- function(df, ...) {
  time_zone <- lubridate::tz(df[[1, 1]])
  t <- df[[1]]
  cn1 = colnames(df)[1]
  df <- df[2:ncol(df)]
  result <- plyr::alply(df,
                        .margins = 2, function(col_data) {
                          col_name <- names(col_data)[1]
                          output <- extrapolate_single_col(t, col_data[[1]], ...)
                          colnames(output) <- c(cn1, col_name)
                          return(output)
                        }
  )
  rm(df);
  result <- Reduce(
    function(x, y) {
      return(dplyr::inner_join(x, y, by = colnames(x)[1]))
    },
    result
  )
  names(result[2:ncol(result)]) <-
    paste("EXTRAPOLATED", names(result[2:ncol(result)]), sep = "_")
  result[, 1] <-
    as.POSIXct(result[, 1], origin = "1970-01-01", tz = time_zone)
  return(result)
}

#' @rdname extrapolate
#' @export
extrapolate_single_col <-
  function(t,
           value,
           range,
           noise_level = 0.03,
           k = 0.05,
           spar = 0.6) {
    # over sampling to 100Hz
    t_over <- seq(t[1], t[length(t)], by = 1 / 100)

    # over sampling to 100Hz with spline interpolation
    dat_over <-
      stats::spline(
        x = t,
        y = value,
        xout = t_over,
        method = "natural"
      )
    t1 = t[1]
    rm(t)
    rm(t_over)
    rm(value)
    dat_over <- data.frame(dat_over)
    if (lubridate::is.POSIXct(t1)) {
      dat_over[1] <- as.POSIXct(dat_over[[1]], origin = "1970-01-01")
    }
    if (is.unsorted(dat_over$x)) {
      dat_over <- dat_over[order(dat_over$x), ]
    }
    if (anyDuplicated(dat_over$x)) {
      not_dups <- !duplicated(dat_over$x)
      t <- dat_over$x[not_dups]
      value <- dat_over$y[not_dups]
      rm(not_dups)
    } else {
      t <- dat_over$x
      value <- dat_over$y
    }
    rm(dat_over); gc()

    # mark maxed out region using gamma distribution or threshold
    marker_fun <- .extrapolate_mark("gamma")
    marker <- marker_fun(t, value, range[1], range[2], noise_level)

    # mark neighbors
    neighbors <- .extrapolate_neighbor(marker, 100, k)

    # fit local spline regression
    points_ex <-
      .extrapolate_fit(t, value, neighbors, marker, spar, 100, k)
    rm(neighbors); gc()

    # interpolate with the original
    dat_interp <-
      .extrapolate_interpolate(t, value, marker, points_ex, 100)
    rm(points_ex); gc()
    return(dat_interp)
  }

.extrapolate_oversampling <- function(t, value) {
  time_zone <- lubridate::tz(t[1])
  t_over <- seq(t[1], t[length(t)], by = 1 / 100)

  # over sampling to 100Hz with spline interpolation
  dat_over <-
    stats::spline(
      x = t,
      y = value,
      xout = t_over,
      method = "natural"
    )
  t1 = t[1]
  rm(t_over)
  rm(t)
  rm(value)
  dat_over <- data.frame(dat_over)
  if (lubridate::is.POSIXct(t1)) {
    dat_over[1] <- as.POSIXct(dat_over[[1]], origin = "1970-01-01", tz = time_zone)
  }
  if (is.unsorted(dat_over$x)) {
    dat_over <- dat_over[order(dat_over$x), ]
  }
  if (anyDuplicated(dat_over$x)) {
    not_dups <- !duplicated(dat_over$x)
    t <- dat_over$x[not_dups]
    value <- dat_over$y[not_dups]
    rm(not_dups)
  } else {
    t <- dat_over$x
    value <- dat_over$y
  }
  rm(dat_over)

  return(list(t = t, value = value))
}

.extrapolate_mark <- function(method = "gamma") {
  return(switch(method, gamma = .mark_gamma, threshold = .mark_threshold))
}

.mark_gamma <-
  function(t, value, range_low, range_high, noise_sd = noise_sd) {
    # init the mark vector
    marker <- rep(0, length(t))

    # model of 3sd and shape para with confident probability at 0.95
    noise_sd <- noise_sd + 1e-05
    shape <- .optimize_gamma(3 * noise_sd)
    # mark using gamma distribution
    index = value >= 0
    marker[index] <-
      stats::pgamma(value[index] - (range_high - 5 * noise_sd),
                    shape = shape,
                    scale = 1
      )
    index = value < 0
    marker[index] <-
      -stats::pgamma(-value[index] + (range_low + 5 * noise_sd),
                     shape = shape,
                     scale = 1
      )
    return(marker)
  }

.optimize_gamma <- function(value) {
  i <- seq(0.5, 0.001, by = -0.001)
  result <- 0
  previous <- 1
  previous_ii <- 0
  for (ii in i)
  {
    current <- stats::pgamma(value, shape = ii, scale = 1)
    if (previous < 0.95 & current >= 0.95) {
      if (abs(0.95 - previous) > abs(current - 0.95)) {
        result <- ii
      } else {
        result <- previous_ii
      }
      break
    }
    previous <- current
    previous_ii <- ii
  }
  return(result)
}

.mark_threshold <-
  function(t, value, range_low, range_high, noise_sd = noise_sd) {
    # init the mark vector
    marker <- rep(0, length(t))

    # Compute the upper and lower threshold bound
    # according to the range and threshold
    u_bound <- range_high - 5 * noise_sd
    l_bound <- range_low + 5 * noise_sd

    marker[value >= u_bound] <- 1
    marker[value <= l_bound] <- -1
    return(marker)
  }

#' @importFrom magrittr %>%
.extrapolate_edges <- function(marker, confident, sr) {
  marker_diff_left <- c(0, diff(marker))
  marker_diff_right <- c(diff(marker), 0)

  threshold_maxedout <- sr * 5

  # hills
  positive_left_end <-
    which(marker_diff_left > confident & marker > 0)
  positive_right_start <-
    which(marker_diff_right < -confident & marker > 0)

  if (length(positive_left_end) - length(positive_right_start) == 1) {
    # end case > 2 second maxed out edge region
    if (length(marker) - positive_left_end[length(positive_left_end)] >
        threshold_maxedout) {
      positive_right_start <- c(positive_right_start, -1)
    } else {
      # < 2 second maxed out edge region, do nothing
      if (length(positive_left_end) == 1) {
        positive_left_end <- c()
      } else {
        positive_left_end <-
          positive_left_end[1:(length(positive_left_end) - 1)]
      }
    }
  } else if (length(positive_left_end) - length(positive_right_start == -1)) {
    # start case > 2 second maxed out edge region
    if (positive_right_start[1] > threshold_maxedout) {
      positive_left_end <- c(-1, positive_left_end)
    } else {
      # < 2 second maxed out edge region, do nothing
      if (length(positive_right_start) == 1) {
        positive_right_start <- c()
      } else {
        positive_right_start <-
          positive_right_start[2:length(positive_right_start)]
      }
    }
  }

  positive_left_end <- positive_left_end %>% stats::na.omit()
  positive_right_start <-
    positive_right_start %>% stats::na.omit()

  if (abs(length(positive_left_end) - length(positive_right_start)) > 2) {
    # something is wrong
    stop("The side of maxed out hills are diff > 1, should never happen")
  }

  if (any(positive_right_start - positive_left_end < 0) &&
      length(positive_right_start) > 1) {
    positive_left_end <- .shift(positive_left_end, 1)
    positive_right_start <- .shift(positive_right_start, -1)
  }

  positive_edges <-
    data.frame(
      left_end = positive_left_end,
      right_start = positive_right_start,
      stringsAsFactors = FALSE
    )
  rm(positive_left_end)
  rm(positive_right_start)

  # valleys
  negative_left_end <-
    which(marker_diff_left < -confident & marker < 0)
  negative_right_start <-
    which(marker_diff_right > confident & marker < 0)

  if (length(negative_left_end) - length(negative_right_start) == 1) {
    # end case > 2 second maxed out edge region
    if (length(marker) - negative_left_end[length(negative_left_end)] >
        threshold_maxedout) {
      negative_right_start <- c(negative_right_start, -1)
    } else {
      # < 2 second maxed out edge region, do nothing
      if (length(negative_left_end) == 1) {
        negative_left_end <- c()
      } else {
        negative_left_end <-
          negative_left_end[1:(length(negative_left_end) - 1)]
      }
    }
  } else if (length(negative_left_end) - length(negative_right_start == -1)) {
    # start case > 5 second maxed out edge region
    if (negative_right_start[1] > threshold_maxedout) {
      negative_left_end <- c(-1, negative_left_end)
    } else {
      # < 2 second maxed out edge region, do nothing
      if (length(negative_right_start) == 1) {
        negative_right_start <- c()
      } else {
        negative_right_start <-
          negative_right_start[2:length(negative_right_start)]
      }
    }
  }
  rm(marker)

  negative_left_end <- negative_left_end %>% stats::na.omit()
  negative_right_start <-
    negative_right_start %>% stats::na.omit()

  if (abs(length(negative_left_end) - length(negative_right_start)) > 2) {
    # something is wrong
    stop("The side of maxed out valleys are diff > 1, should never happen")
  }

  if (any(negative_right_start - negative_left_end < 0) &&
      length(negative_right_start) > 1) {
    negative_left_end <- .shift(negative_left_end, 1)
    negative_right_start <- .shift(negative_right_start, -1)
  }

  negative_edges <-
    data.frame(
      left_end = negative_left_end,
      right_start = negative_right_start,
      stringsAsFactors = FALSE
    )
  rm(negative_left_end)
  rm(negative_right_start)

  return(rbind(positive_edges, negative_edges))
}

#' @importFrom magrittr %>%
.extrapolate_neighbor <- function(marker, sr, k, confident = 0.5) {
  n_neighbor <- k * sr
  edges <- .extrapolate_edges(marker, confident, sr)
  if (nrow(edges) > 0) {
    neighbors <-
      edges %>%
      tibble::as_tibble() %>%
      dplyr::mutate(
        left_start = edges$left_end - n_neighbor + 1,
        right_end = edges$right_start + n_neighbor - 1
      ) %>%
      as.data.frame()
    rm(edges)
    neighbors$left_start <- pmax(neighbors$left_start, 1)
    neighbors$right_end <-
      pmin(neighbors$right_end, length(marker))
    neighbors$left_start[neighbors$left_end == -1] <- -1
    neighbors$right_end[neighbors$right_start == -1] <- -1
  } else {
    rm(edges)
    neighbors <-
      data.frame(
        left_start = c(),
        right_end = c(),
        left_end = c(),
        right_start = c()
      )
  }
  return(neighbors)
}

.shift <- function(v, lag, truncate = TRUE) {
  n <- length(v)
  if (lag > 0) {
    v <- v[-((n - lag + 1):n)]
  } else {
    v <- v[-(1:(-lag))]
  }
  return(v)
}

#' @importFrom magrittr %>%
.extrapolate_fitline <-
  function(t,
           value,
           neighbors,
           marker,
           spar,
           sr,
           k,
           model = "spline") {
    neighbors$index <- 1:nrow(neighbors)
    point_ex <- neighbors %>% plyr::adply(1, function(neighbor) {
      # validate neighboring
      fitted_left <-
        .fit_weighted(
          t,
          value,
          marker,
          neighbor$left_start,
          neighbor$left_end,
          spar,
          sr,
          k,
          model
        )
      fitted_right <-
        .fit_weighted(
          t,
          value,
          marker,
          neighbor$right_start,
          neighbor$right_end,
          spar,
          sr,
          k,
          model
        )

      st <- t[neighbor$left_end]
      left_end <- 0
      right_start <-
        as.numeric(t[neighbor$right_start] - st, units = "secs")

      middle_t <- (left_end + right_start) / 2
      middle_t <- st + middle_t
      left_x_ex <- c(seq(st, middle_t, 1 / 100), middle_t)
      right_x_ex <-
        c(middle_t, seq(middle_t, st + right_start, 1 / 100))
      # if (left_end == right_start) {
      #   return(data.frame(
      #     t_ex = c(),
      #     value_ex = c(),
      #     type_ex = c(),
      #     index =
      #   ))
      # }
      switch(model,
             spline = {
               left_ex <- fitted_left %>% stats::predict(x = as.numeric(left_x_ex))
               right_ex <-
                 fitted_right %>% stats::predict(x = as.numeric(right_x_ex))
               type_ex <- rep("left_line", length(left_ex$y))

               point_ex_left <-
                 fitted_left %>% stats::predict(x = as.numeric(middle_t))
               point_ex_right <-
                 fitted_right %>% stats::predict(x = as.numeric(middle_t))
               point_ex <- (point_ex_left$y + point_ex_right$y) / 2
               type_ex <- c(type_ex, "point")
               type_ex <-
                 c(type_ex, rep("right_line", length(right_ex$y)))
               index <- rep(neighbor$index, length(type_ex))
             }
      )
      return(data.frame(
        t_ex = c(left_x_ex, middle_t, right_x_ex),
        value_ex = c(left_ex$y, point_ex, right_ex$y),
        type_ex = type_ex,
        index = index
      ))
    },
    .inform = TRUE, .id = NULL, .expand = FALSE
    )
    return(point_ex)
  }

#' @importFrom magrittr %>%
.extrapolate_fit <-
  function(t,
           value,
           neighbors,
           marker,
           spar,
           sr,
           k,
           model = "spline") {
    point_ex <- neighbors %>% plyr::adply(1, function(neighbor) {
      # validate neighboring

      fitted_left <-
        .fit_weighted(
          t,
          value,
          marker,
          neighbor$left_start,
          neighbor$left_end,
          spar,
          sr,
          k,
          model
        )
      fitted_right <-
        .fit_weighted(
          t,
          value,
          marker,
          neighbor$right_start,
          neighbor$right_end,
          spar,
          sr,
          k,
          model
        )

      if (is.null(fitted_left)) {
        n_rep <- neighbor$right_start - 1
        point_ex <- rep(-200, n_rep) # give an extra huge value
        middle_t <- t[1:(neighbor$right_start - 1)]
      } else if (is.null(fitted_right)) {
        n_rep <- length(t) - neighbor$left_end
        point_ex <- rep(-200, n_rep) # give an extra huge value
        middle_t <- t[(neighbor$left_end + 1):length(t)]
      } else {
        st <- t[neighbor$left_end]
        left_end <- 0
        right_start <-
          as.numeric(t[neighbor$right_start] - st, units = "secs")

        middle_t <- (left_end + right_start) / 2
        middle_t <- st + middle_t
        switch(model, linear = {
          left_ex <-
            fitted_left %>%
            stats::predict(data.frame(over_t = as.numeric(middle_t)))
          right_ex <-
            fitted_right %>%
            stats::predict(data.frame(over_t = as.numeric(middle_t)))
          point_ex <- (left_ex + right_ex) / 2
        },
        spline = {
          left_ex <- fitted_left %>% stats::predict(x = as.numeric(middle_t))
          right_ex <-
            fitted_right %>% stats::predict(x = as.numeric(middle_t))
          point_ex <- (left_ex$y + right_ex$y) / 2
        }
        )
      }
      return(data.frame(t_ex = middle_t, value_ex = point_ex))
    },
    .inform = TRUE,
    .id = NULL,
    .expand = FALSE
    )
    return(point_ex)
  }

.fit <-
  function(t,
           value,
           marker,
           start,
           end,
           spar,
           sr,
           k,
           model = "spline") {
    if (start < 0) {
      start <- 1
    }
    if (end > length(t)) {
      end <- length(t)
    }
    # over sampling so that we have enough points
    n_over <- k * sr
    index = start:end
    sub_t <- t[index]
    sub_value <- value[index]
    sp <- stats::approx(x = sub_t, y = sub_value, n = n_over)
    over_t <- sp$x[-n_over]
    over_value <- sp$y[-n_over]

    # fit locally with spline smoothing
    fitted <-
      switch(
        model,
        spline = stats::smooth.spline(over_t, over_value, spar = spar),
        linear = stats::lm(over_value ~ over_t, na.action = stats::na.omit)
      )
    return(fitted)
  }

.fit_weighted <-
  function(t,
           value,
           marker,
           start,
           end,
           spar,
           sr,
           k,
           model = "spline") {
    # if(start < 0) start = 1 if(end > length(t)) end = length(t)

    # over sampling so that we have enough points
    n_over <- k * sr

    if (start == -1 && end == -1) {
      # start edge case
      fitted <- NULL
    } else {
      index = start:end
      sub_t <- t[index]
      sub_value <- value[index]
      weight <- 1 - marker[index]
      sp <- stats::approx(x = sub_t, y = sub_value, n = n_over)
      rm(sub_value)
      weight <- stats::approx(x = sub_t, y = weight, n = n_over)
      rm(sub_t)
      over_t <- sp$x
      over_value <- sp$y
      rm(sp)
      weight <- weight$y
      # fit locally with spline smoothing
      fitted <-
        switch(
          model,
          spline = stats::smooth.spline(over_t,
                                        over_value,
                                        weight,
                                        spar = spar
          ),
          linear = stats::lm(
            over_value ~ over_t,
            weights = weight,
            na.action = stats::na.omit
          )
        )
    }
    return(fitted)
  }

#' @importFrom magrittr %>%
.extrapolate_interpolate <-
  function(t, value, marker, points_ex, sr, confident = 0.5) {

    first_t = dplyr::first(t)
    last_t = dplyr::last(t)

    mark_it <- abs(marker) < confident
    rm(marker); gc()
    length_t_mark = sum(mark_it, na.rm = TRUE)

    # t_mark = t value_mark = value
    if (length_t_mark / length(t) < 0.3) {
      dat <- data.frame(t = t, value = value)
    } else {
      # only have to calculate these if the above isn't true
      # since mark_it is logical, can just do the sum above
      value_mark <- value[mark_it]
      rm(value)

      t_mark <- t[mark_it]
      rm(mark_it)

      rm(t); gc()

      dat <-
        rbind(
          data.frame(t = t_mark, value = value_mark),
          data.frame(t = points_ex$t_ex, value = points_ex$value_ex)
        ) %>% dplyr::arrange(t)
      rm(t_mark); gc()
      rm(value_mark); gc()
    }
    t = dat$t
    value = dat$value
    rm(dat); gc()
    t_interp <-
      seq(first_t, last_t, by = 1 / sr)
    dat <-
      stats::spline(t, y = value, xout = t_interp) %>%
      as.data.frame() %>%
      stats::na.omit()
    names(dat) <- c("t", "value")

    return(dat)
  }

#' Sample raw accelerometer data
#'
#' A raw accelerometer data file contains treadmill data collected from a human
#' subject.
#'
#' @format A data frame with 480 rows and 4 variables:
#' \describe{
#'   \item{HEADER_TIME_STAMP}{Timestamp, in POSIXct}
#'   \item{X}{X axis values, in number}
#'   \item{Y}{Y axis values, in number}
#'   \item{Z}{Z axis values, in number}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit/}
"sample_raw_accel_data"

#' Coefficient of variation values for different acceleration data summary
#' algorithms
#'
#' A dataset containing the coefficient of variation values at different
#' frequencies for the dataset that includes accelerometer measures of different
#' devices on a standard elliptical shaker.
#'
#' @format A data frame with 30 rows and 3 variables:
#' \describe{
#'   \item{TYPE}{Accelerometer summary algorithm name, in character}
#'   \item{HZ}{The frequency of the elliptical shaker, in number}
#'   \item{COEFF_OF_VARIATION}{The coefficient of variation values, in number}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit-dataset-shaker/}
"cv_different_algorithms"


#' The mean and standard deviation of accelerometer summary measure for
#' different acceleration data summary algorithms and for different devices.
#'
#' A dataframe contains the mean and standard deviation of accelerometer summary
#' measured at different frequencies for the raw accelerometer signals
#' from different devices collected from on a standard elliptical shaker.
#'
#' @format A data frame with 235 rows and 8 variables:
#' \describe{
#'   \item{DEVICE}{The name of different devices, in character}
#'   \item{GRANGE}{The dynamic range of the device in g, in number}
#'   \item{SR}{The sampling rate in Hz of the device, in number}
#'   \item{TYPE}{Accelerometer summary algorithm name, in character}
#'   \item{HZ}{The frequency of the elliptical shaker, in number}
#'   \item{NAME}{An alternative name that is friendly for plotting for devices, in character}
#'   \item{mean}{The mean values of accelerometer summary measure, in number}
#'   \item{sd}{The standard deviation values of accelerometer summary measure, in number}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit-dataset-shaker/}
"measurements_different_devices"


#' The input accelerometer data used to generate the conceptual diagram
#' (Figure 1) in the manuscript.
#'
#' The dataset includes accelerometer data from four devices. Device 0 is a real
#'  Actigraph GT9X device configured at 80Hz and 8g. Device 1 to 3 are simulated
#'  data from the data of device 0 using function \code{\link{simulate_new_data}}.
#'  Data for device 0 is a random selected nondominant wrist data from a
#'  participant doing Jumping jack. The data is manipulated to insert an
#'  artificial impulse to demonstrate the effect of the MIMS-unit algorithm when
#'  dealing on it.
#'
#' @format A data frame with 1704 rows and 5 variables:
#' \describe{
#'   \item{HEADER_TIME_STAMP}{The timestamp of raw accelerometer data, in POSIXct}
#'   \item{X}{The x axis value of raw accelerometer data, in number}
#'   \item{GRANGE}{The dynamic range of the simulated device in g, in number}
#'   \item{SR}{The sampling rate in Hz of the simulated device, in number}
#'   \item{NAME}{An alternative name that is friendly for plotting for different devices, in character}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit/}
"conceptual_diagram_data"

#' A short snippet of raw accelerometer signal from a device resting on a table.
#'
#' The dataset includes accelerometer data sampled at 80Hz and 6g. This data is used to derive the thresholding.
#'
#' @format A data frame with 5000 rows and 4 variables:
#' \describe{
#'   \item{HEADER_TIME_STAMP}{The timestamp of raw accelerometer data, in POSIXct}
#'   \item{X}{The x axis value of raw accelerometer data, in number}
#'   \item{Y}{The x axis value of raw accelerometer data, in number}
#'   \item{Z}{The x axis value of raw accelerometer data, in number}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit/}
"rest_on_table"

#' A short snippet of raw accelerometer signal from a device that has ending data maxed out.
#'
#' The dataset includes accelerometer data sampled at 80Hz and 6g. This data is used to test the edge case.
#'
#' @format A data frame with 20001 rows and 4 variables:
#' \describe{
#'   \item{HEADER_TIME_STAMP}{The timestamp of raw accelerometer data, in POSIXct}
#'   \item{X}{The x axis value of raw accelerometer data, in number}
#'   \item{Y}{The x axis value of raw accelerometer data, in number}
#'   \item{Z}{The x axis value of raw accelerometer data, in number}
#' }
#' @source \url{https://github.com/mHealthGroup/MIMSunit/}
"edge_case"

#' Cut off input multi-channel signal according to a new dynamic range
#'
#' \code{cut_off_signal} cuts off the input multi-channel accelerometer data
#' according to a new dynamic range, then adds gausian noise to the cut-off
#' samples.
#'
#' This function simulates the behavior that a low dynamic range device is
#' trying to record high intensity movement, where recorded accelerometer signal
#' will be cut off at the dynamic range, but the true movement should have
#' higher acceleration values than the dynamic range. This function also adds
#' gaussian noise to the cut off samples to better simulate the real world
#' situation.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that is used to simulate the behaviors of low dynamic range
#'   devices during algorithm validation.
#'
#' @param df dataframe. Input multi-channel accelerometer data.
#' @param range numerical vector. The new dynamic ranges to cut off the signal.
#'   Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach. Default is \code{NULL},
#'   meaning the function will do nothing but return the input data.
#' @param noise_std number. The standard deviation of the added gaussian noise.
#' @return dataframe. The multi-channel accelerometer data with the new dynamic
#'   range as specified in \code{range}.
#' @family utility functions
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = sample_raw_accel_data
#'
#'   # Show df
#'   illustrate_signal(df, range=c(-8, 8))
#'
#'   # cut off the signal to c(-2, 2)
#'   new_df = cut_off_signal(df, range=c(-2, 2), noise_std=0.03)
#'
#'   # Show new df
#'   illustrate_signal(new_df, range=c(-2, 2))
cut_off_signal <- function(df,
                           range = NULL,
                           noise_std = 0.03) {
  if (!is.null(range)) {
    upper <- range[2]
    lower <- range[1]
    crop_fun <- plyr::colwise(function(col_data) {
      col_data[col_data > upper] <-
        upper + stats::rnorm(sum(col_data > upper), 0, noise_std)
      col_data[col_data < lower] <-
        lower + stats::rnorm(sum(col_data < lower), 0, noise_std)
      return(col_data)
    })
    df[, 2:ncol(df)] <- crop_fun(df[2:ncol(df)])
  }
  return(df)
}

#' Simulate new data based on the given multi-channel accelerometer data
#'
#' \code{simulate_new_data} simulate new data based on the given multi-channel
#' accelerometer data, a new dynamic range and a new sampling rate.
#'
#' This function simulates the data from a new device based on the signal from a
#' baseline device. It first changes the sampling rate using function
#' \code{\link{interpolate_signal}}, and then changes the dynamic range using
#' function \code{\link{cut_off_signal}}.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is a utility
#'   function that is used to simulate new devices with different sampling rates
#'   and dynamic ranges during algorithm validation.
#'
#' @param old_data dataframe. Input multi-channel accelerometer data.
#' @param new_range numerical vector. The new dynamic ranges to cut off the signal.
#'   Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach. Default is \code{NULL},
#'   meaning the function will do nothing but return the input data.
#' @param new_sr number. New sampling rate in Hz.
#' @family utility functions
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = sample_raw_accel_data
#'
#'   # Show df
#'   illustrate_signal(df, range=c(-8, 8))
#'
#'   # simulate new data by changing range and sampling rate
#'   new_df = simulate_new_data(df, new_range=c(-2, 2), new_sr = 30)
#'
#'   # Show new df
#'   illustrate_signal(new_df, range=c(-2, 2))
simulate_new_data <- function(old_data, new_range, new_sr) {
  if (sampling_rate(old_data) != new_sr) {
    new_data <-
      interpolate_signal(old_data, method = "spline_natural", sr = new_sr)
  } else {
    new_data <- old_data
  }
  new_data <- cut_off_signal(new_data, range = new_range)
  return(new_data)
}

check_epoch = function(breaks) {
  # taken from cut.POSIXct
  if (is.character(breaks) && length(breaks) == 1L) {
    by2 <- strsplit(breaks, " ", fixed = TRUE)[[1L]]
    if (length(by2) > 2L || length(by2) < 1L)
      stop("invalid specification of 'breaks'")
    valid <- pmatch(by2[length(by2)], c("secs", "mins", "hours",
                                        "days", "weeks", "months", "years", "DSTdays", "quarters"))
    if (is.na(valid)) {
      stop("invalid specification of 'breaks'")
    }
  }
}

#' Apply IIR filter to the signal
#'
#' \code{iir} function takes a multi-channel signal and applies an IIR filter to
#' the signal.
#'
#' This function filters the input multi-channel signal by applying an IIR
#' filter. See
#' \href{https://en.wikipedia.org/wiki/Infinite_impulse_response}{wiki} for the
#' explanation of the filter. The implementations of IIR filters can be found in
#' \code{\link[signal]{butter}}, \code{\link[signal]{cheby1}}, and \code{\link[signal]{ellip}}.
#'
#' For Chebyshev Type I, Type II and Elliptic filter, the passband ripple is
#' fixed to be 0.05 dB. For Elliptic filter, the stopband ripple is fixed to be
#' -50dB.
#'
#' @section How is it used in MIMS-unit algorithm?: This function has been used
#'   as the main filtering method in MIMS-unit algorithm. Specifically, it uses
#'   a 0.5 - 5 Hz bandpass butterworth filter during filtering.
#'
#' @param df dataframe. The input multi-channel signal. The first column is
#'   timestamps in POSXlct format. The rest columns are signal values.
#' @param sr number. Sampling rate in Hz of the input signal.
#' @param cutoff_freq number or numerical vector. The cutoff frequencies in Hz.
#'   If the IIR filter is a bandpass or bandstop filter, it will be a 2-element
#'   numerical vector specifying the low and high end cutoff frequencies
#'   \code{c(low, high)}.
#' @param order number. The order of the filter. Default is 4.
#' @param type string. Filtering type, one of "low" for a low-pass filter,
#'   "high" for a high-pass filter, "stop" for a stop-band (band-reject) filter,
#'   or "pass" for a pass-band filter.
#' @param filter_type string. IIR filter type, one of "butter" for butterworth
#'   filter, "chebyI" for Chebyshev Type I filter, or "ellip" for Elliptic filter.
#' @return dataframe. Filtered signal.
#' @family filtering functions
#' @export
#' @examples
#'   # Use sample data
#'   df = sample_raw_accel_data
#'
#'   # View input
#'   illustrate_signal(df, plot_maxed_out_line = FALSE)
#'
#'   # Apply filtering that uses the same setting as in MIMSunit algorithm
#'   output = iir(df, sr=80, cutoff_freq=c(0.2, 5), type='pass')
#'
#'   # View output
#'   illustrate_signal(output, plot_maxed_out_line = FALSE)
iir <-
  function(df,
           sr,
           cutoff_freq,
           order = 4,
           type = "high",
           filter_type = "butter") {
    nyquist <- sr / 2

    coeffs <-
      switch(
        filter_type,
        butter = signal::butter(order, cutoff_freq / nyquist, type),
        chebyI = signal::cheby1(order, 0.05,
                                W = cutoff_freq / nyquist,
                                type, plane = "z"
        ),
        chebyII = signal::cheby2(order, 0.05,
                                 W = cutoff_freq / nyquist,
                                 type, plane = "z"
        ),
        ellip = signal::ellip(order, 0.05,
                              50,
                              W = cutoff_freq / nyquist, type, plane = "z"
        )
      )

    n_cols <- ncol(df)

    filt_fun = function(x, filt, a) {
      filtered <- signal::filter(filt, a, x)
      result <- as.numeric(filtered)
      return(result)
    }
    for (icol in 2:n_cols) {
      filt_result = filt_fun(
        df[[icol]],
        filt = coeffs$b,
        a = coeffs$a)
      df[[icol]] = filt_result
    }
    colnames(df)[2:n_cols] <-
      paste0("IIR_", colnames(df)[2:n_cols])
    return(df)
  }

#' Apply a bandlimited interpolation filter to the signal to change the sampling
#' rate
#'
#' \code{bandlimited_interp} function takes a multi-channel signal and applies a
#' bandlimited interpolation filter to the signal to change its sampling rate.
#'
#' This function filters the input multi-channel signal by applying a bandlimited
#' interpolation filter. See \code{\link[signal]{resample}} for the underlying
#' implementation.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is not used in
#'   the released version of MIMS-unit algorithm, but has once been considered
#'   to be used after extrapolation to harmonize sampling rate before filtering.
#'   But in the end, we decided to use linear interpolation before extrapolation
#'   to increase the sampling rate to 100Hz, so this method is no longer needed.
#'
#' @param df dataframe. The input multi-channel signal. The first column is
#'   timestamps in POSXlct format. The rest columns are signal values.
#' @param orig_sr number. Sampling rate in Hz of the input signal.
#' @param new_sr number. The desired sampling rate in Hz of the output signal.
#' @return dataframe. Filtered signal.
#' @family filtering functions
#' @export
#' @examples
#'   # Use sample data
#'   df = sample_raw_accel_data
#'
#'   # View input
#'   illustrate_signal(df, plot_maxed_out_line = FALSE)
#'
#'   # Apply filtering that uses the same setting as in MIMSunit algorithm
#'   output = bandlimited_interp(df, orig_sr=80, new_sr=30)
#'
#'   # View output
#'   illustrate_signal(output, plot_maxed_out_line = FALSE)
bandlimited_interp <- function(df, orig_sr, new_sr) {
  n_cols <- ncol(df)

  col_filter <- plyr::colwise(
    .fun = function(x) {
      resampled <-
        as.numeric(signal::resample(x, p = new_sr, q = orig_sr))
    }
  )
  resampled_value <- col_filter(df[2:n_cols])
  resampled_ts <-
    seq(
      from = df[1, 1],
      to = dplyr::last(df[, 1]),
      length = nrow(resampled_value)
    )
  resampled_value <- cbind(resampled_ts, resampled_value)
  colnames(resampled_value)[1] <- colnames(df)[1]
  return(resampled_value)
}

#' Estimate the accelerometer orientation
#'
#' \code{compute_orientation} returns a dataframe with accelerometer
#' orientations estimated by
#' \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#' 2003} for the input dataframe.
#'
#' This function accepts a dataframe (in mhealth accelerometer data format) and
#' computes the estimated acclerometer orientations (in x, y, and z angles) for
#' every \code{estimation_window} seconds of the entire sequence, and outputs
#' the mean of these angles. The returned dataframe will have the same format as
#' input dataframe, including four columns, and have the same datetime format as
#' input dataframe in the timestamp column. The orientation estimation method
#' used in the function is based on
#' \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#' 2003}.
#'
#' @section How is it used in mims-unit algorithm?: This function is used in
#'   function (\code{\link{aggregate_for_orientation}}).
#'
#' @param df dataframe. Input multi-channel signal. First column should be
#'   timestamps in POSIXt format.
#' @param estimation_window number. window size in seconds to be used to
#'   estimate orientations. Default is 2 (seconds), as suggested by
#'   \href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
#'   2003}.
#' @param unit string. The unit of orientation angles. Can be "deg" (degree) or
#'   "rad" (radian). Default is "deg".
#' @return dataframe. The returned dataframe will have the same format as input
#'   dataframe.
#'
#' @family transformation functions
#' @export
#' @examples
#'   # Use first 10 second sample data for testing
#'   df = sample_raw_accel_data
#'   df = clip_data(df, start_time = df[1,1], stop_time = df[1, 1] + 600)
#'
#'   # compute orientation angles in degrees
#'   compute_orientation(df)
#'
#'   # compute orientation angles in radian angles
#'   compute_orientation(df, unit='rad')
compute_orientation <- function(df, estimation_window = 2, unit = "deg") {
  sr <- sampling_rate(df)
  segmented_df <-
    MIMSunit::segment_data(df, breaks = paste(estimation_window, "sec"))
  angles_df <-
    plyr::ddply(segmented_df, c("SEGMENT"), function(rows) {
      if (nrow(rows) < sr * estimation_window * 0.8) {
        return(data.frame(
          ts = rows[1, 1],
          x = NaN,
          y = NaN,
          z = NaN
        ))
      }
      axis_means <- colMeans(rows[, 2:4])
      gravity <- sqrt(sum(axis_means^2))
      angles <- acos(axis_means / gravity)
      if (unit == "deg") {
        angles <- angles * 180 / pi
      }
      return(data.frame(
        ts = rows[1, 1],
        x = angles[1],
        y = angles[2],
        z = angles[3]
      ))
    })
  mean_angles <- colMeans(angles_df[, 3:5], na.rm = TRUE)
  mean_angles_df <-
    data.frame(
      ts = df[1, 1],
      x = mean_angles[[1]],
      y = mean_angles[[2]],
      z = mean_angles[[3]]
    )
  colnames(mean_angles_df) <-
    c("HEADER_TIME_STAMP", "X_ANGLE", "Y_ANGLE", "Z_ANGLE")
  return(mean_angles_df)
}

#' Compute Monitor Independent Motion Summary unit (MIMS-unit)
#'
#' \code{mims_unit} computes the Monitor Independent Motion Summary unit for the
#' input multi-channel accelerometer signal. The input signal can be from
#' devices of any sampling rate and dynamic range. Please refer to the
#' manuscript for detailed description of the algorithm. Please refer to
#' functions for the intermediate steps: \code{\link{extrapolate}} for
#' extrapolation, \code{\link{iir}} for filtering,
#' \code{\link{aggregate_for_mims}} for aggregation.
#'
#' @note This function is a wrapper function for the low-level
#'   \code{\link{custom_mims_unit}} function. It has set internal parameters as
#'   described in the manuscript. If you want to run customized algorithm for
#'   MIMSunit or if you want to develop better algorithms based on MIMS-unit
#'   algorithm, please use function \code{\link{custom_mims_unit}} where all
#'   parameters are tunable.
#'
#'   \code{before_df} and \code{after_df} are often set when the accelerometer
#'   data are divided into files of smaller chunk.
#'
#'   Please make sure the input data do not contain duplicated timestamps. See
#'    more information about this \href{https://github.com/mHealthGroup/MIMSunit/issues/32}{issue}.
#'    Otherwise the computation will stop.
#'
#' @section How is it used in MIMS-unit algorithm?: This is the main entry of
#'   MIMS-unit algorithm.
#'
#' @param epoch string. Any format that is acceptable by argument \code{breaks}
#'   in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
#'   sec", "10 min". Default is "5 sec".
#' @param dynamic_range numerical vector. The dynamic ranges of the input
#'   signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach.
#' @param output_mims_per_axis logical. If it is TRUE, the output MIMS-unit
#'   dataframe will have MIMS-unit values for each axis from the third column.
#'   Default is FALSE.
#' @param use_gui_progress logical. If it is TRUE, show GUI progress bar on
#'   windows platform. Default is FALSE.
#' @param use_snapshot_to_check logical. If TRUE, the function will use the first
#'   100 rows or 10% (whichever is smaller) to check timestamp duplications. Otherwise,
#'   the algorithm will use all data to check timestamp duplications. Default is FALSE.
#' @return dataframe. The MIMS-unit dataframe. The first column is the start
#'   time of each epoch in POSIXct format. The second column is the MIMS-unit
#'   value for the input signal. If \code{output_mims_per_axis} is TRUE, the
#'   third column and then are the MIMS-unit values for each axis of the input
#'   signal.
#'
#' @family Top level API functions
#' @name mims_unit


#' @rdname mims_unit
#' @param df dataframe. Input multi-channel accelerometer signal. The
#' first column should be the time component. The accelerometer data values
#'  (typically starting from the second column) should be in \eqn{g} (per \eqn{9.81m/s^2}) unit.
#' @param before_df dataframe. The multi-channel accelerometer signal comes
#'   before the input signal to be prepended to the input signal during
#'   computation. This is used to eliminate the edge effect during extrapolation
#'   and filtering. If it is \code{NULL}, algorithm will run directly on the
#'   input signal. Default is NULL.
#' @param after_df dataframe. The multi-channel accelerometer signal comes after
#'   the input signal to be append to the input signal. This is used to
#'   eliminate the edge effect during extrapolation and filtering. If it is
#'   \code{NULL}, algorithm will run directly on the input signal. Default is
#'   NULL.
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the epochs generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate epochs. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = head(sample_raw_accel_data, 240)
#'
#'   # compute mims unit values and output axial values
#'   output = mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), output_mims_per_axis=TRUE)
#'   head(output)
mims_unit <-
  function(df,
           before_df = NULL,
           after_df = NULL,
           epoch = "5 sec",
           dynamic_range,
           output_mims_per_axis = FALSE,
           use_gui_progress = FALSE,
           st = NULL,
           use_snapshot_to_check=FALSE) {
    mims_df <- custom_mims_unit(
      df = df,
      epoch = epoch,
      dynamic_range = dynamic_range,
      noise_level = 0.03,
      k = 0.05,
      spar = 0.6,
      filter_type = "butter",
      cutoffs = c(0.2, 5),
      axes = c(2, 3, 4),
      use_extrapolation = TRUE,
      use_filtering = TRUE,
      combination = "sum",
      allow_truncation = TRUE,
      output_mims_per_axis = output_mims_per_axis,
      output_orientation_estimation = FALSE,
      before_df = before_df,
      after_df = after_df,
      use_gui_progress = use_gui_progress,
      st = st,
      use_snapshot_to_check = use_snapshot_to_check
    )
    return(mims_df)
  }

#' Estimates sensor orientation
#'
#' \code{sensor_orientations} estimates the orientation angles for the input
#' multi-channel accelerometer signal. The input signal can be from devices of
#' any sampling rate and dynamic range. Please refer to function
#' \code{\link{compute_orientation}} for the implementation of the estimation
#' algorithm.
#'
#' @note This function interpolates and extrapolates the signal before
#'   estimating the orientation angles.
#'
#'   \code{before_df} and \code{after_df} are often set when the accelerometer
#'   data are divided into files of smaller chunk.
#'
#' @section How is it used in MIMS-unit algorithm?: This is not included in the
#'   official MIMS-unit algorithm nor the manuscript, but we found it is useful
#'   to know the sensor orientations in addition to the summary of movement.
#'
#' @param df dataframe. Input multi-channel accelerometer signal.
#' @param before_df dataframe. The multi-channel accelerometer signal comes
#'   before the input signal to be prepended to the input signal during
#'   computation. This is used to eliminate the edge effect during extrapolation
#'   and filtering. If it is \code{NULL}, algorithm will run directly on the
#'   input signal. Default is NULL.
#' @param after_df dataframe. The multi-channel accelerometer signal comes after
#'   the input signal to be append to the input signal. This is used to
#'   eliminate the edge effect during extrapolation and filtering. If it is
#'   \code{NULL}, algorithm will run directly on the input signal. Default is
#'   NULL.
#' @param epoch string. Any format that is acceptable by argument \code{breaks}
#'   in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
#'   sec", "10 min". Default is "5 sec".
#' @param dynamic_range numerical vector. The dynamic ranges of the input
#'   signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach.
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the epochs generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate epochs. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#' @return dataframe. The orientation dataframe. The first column is the start
#'   time of each epoch in POSIXct format. The second to fourth columns are the
#'   orientation angles.
#'
#' @family Top level API functions
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = head(sample_raw_accel_data, 240)
#'
#'   # compute sensor orientation angles
#'   output = sensor_orientations(df, epoch = '1 sec', dynamic_range=c(-8, 8))
#'   head(output)
sensor_orientations <-
  function(df,
           before_df = NULL,
           after_df = NULL,
           epoch = "5 sec",
           dynamic_range,
           st = NULL) {
    ori_df <- custom_mims_unit(
      df = df,
      epoch = epoch,
      dynamic_range = dynamic_range,
      output_orientation_estimation = TRUE,
      epoch_for_orientation_estimation = epoch,
      before_df = before_df,
      after_df = after_df,
      st = st
    )[[2]]
    return(ori_df)
  }


#' Compute both MIMS-unit and sensor orientations with custom settings
#'
#' \code{custom_mims_unit} computes the Monitor Independent Motion Summary unit
#' and estimates the sensor orientations for the input multi-channel
#' accelerometer signal with custom settings. The input signal can be from
#' devices of any sampling rate and dynamic range. Please refer to the
#' manuscript for detailed description of the algorithm. Please refer to
#' functions for the intermediate steps: \code{\link{extrapolate}} for
#' extrapolation, \code{\link{iir}} for filtering,
#' \code{\link{aggregate_for_mims}} and \code{\link{aggregate_for_orientation}}
#' for aggregation.
#'
#' @note This function allows you to run customized algorithm for MIMSunit and
#'   sensor orientations.
#'
#'   \code{before_df} and \code{after_df} are often set when the accelerometer
#'   data are divided into files of smaller chunk.
#'
#' @section How is it used in MIMS-unit algorithm?: This is the low-level entry
#'   of MIMS-unit and orientation estimation algorithm. \code{\link{mims_unit}}
#'   calls this function internally.
#'
#' @param df dataframe. Input multi-channel accelerometer signal.
#' @param epoch string. Any format that is acceptable by argument \code{breaks}
#'   in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
#'   sec", "10 min". Default is "5 sec".
#' @param dynamic_range numerical vector. The dynamic ranges of the input
#'   signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
#'   \code{low} is the negative max value the device can reach and \code{high}
#'   is the positive max value the device can reach.
#' @param noise_level number. The tolerable noise level in \eqn{g} unit, should
#'   be between 0 and 1. Default is 0.03, which applies to most devices.
#' @param k number. Duration of neighborhood to be used in local spline
#'   regression for each side, in seconds. Default is 0.05, as optimized by
#'   MIMS-unit algorithm.
#' @param spar number. Between 0 and 1, to control how smooth we want to fit
#'   local spline regression, 0 is linear and 1 matches all local points.
#'   Default is 0.6, as optimized by MIMS-unit algorithm.
#' @param filter_type string. The type of filter to be applied. Could be
#'   'butter' for butterworth bandpass filter, 'ellip' for elliptic bandpass
#'   filter or 'bessel' for bessel lowpass filter + average removal highpass
#'   filter. Default is "butter".
#' @param cutoffs numerical vector. Cut off frequencies to be used in filtering.
#'   If \code{filter_type} is "bessel", the cut off frequency for lowpass filter
#'   would be multiplied by 2 when being used. Default is 0.2Hz and 5Hz.
#' @param axes numerical vector. Indices of columns that specifies the axis
#'   values of the input signal. Default is \code{c(2,3,4)}.
#' @param use_extrapolation logical. If it is TRUE, the function will apply
#'   extrapolation algorithm to the input signal, otherwise it will skip
#'   extrapolation but only linearly interpolate the signal to 100Hz. Default is
#'   TRUE.
#' @param use_filtering logical. If it is TRUE, the function will apply bandpass
#'   filtering to the input signal, otherwise it will skip the filtering.
#'   Default is TRUE.
#' @param combination string. Method to combine MIMS-unit values for each axis.
#'   Could be "sum" for \code{\link{sum_up}} or "vm" for
#'   \code{\link{vector_magnitude}}.
#' @param allow_truncation logical. If it is TRUE, the algorithm will truncate
#'   very small MIMS-unit values to zero. Default is TRUE.
#' @param output_mims_per_axis logical. If it is TRUE, the output MIMS-unit
#'   dataframe will have MIMS-unit values for each axis from the third column.
#'   Default is FALSE.
#' @param output_orientation_estimation logical. If it is TRUE, the function
#'   will also estimate sensor orientations over each epoch. And the output will
#'   be a list, with the first element being the MIMS-unit dataframe, and the
#'   second element being the sensor orientation dataframe. Default is FALSE.
#' @param epoch_for_orientation_estimation string. string. Any format that is
#'   acceptable by argument \code{breaks} in method
#'   \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5 sec", "10
#'   min". Default is "5 sec". It is independent from \code{epoch} for
#'   MIMS-unit.
#' @param before_df dataframe. The multi-channel accelerometer signal comes
#'   before the input signal to be prepended to the input signal during
#'   computation. This is used to eliminate the edge effect during extrapolation
#'   and filtering. If it is \code{NULL}, algorithm will run directly on the
#'   input signal. Default is NULL.
#' @param after_df dataframe. The multi-channel accelerometer signal comes after
#'   the input signal to be append to the input signal. This is used to
#'   eliminate the edge effect during extrapolation and filtering. If it is
#'   \code{NULL}, algorithm will run directly on the input signal. Default is
#'   NULL.
#' @param use_gui_progress logical. If it is TRUE, show GUI progress bar on
#'   windows platform. Default is FALSE.
#' @param st character or POSIXct timestamp. An optional start time you can set to
#'   force the epochs generated by referencing this start time. If it is NULL, the
#'   function will use the first timestamp in the timestamp column as start time to
#'   generate epochs. This is useful when you are processing a stream of data and
#'   want to use a common start time for segmenting data. Default is NULL.
#' @param use_snapshot_to_check logical. If TRUE, the function will use the first
#'   100 rows or 10% (whichever is smaller) to check timestamp duplications. Otherwise,
#'   the algorithm will use all data to check timestamp duplications. Default is FALSE.
#' @return dataframe or list. If \code{output_orientation_estimation} is TRUE,
#'   the output will be a list, otherwise the output will be the MIMS-unit
#'   dataframe.
#'
#'   The first element will be the MIMS-unit dataframe, in which the first
#'   column is the start time of each epoch in POSIXct format, and the second
#'   column is the MIMS-unit value for the input signal, and the third column
#'   and on are the MIMS-unit values for each axis of the input signal if
#'   \code{output_mims_per_axis} is TRUE.
#'
#'   The second element will be the orientation dataframe, in which the first
#'   column is the start time of each epoch in POSIXct format, and the second to
#'   fourth column is the estimated orientations for the input signal.
#'
#' @family Top level API functions
#' @export
#' @examples
#'   # Use sample data for testing
#'   df = sample_raw_accel_data
#'
#'   # compute mims unit values with custom parameter
#'   output = custom_mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), spar=0.7)
#'   head(output)
custom_mims_unit <-
  function(df,
           epoch = "5 sec",
           dynamic_range,
           noise_level = 0.03,
           k = 0.05,
           spar = 0.6,
           filter_type = "butter",
           cutoffs = c(0.2, 5),
           axes = c(2, 3, 4),
           use_extrapolation = TRUE,
           use_filtering = TRUE,
           combination = "sum",
           allow_truncation = TRUE,
           output_mims_per_axis = FALSE,
           output_orientation_estimation = FALSE,
           epoch_for_orientation_estimation = NULL,
           before_df = NULL,
           after_df = NULL,
           use_gui_progress = FALSE,
           st = NULL,
           use_snapshot_to_check=FALSE) {


    check_epoch(breaks = epoch)
    if (inherits(df, "tbl_df")) {
      df = as.data.frame(df)
    }
    first_col = df[[1]]
    if (is.unsorted(first_col)) {
      df = df[ order(first_col), ]
    }
    rm(first_col)
    stopifnot("HEADER_TIME_STAMP" %in% colnames(df))

    # check timestamp duplication after the timestamp column is sorted
    if (use_snapshot_to_check) {
      ts_has_duplication = any(diff(df[1:pmax(100,round(nrow(df[,1])*0.1)),1]) == 0)
    } else {
      ts_has_duplication = any(diff(df[,1]) == 0)
    }
    if (ts_has_duplication) {
      stop("Input data contains duplicated timestamps!")
    }

    # save the start and stop time of original df
    if (shiny::isRunning()) {
      pb = shiny::Progress$new()
      pb$set(message = "Computing MIMS-unit values", value=0)

    }
    else if (use_gui_progress & .Platform$OS.type == 'windows') {
      ProgressBar = utils::winProgressBar
      setProgressBar = utils::setWinProgressBar
      pb = ProgressBar(min = 0,
                       max = 1,
                       title = "Computing MIMS-unit values",
                       label = 'Starting...')
    }
    else {
      ProgressBar = utils::txtProgressBar
      setProgressBar = utils::setTxtProgressBar
      pb = ProgressBar(min = 0,
                       max = 1,
                       title = "Computing MIMS-unit values",
                       label = 'Starting...')
    }



    start_time <- lubridate::floor_date(df[[1]][1], unit = "seconds")
    stop_time <-
      lubridate::floor_date(df[[1]][nrow(df)], unit = "seconds")

    # concatenate with before and after df
    if (is.data.frame(before_df)) {
      if (shiny::isRunning()) {
        pb$inc(0.2, detail='Concatenating before_df...')
      } else {
        setProgressBar(pb, 0.2, label = 'Concatenating before_df...')
      }
      df <- rbind(before_df, df)
    }
    if (is.data.frame(after_df)) {
      if (shiny::isRunning()) {
        pb$inc(0.3, detail='Concatenating after_df...')
      } else {
        setProgressBar(pb, 0.3, label = 'Concatenating after_df...')
      }
      df <- rbind(df, after_df)
    }

    # apply extrapolation algorithm
    if (use_extrapolation) {
      stopifnot(length(dynamic_range) == 2)
      if (shiny::isRunning()) {
        pb$inc(0.4, detail='Extrapolating signal...')
      } else {
        setProgressBar(pb, 0.4, label = 'Extrapolating signal...')
      }
      resampled_data <-
        extrapolate(df, dynamic_range, noise_level, k, spar)
    } else {
      if (shiny::isRunning()) {
        pb$inc(0.4, detail='Interpolating signal...')
      } else {
        setProgressBar(pb, 0.4, label = 'Interpolating signal...')
      }
      resampled_data <-
        interpolate_signal(df, sr = 100, method = "linear")
    }
    rm(df)
    # removed the extrapolated_data
    sr <- sampling_rate(resampled_data)


    # store -0.01 values separately
    row_abnormal <- rep(FALSE, nrow(resampled_data))
    for (i in 2:ncol(resampled_data))
    {
      row_abnormal <- row_abnormal | resampled_data[[i]] < -150
    }

    abnormal_data <- resampled_data[row_abnormal, ]
    normal_data <- resampled_data[!row_abnormal, ]
    rm(row_abnormal)

    # Compute orientations
    if (output_orientation_estimation) {
      if (is.null(epoch_for_orientation_estimation)) {
        epoch_for_orientation_estimation <- epoch
      }
      orientation_data <-
        aggregate_for_orientation(resampled_data,
                                  epoch = epoch_for_orientation_estimation,
                                  st = st
        )
    } else {
      orientation_data <- NULL
    }
    rm(resampled_data); gc();

    # Apply filter cascade
    if (use_filtering) {
      if (shiny::isRunning()) {
        pb$inc(0.6, detail='Filtering signal...')
      } else {
        setProgressBar(pb, 0.6, label = 'Filtering signal...')
      }
      if (filter_type == "butter") {
        filtered_data <-
          iir(
            normal_data,
            sr = sr,
            cutoff_freq = cutoffs,
            order = 4,
            type = "pass",
            filter_type = "butter"
          )
      } else if (filter_type == "ellip") {
        filtered_data <-
          iir(
            normal_data,
            sr = sr,
            cutoff_freq = cutoffs,
            order = 4,
            type = "pass",
            filter_type = "ellip"
          )
      }
    } else {
      filtered_data <- normal_data
    }
    rm(normal_data); gc()

    # sort by timestamp
    colnames(abnormal_data)[2:4] <- colnames(filtered_data)[2:4]
    filtered_data <- rbind(filtered_data, abnormal_data)
    rm(abnormal_data)
    filtered_data <-
      filtered_data[order(filtered_data[[1]]), ]


    # Compute the AUC
    if (shiny::isRunning()) {
      pb$inc(0.8, detail='Computing AUC...')
    } else {
      setProgressBar(pb, 0.8, label = 'Computing AUC...')
    }
    integrated_data <-
      aggregate_for_mims(
        filtered_data,
        epoch = epoch,
        method = "trapz",
        rectify = TRUE,
        st = st
      )
    rm(filtered_data);

    if (allow_truncation) {
      if (shiny::isRunning()) {
        pb$inc(0.9, detail='Truncating signal...')
      } else {
        setProgressBar(pb, 0.9, label = 'Truncating signal...')
      }
      truncate_indices <-
        integrated_data[, 2:ncol(integrated_data)] > 0 &
          (integrated_data[, 2:ncol(integrated_data)] <=
            (1e-04 * parse_epoch_string(epoch, sr)))
      truncate_indices <- data.frame(truncate_indices)
      integrated_data[, 2:ncol(integrated_data)] <-
        sapply(1:(ncol(integrated_data) - 1), function(n) {
          integrated_data[truncate_indices[, n], n + 1] <- 0
          return(integrated_data[, n + 1])
        })
    }

    # Compute vector magnitude
    if (shiny::isRunning()) {
      pb$inc(1, detail='Summing up axial values...')
      pb$close()
    } else {
      setProgressBar(pb, 1, label = 'Summing up axial values...')
      close(pb)
    }
    row_abnormal <- rep(FALSE, nrow(integrated_data))
    for (i in 2:ncol(integrated_data))
    {
      row_abnormal <- row_abnormal | integrated_data[[i]] < 0
    }
    if (combination == "vm") {
      mims_data <-
        vector_magnitude(integrated_data, axes = axes)
    } else if (combination == "sum") {
      mims_data <- sum_up(integrated_data, axes = axes)
    } else {
      mims_data <- sum_up(integrated_data, axes = axes)
    }

    if (output_mims_per_axis) {
      mims_data <-
        cbind(mims_data, integrated_data[, 2:ncol(integrated_data)])
      colnames(mims_data)[2:ncol(mims_data)] <-
        c("MIMS_UNIT", "MIMS_UNIT_X", "MIMS_UNIT_Y", "MIMS_UNIT_Z")
      mims_data[row_abnormal, c(2, 3, 4, 5)] <- -0.01
    } else {
      colnames(mims_data)[2] <- "MIMS_UNIT"
      mims_data[row_abnormal, 2] <- -0.01
    }
    rm(row_abnormal)
    rm(integrated_data)


    # only keep data between start and end time
    keep_mask <-
      mims_data[[1]] >= start_time & mims_data[[1]] < stop_time
    mims_data <- mims_data[keep_mask, ]


    if (output_orientation_estimation) {
      return(list(mims = mims_data, orientation = orientation_data))
    } else {
      return(mims_data)
    }
  }

#' @rdname mims_unit
#'
#' @param files character vector. A list of csv filepaths for raw accelerometer
#'   data organized in order to be processed. The data should be consecutive in
#'   timestamps. A typical case is a set of hourly or daily files for
#'   continuous accelerometer sampling. For a single file, please wrap the filepath
#'   in a vector `c(filepath)`.
#' @param file_type character. "mhealth" or "actigraph". The type of the csv files
#' that store the raw accelerometer data.
#' @param ... additional parameters passed to the import function when reading
#'   in the data from the files.
#' @export
mims_unit_from_files <-
  function(files,
           epoch = "5 sec",
           dynamic_range,
           output_mims_per_axis = FALSE,
           use_gui_progress = FALSE,
           use_snapshot_to_check = FALSE,
           file_type = "mhealth", ...) {
    num_of_files <- length(files)
    dots = list(...)
    df <- NULL
    results <- list()

    if (file_type == "mhealth") {
      import_fun <- import_mhealth_csv_chunked
      header = TRUE
    } else if (file_type == "actigraph") {
      import_fun <- function(x, chunk_samples) import_actigraph_csv_chunked(x, chunk_samples = chunk_samples, ...)
      header = dots$header
    } else {
      stop('Only "mhealth" or "actigraph" file types are supported')
    }

    meta = .get_meta(files[1], file_type = file_type, header = header)
    sr = meta$sr
    st = meta$st
    num_samples_epoch = parse_epoch_string(epoch, sr)
    num_per_load = max(num_samples_epoch, 180000)
    last_epoch_st = NULL
    last_chunk = NULL
    j = 1
    if (shiny::isRunning()) {
      file_pb = shiny::Progress$new()
    }
    for (i in 1:num_of_files) {
      if (shiny::isRunning()) {
        file_pb$set(value=i/num_of_files, message=paste("Compute MIMS-unit values for", files[i]))
      }
      funcs = import_fun(files[i], chunk_samples = num_per_load)
      next_chunk = funcs[[1]]
      close_con = funcs[[2]]

      if (shiny::isRunning()) {
        chunk_pb = shiny::Progress$new()
        chunk_pb$set(value=0, message="Compute MIMS-unit values for chunks")
      }
      repeat {
        chunk = next_chunk()
        if (nrow(chunk) > 0) {
          if (shiny::isRunning()) {
            chunk_pb$inc(amount=0.01)
          }
          num_per_df = nrow(chunk)
          if (!is.null(df)) {
            if (!is.null(last_epoch_st) & !is.null(last_chunk)) {
              df = clip_data(last_chunk, start_time = last_epoch_st,
                             stop_time = last_chunk[nrow(last_chunk),1])
              df = rbind(df, chunk[1:num_per_df,])
            } else {
              df = chunk[1:num_per_df,]
            }
          } else {
            df = chunk[1:num_per_df,]
          }
          result = mims_unit(df,
                             before_df = NULL,
                             after_df = NULL,
                             epoch = epoch,
                             dynamic_range = dynamic_range,
                             output_mims_per_axis = output_mims_per_axis,
                             use_gui_progress = use_gui_progress,
                             st = st,
                             use_snapshot_to_check = use_snapshot_to_check
          )
          last_epoch_st = result[nrow(result),1]
          last_chunk = chunk
          # discard the last epoch
          result = result[1:(nrow(result) - 1),]
          results[[j]] = result
          j = j + 1
        }
        else {
          if (shiny::isRunning()) {
            chunk_pb$set(value=1)
            chunk_pb$close()
          }
          close_con()
          break
        }
      }
    }
    if (shiny::isRunning()) {
      file_pb$close()
    }
    result <- do.call(rbind, results)
    return(result)
  }


.get_meta <- function(filepath, file_type, header) {
  if (file_type == 'actigraph') {
    meta = import_actigraph_meta(filepath, header = header)
    st = meta$st
    sr = meta$sr
    return(list(st = st, sr = sr))
  } else if (file_type == 'mhealth') {
    # use the first 200 rows to compute sampling rate
    funcs = import_mhealth_csv_chunked(filepath, chunk_samples = 200)
    next_chunk = funcs[[1]]
    close_con = funcs[[2]]
    df = next_chunk()
    close_con()
    sr = sampling_rate(df)
    st = df[1, 1]
    return(list(st = st, sr = sr))
  }
}

#' Vector magnitude of multi-channel signal.
#'
#' \code{vector_magnitude} computes the vector magnitude value for each sample
#' (row) of a multi-channel signal.
#'
#' This function takes a dataframe of a multi-channel signal as input, and then
#' computes the 2-norm (vector magnitude) for each row and returns a transformed
#' dataframe with two columns.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is not used in
#'   the released version of MIMS-unit algorithm, but was used to compare the
#'   alternative \code{\link{sum_up}} method when combining MIMS-unit values on
#'   each axis into a single value.
#'
#' @param df dataframe. multi-channel signal, with the first column being the
#'   timestamp in POSXct format.
#' @param axes numerical vector. Specify the column indices for each axis. When
#'   this value is NULL, the function assumes the axes are starting from column
#'   2 to the end. Default is \code{NULL}.
#'
#' @return dataframe. The transformed dataframe will have the same number of
#'   rows as input dataframe but only two columns, with the first being
#'   timestamps and second being the vector magnitude values.
#'
#' @seealso \code{\link{sum_up}}
#' @family transformation functions
#' @export
#' @examples
#'   # Use the first 10 rows of the sample data as an example
#'   df = sample_raw_accel_data[1:10,]
#'   df
#'
#'   # By default, the function will assume columns starting from 2 to be axial
#'   # values.
#'   vector_magnitude(df)
#'
#'   # Or, you may specify the column indices yourself
#'   vector_magnitude(df, axes=c(2,3,4))
#'
#'   # Or, if you only want to consider x and y axes
#'   vector_magnitude(df, axes=c(2,3))
#'
#'   # Or, just return the chosen column
#'   vector_magnitude(df, axes=c(2))
#'
vector_magnitude <- function(df, axes = NULL) {
  n_cols <- ncol(df)
  vm_data <- df
  stamp_name <- colnames(df)[1]
  if (is.null(axes)) {
    axes = 2:n_cols
  }
  tokens <- names(df)[axes]
  label_name <-
    paste(c("MAGNITUDE", tokens), collapse = "_")
  if (length(axes) == 1) {
    vm_data[label_name] <- df[, axes]^2
  } else {
    vm_data[label_name] <- rowSums(df[, axes]^2)
  }
  vm_data[, label_name] <- sqrt(vm_data[, label_name])
  vm_data <- vm_data[, c(stamp_name, label_name)]
  return(vm_data)
}

#' Sum of multi-channel signal.
#'
#' \code{sum_up} computes the sum up value for each sample (row) of a
#' multi-channel signal.
#'
#' This function takes a dataframe of a multi-channel signal as input, and then
#' computes the sum of each row and returns a transformed dataframe with two
#' columns.
#'
#' @section How is it used in MIMS-unit algorithm?: This function is used to
#'   combine MIMS-unit values on each axis into a single value after aggregating
#'   over each epoch using \code{\link{aggregate_for_mims}}.
#'
#' @param df dataframe. multi-channel signal, with the first column being the
#'   timestamp in POSXct format.
#' @param axes numerical vector. Specify the column indices for each axis. When
#'   this value is NULL, the function assumes the axes are starting from column
#'   2 to the end. Default is \code{NULL}.
#'
#' @return dataframe. The transformed dataframe will have the same number of
#'   rows as input dataframe but only two columns, with the first being
#'   timestamps and second being the sum up values.
#'
#' @seealso \code{\link{vector_magnitude}}
#' @family transformation functions
#' @export
#' @examples
#'   # Use the first 10 rows of the sample data as an example
#'   df = sample_raw_accel_data[1:10,]
#'   df
#'
#'   # By default, the function will assume columns starting from 2 to be axial
#'   # values.
#'   sum_up(df)
#'
#'   # Or, you may specify the column indices yourself
#'   sum_up(df, axes=c(2,3,4))
#'
#'   # Or, if you only want to consider x and y axes
#'   sum_up(df, axes=c(2,3))
#'
#'   # Or, just return the chosen column
#'   sum_up(df, axes=c(2))
#'
sum_up <- function(df, axes = NULL) {
  n_cols <- ncol(df)
  stamp_name <- colnames(df)[1]
  if (is.null(axes)) {
    axes = 2:n_cols
  }
  tokens <- names(df)[axes]
  label_name <-
    paste(c("SUMUP", tokens), collapse = "_")
  if (length(axes) == 1) {
    df[label_name] <- df[, axes]
  } else {
    df[label_name] <- rowSums(df[, axes])
  }
  df <- df[, c(stamp_name, label_name)]
  return(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_data.R
\name{simulate_new_data}
\alias{simulate_new_data}
\title{Simulate new data based on the given multi-channel accelerometer data}
\usage{
simulate_new_data(old_data, new_range, new_sr)
}
\arguments{
\item{old_data}{dataframe. Input multi-channel accelerometer data.}

\item{new_range}{numerical vector. The new dynamic ranges to cut off the signal.
Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach. Default is \code{NULL},
meaning the function will do nothing but return the input data.}

\item{new_sr}{number. New sampling rate in Hz.}
}
\description{
\code{simulate_new_data} simulate new data based on the given multi-channel
accelerometer data, a new dynamic range and a new sampling rate.
}
\details{
This function simulates the data from a new device based on the signal from a
baseline device. It first changes the sampling rate using function
\code{\link{interpolate_signal}}, and then changes the dynamic range using
function \code{\link{cut_off_signal}}.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that is used to simulate new devices with different sampling rates
  and dynamic ranges during algorithm validation.
}

\examples{
  # Use sample data for testing
  df = sample_raw_accel_data

  # Show df
  illustrate_signal(df, range=c(-8, 8))

  # simulate new data by changing range and sampling rate
  new_df = simulate_new_data(df, new_range=c(-2, 2), new_sr = 30)

  # Show new df
  illustrate_signal(new_df, range=c(-2, 2))
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{cut_off_signal}()},
\code{\link{interpolate_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{sampling_rate}()},
\code{\link{segment_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_actigraph_csv}
\alias{import_actigraph_csv}
\title{Import raw multi-channel accelerometer data stored in Actigraph raw csv
format}
\usage{
import_actigraph_csv(
  filepath,
  in_voltage = FALSE,
  has_ts = TRUE,
  header = TRUE
)
}
\arguments{
\item{filepath}{string. The filepath of the input data. The first column of
the input data should always include timestamps.}

\item{in_voltage}{set as TRUE only when the input Actigraph csv file is in
analog quantized format and need to be converted into g value}

\item{has_ts}{boolean. If TRUE, the input csv file will have a timestamp column.}

\item{header}{boolean. If TRUE, the input csv file will have column names in
the first row.}
}
\value{
dataframe. The imported multi-channel accelerometer signal, with the
  first column being the timestamps in POSXlct format, and the rest columns
  being accelerometer values in \eqn{g} unit.
}
\description{
\code{import_actigraph_csv} imports the raw multi-channel accelerometer data
stored in Actigraph raw csv format. It supports files from the following
devices: GT3X, GT3X+, GT3X+BT, GT9X, and GT9X-IMU.
}
\details{
For old device (GT3X) that stores accelerometer values as digital voltage.
The function will convert the values to \eqn{g} unit using the following
equation.

\deqn{x_g = \frac{x_{voltage}r}{(2 ^ r) - \frac{v}{2}}}

Where \eqn{v} is the max voltage corresponding to the max accelerometer value
that can be found in the meta section in the csv file; \eqn{r} is the
resolution level which is the number of bits used to store the voltage
values. \eqn{r} can also be found in the meta section in the csv file.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import data from Actigraph devices during
  algorithm validation.
}

\examples{
  default_ops = options()
  options(digits.secs=3)

  # Use the sample actigraph csv file provided by the package
  filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')

  # Check file format
  readLines(filepath)[1:15]

  # Load the file with timestamp column
  df = import_actigraph_csv(filepath)

  # Check loaded file
  head(df)

  # Check more
  summary(df)

  # Use the sample actigraph csv file without timestamp
  filepath = system.file('extdata', 'actigraph_no_timestamp.csv', package='MIMSunit')

  # Check file format
  readLines(filepath)[1:15]

  # Load the file without timestamp column
  df = import_actigraph_csv(filepath, has_ts = FALSE)

  # Check loaded file
  head(df)

  # Check more
  summary(df)

  # Restore default options
  options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mims_unit.R
\name{custom_mims_unit}
\alias{custom_mims_unit}
\title{Compute both MIMS-unit and sensor orientations with custom settings}
\usage{
custom_mims_unit(
  df,
  epoch = "5 sec",
  dynamic_range,
  noise_level = 0.03,
  k = 0.05,
  spar = 0.6,
  filter_type = "butter",
  cutoffs = c(0.2, 5),
  axes = c(2, 3, 4),
  use_extrapolation = TRUE,
  use_filtering = TRUE,
  combination = "sum",
  allow_truncation = TRUE,
  output_mims_per_axis = FALSE,
  output_orientation_estimation = FALSE,
  epoch_for_orientation_estimation = NULL,
  before_df = NULL,
  after_df = NULL,
  use_gui_progress = FALSE,
  st = NULL,
  use_snapshot_to_check = FALSE
)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer signal.}

\item{epoch}{string. Any format that is acceptable by argument \code{breaks}
in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
sec", "10 min". Default is "5 sec".}

\item{dynamic_range}{numerical vector. The dynamic ranges of the input
signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach.}

\item{noise_level}{number. The tolerable noise level in \eqn{g} unit, should
be between 0 and 1. Default is 0.03, which applies to most devices.}

\item{k}{number. Duration of neighborhood to be used in local spline
regression for each side, in seconds. Default is 0.05, as optimized by
MIMS-unit algorithm.}

\item{spar}{number. Between 0 and 1, to control how smooth we want to fit
local spline regression, 0 is linear and 1 matches all local points.
Default is 0.6, as optimized by MIMS-unit algorithm.}

\item{filter_type}{string. The type of filter to be applied. Could be
'butter' for butterworth bandpass filter, 'ellip' for elliptic bandpass
filter or 'bessel' for bessel lowpass filter + average removal highpass
filter. Default is "butter".}

\item{cutoffs}{numerical vector. Cut off frequencies to be used in filtering.
If \code{filter_type} is "bessel", the cut off frequency for lowpass filter
would be multiplied by 2 when being used. Default is 0.2Hz and 5Hz.}

\item{axes}{numerical vector. Indices of columns that specifies the axis
values of the input signal. Default is \code{c(2,3,4)}.}

\item{use_extrapolation}{logical. If it is TRUE, the function will apply
extrapolation algorithm to the input signal, otherwise it will skip
extrapolation but only linearly interpolate the signal to 100Hz. Default is
TRUE.}

\item{use_filtering}{logical. If it is TRUE, the function will apply bandpass
filtering to the input signal, otherwise it will skip the filtering.
Default is TRUE.}

\item{combination}{string. Method to combine MIMS-unit values for each axis.
Could be "sum" for \code{\link{sum_up}} or "vm" for
\code{\link{vector_magnitude}}.}

\item{allow_truncation}{logical. If it is TRUE, the algorithm will truncate
very small MIMS-unit values to zero. Default is TRUE.}

\item{output_mims_per_axis}{logical. If it is TRUE, the output MIMS-unit
dataframe will have MIMS-unit values for each axis from the third column.
Default is FALSE.}

\item{output_orientation_estimation}{logical. If it is TRUE, the function
will also estimate sensor orientations over each epoch. And the output will
be a list, with the first element being the MIMS-unit dataframe, and the
second element being the sensor orientation dataframe. Default is FALSE.}

\item{epoch_for_orientation_estimation}{string. string. Any format that is
acceptable by argument \code{breaks} in method
\code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5 sec", "10
min". Default is "5 sec". It is independent from \code{epoch} for
MIMS-unit.}

\item{before_df}{dataframe. The multi-channel accelerometer signal comes
before the input signal to be prepended to the input signal during
computation. This is used to eliminate the edge effect during extrapolation
and filtering. If it is \code{NULL}, algorithm will run directly on the
input signal. Default is NULL.}

\item{after_df}{dataframe. The multi-channel accelerometer signal comes after
the input signal to be append to the input signal. This is used to
eliminate the edge effect during extrapolation and filtering. If it is
\code{NULL}, algorithm will run directly on the input signal. Default is
NULL.}

\item{use_gui_progress}{logical. If it is TRUE, show GUI progress bar on
windows platform. Default is FALSE.}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the epochs generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate epochs. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}

\item{use_snapshot_to_check}{logical. If TRUE, the function will use the first
100 rows or 10% (whichever is smaller) to check timestamp duplications. Otherwise,
the algorithm will use all data to check timestamp duplications. Default is FALSE.}
}
\value{
dataframe or list. If \code{output_orientation_estimation} is TRUE,
  the output will be a list, otherwise the output will be the MIMS-unit
  dataframe.

  The first element will be the MIMS-unit dataframe, in which the first
  column is the start time of each epoch in POSIXct format, and the second
  column is the MIMS-unit value for the input signal, and the third column
  and on are the MIMS-unit values for each axis of the input signal if
  \code{output_mims_per_axis} is TRUE.

  The second element will be the orientation dataframe, in which the first
  column is the start time of each epoch in POSIXct format, and the second to
  fourth column is the estimated orientations for the input signal.
}
\description{
\code{custom_mims_unit} computes the Monitor Independent Motion Summary unit
and estimates the sensor orientations for the input multi-channel
accelerometer signal with custom settings. The input signal can be from
devices of any sampling rate and dynamic range. Please refer to the
manuscript for detailed description of the algorithm. Please refer to
functions for the intermediate steps: \code{\link{extrapolate}} for
extrapolation, \code{\link{iir}} for filtering,
\code{\link{aggregate_for_mims}} and \code{\link{aggregate_for_orientation}}
for aggregation.
}
\note{
This function allows you to run customized algorithm for MIMSunit and
  sensor orientations.

  \code{before_df} and \code{after_df} are often set when the accelerometer
  data are divided into files of smaller chunk.
}
\section{How is it used in MIMS-unit algorithm?}{
 This is the low-level entry
  of MIMS-unit and orientation estimation algorithm. \code{\link{mims_unit}}
  calls this function internally.
}

\examples{
  # Use sample data for testing
  df = sample_raw_accel_data

  # compute mims unit values with custom parameter
  output = custom_mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), spar=0.7)
  head(output)
}
\seealso{
Other Top level API functions: 
\code{\link{mims_unit}()},
\code{\link{sensor_orientations}()},
\code{\link{shiny_app}()}
}
\concept{Top level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export_data.R
\name{export_to_actilife}
\alias{export_to_actilife}
\title{Export accelerometer data in Actilife RAW CSV format}
\usage{
export_to_actilife(
  df,
  filepath,
  actilife_version = "6.13.3",
  firmware_version = "1.6.0"
)
}
\arguments{
\item{df}{dataframe. Input accelerometer data. The first column is timestamp
in POSXlct format, and the rest columns are accelerometer values in g
(\eqn{9.81m/s^2}).}

\item{filepath}{string. The output filepath.}

\item{actilife_version}{string. The Actilife version number to be added to
the header. Default is "6.13.3", that was used by the algorithm during
development.}

\item{firmware_version}{string. The firmware version number to be added to
the header. This is supposed to be the firmware version of the Actigraph
devices. We did not see any usage of the number during the computation of
Actigraph counts by Actilife, so it may be set with an arbitrary version
code seen in any Actigraph devices. We use default version code "1.6.0".}
}
\value{
No return value.
}
\description{
\code{export_to_actilife} exports the input dataframe as a csv file that is
compatible with Actilife.
}
\details{
This function takes an input accelerometer dataframe and exports it in
Actilife RAW CSV format with a prepended a madeup header. The exported file
csv file has compatible header, column names, timestamp format with Actilife
and can be imported directly into Actilife software.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is an utility
function that was used to convert validation data into Actilife RAW CSV
format so that we can use Actilife to compute Actigraph counts values for
these data.
}

\examples{
  # Use the first 5 rows from sample data
  df = sample_raw_accel_data[1:5,]
  head(df)

  # Save to current path with default mocked actilife and firmware versions
  filepath = tempfile()
  export_to_actilife(df, filepath)

  # The saved file will have the same format as Actigraph csv files
  readLines(filepath)

  # Cleanup
  file.remove(filepath)
}
\seealso{
Other File I/O functions: 
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.R
\name{iir}
\alias{iir}
\title{Apply IIR filter to the signal}
\usage{
iir(df, sr, cutoff_freq, order = 4, type = "high", filter_type = "butter")
}
\arguments{
\item{df}{dataframe. The input multi-channel signal. The first column is
timestamps in POSXlct format. The rest columns are signal values.}

\item{sr}{number. Sampling rate in Hz of the input signal.}

\item{cutoff_freq}{number or numerical vector. The cutoff frequencies in Hz.
If the IIR filter is a bandpass or bandstop filter, it will be a 2-element
numerical vector specifying the low and high end cutoff frequencies
\code{c(low, high)}.}

\item{order}{number. The order of the filter. Default is 4.}

\item{type}{string. Filtering type, one of "low" for a low-pass filter,
"high" for a high-pass filter, "stop" for a stop-band (band-reject) filter,
or "pass" for a pass-band filter.}

\item{filter_type}{string. IIR filter type, one of "butter" for butterworth
filter, "chebyI" for Chebyshev Type I filter, or "ellip" for Elliptic filter.}
}
\value{
dataframe. Filtered signal.
}
\description{
\code{iir} function takes a multi-channel signal and applies an IIR filter to
the signal.
}
\details{
This function filters the input multi-channel signal by applying an IIR
filter. See
\href{https://en.wikipedia.org/wiki/Infinite_impulse_response}{wiki} for the
explanation of the filter. The implementations of IIR filters can be found in
\code{\link[signal]{butter}}, \code{\link[signal]{cheby1}}, and \code{\link[signal]{ellip}}.

For Chebyshev Type I, Type II and Elliptic filter, the passband ripple is
fixed to be 0.05 dB. For Elliptic filter, the stopband ripple is fixed to be
-50dB.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function has been used
  as the main filtering method in MIMS-unit algorithm. Specifically, it uses
  a 0.5 - 5 Hz bandpass butterworth filter during filtering.
}

\examples{
  # Use sample data
  df = sample_raw_accel_data

  # View input
  illustrate_signal(df, plot_maxed_out_line = FALSE)

  # Apply filtering that uses the same setting as in MIMSunit algorithm
  output = iir(df, sr=80, cutoff_freq=c(0.2, 5), type='pass')

  # View output
  illustrate_signal(output, plot_maxed_out_line = FALSE)
}
\seealso{
Other filtering functions: 
\code{\link{bandlimited_interp}()}
}
\concept{filtering functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{shiny_app}
\alias{shiny_app}
\title{Run shiny app to compute MIMSunit values from files}
\usage{
shiny_app(options = list())
}
\arguments{
\item{options}{The options passed to \code{\link[shiny]{shinyApp}}.}
}
\description{
\code{shiny_app} runs a local shiny app that provides a user friendly interface
to compute mims unit values from files stored in mhealth or actigraph format.
}
\section{How is it used in MIMS-unit algorithm?}{
 This provides a user friendly
graphical interface to load local files, call \code{\link{mims_unit_from_files}}
and display the results as an interactive graph.
}

\examples{
  \dontrun{
  shiny_app()
  }
}
\seealso{
Other Top level API functions: 
\code{\link{custom_mims_unit}()},
\code{\link{mims_unit}()},
\code{\link{sensor_orientations}()}
}
\concept{Top level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mims_unit.R
\name{mims_unit}
\alias{mims_unit}
\alias{mims_unit_from_files}
\title{Compute Monitor Independent Motion Summary unit (MIMS-unit)}
\usage{
mims_unit(
  df,
  before_df = NULL,
  after_df = NULL,
  epoch = "5 sec",
  dynamic_range,
  output_mims_per_axis = FALSE,
  use_gui_progress = FALSE,
  st = NULL,
  use_snapshot_to_check = FALSE
)

mims_unit_from_files(
  files,
  epoch = "5 sec",
  dynamic_range,
  output_mims_per_axis = FALSE,
  use_gui_progress = FALSE,
  use_snapshot_to_check = FALSE,
  file_type = "mhealth",
  ...
)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer signal. The
first column should be the time component. The accelerometer data values
 (typically starting from the second column) should be in \eqn{g} (per \eqn{9.81m/s^2}) unit.}

\item{before_df}{dataframe. The multi-channel accelerometer signal comes
before the input signal to be prepended to the input signal during
computation. This is used to eliminate the edge effect during extrapolation
and filtering. If it is \code{NULL}, algorithm will run directly on the
input signal. Default is NULL.}

\item{after_df}{dataframe. The multi-channel accelerometer signal comes after
the input signal to be append to the input signal. This is used to
eliminate the edge effect during extrapolation and filtering. If it is
\code{NULL}, algorithm will run directly on the input signal. Default is
NULL.}

\item{epoch}{string. Any format that is acceptable by argument \code{breaks}
in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
sec", "10 min". Default is "5 sec".}

\item{dynamic_range}{numerical vector. The dynamic ranges of the input
signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach.}

\item{output_mims_per_axis}{logical. If it is TRUE, the output MIMS-unit
dataframe will have MIMS-unit values for each axis from the third column.
Default is FALSE.}

\item{use_gui_progress}{logical. If it is TRUE, show GUI progress bar on
windows platform. Default is FALSE.}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the epochs generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate epochs. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}

\item{use_snapshot_to_check}{logical. If TRUE, the function will use the first
100 rows or 10% (whichever is smaller) to check timestamp duplications. Otherwise,
the algorithm will use all data to check timestamp duplications. Default is FALSE.}

\item{files}{character vector. A list of csv filepaths for raw accelerometer
data organized in order to be processed. The data should be consecutive in
timestamps. A typical case is a set of hourly or daily files for
continuous accelerometer sampling. For a single file, please wrap the filepath
in a vector `c(filepath)`.}

\item{file_type}{character. "mhealth" or "actigraph". The type of the csv files
that store the raw accelerometer data.}

\item{...}{additional parameters passed to the import function when reading
in the data from the files.}
}
\value{
dataframe. The MIMS-unit dataframe. The first column is the start
  time of each epoch in POSIXct format. The second column is the MIMS-unit
  value for the input signal. If \code{output_mims_per_axis} is TRUE, the
  third column and then are the MIMS-unit values for each axis of the input
  signal.
}
\description{
\code{mims_unit} computes the Monitor Independent Motion Summary unit for the
input multi-channel accelerometer signal. The input signal can be from
devices of any sampling rate and dynamic range. Please refer to the
manuscript for detailed description of the algorithm. Please refer to
functions for the intermediate steps: \code{\link{extrapolate}} for
extrapolation, \code{\link{iir}} for filtering,
\code{\link{aggregate_for_mims}} for aggregation.
}
\note{
This function is a wrapper function for the low-level
  \code{\link{custom_mims_unit}} function. It has set internal parameters as
  described in the manuscript. If you want to run customized algorithm for
  MIMSunit or if you want to develop better algorithms based on MIMS-unit
  algorithm, please use function \code{\link{custom_mims_unit}} where all
  parameters are tunable.

  \code{before_df} and \code{after_df} are often set when the accelerometer
  data are divided into files of smaller chunk.

  Please make sure the input data do not contain duplicated timestamps. See
   more information about this \href{https://github.com/mHealthGroup/MIMSunit/issues/32}{issue}.
   Otherwise the computation will stop.
}
\section{How is it used in MIMS-unit algorithm?}{
 This is the main entry of
  MIMS-unit algorithm.
}

\examples{
  # Use sample data for testing
  df = head(sample_raw_accel_data, 240)

  # compute mims unit values and output axial values
  output = mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), output_mims_per_axis=TRUE)
  head(output)
}
\seealso{
Other Top level API functions: 
\code{\link{custom_mims_unit}()},
\code{\link{sensor_orientations}()},
\code{\link{shiny_app}()}
}
\concept{Top level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_actigraph_csv_chunked}
\alias{import_actigraph_csv_chunked}
\title{Import large raw multi-channel accelerometer data stored in Actigraph raw csv
format in chunks}
\usage{
import_actigraph_csv_chunked(
  filepath,
  in_voltage = FALSE,
  header = TRUE,
  has_ts = TRUE,
  chunk_samples = 180000
)
}
\arguments{
\item{filepath}{string. The filepath of the input data.The first column of
the input data should always include timestamps.}

\item{in_voltage}{set as TRUE only when the input Actigraph csv file is in
analog quantized format and need to be converted into g value}

\item{header}{boolean. If TRUE, the input csv file will have column names in
the first row.}

\item{has_ts}{boolean. If TRUE, the input csv file should have a timestamp
column at first.}

\item{chunk_samples}{number. The number of samples in each chunk. Default is
180000.}
}
\value{
list. The list contains two items. The first item is a generator
  function that each time it is called, it will return a data.frame of the
  imported chunk. The second item is a \code{close} function which you can
  call at any moment to close the file loading.
}
\description{
\code{import_actigraph_csv_chunked} imports the raw multi-channel accelerometer data
stored in Actigraph raw csv format. It supports files from the following
devices: GT3X, GT3X+, GT3X+BT, GT9X, and GT9X-IMU.
}
\details{
For old device (GT3X) that stores accelerometer values as digital voltage.
The function will convert the values to \eqn{g} unit using the following
equation.

\deqn{x_g = \frac{x_{voltage}r}{(2 ^ r) - \frac{v}{2}}}

Where \eqn{v} is the max voltage corresponding to the max accelerometer value
that can be found in the meta section in the csv file; \eqn{r} is the
resolution level which is the number of bits used to store the voltage
values. \eqn{r} can also be found in the meta section in the csv file.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import data from Actigraph devices during
  algorithm validation.
}

\examples{
  default_ops = options()
  options(digits.secs=3)

  # Use the actigraph csv file shipped with the package
  filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')

  # Check original file format
  readLines(filepath)[1:15]

  # Example 1: Load chunks every 2000 samples
  results = import_actigraph_csv_chunked(filepath, chunk_samples=2000)
  next_chunk = results[[1]]
  close_connection = results[[2]]
  # Check data as chunks, you can see chunks are shifted at each iteration.
  n = 1
  repeat {
    df = next_chunk()
    if (nrow(df) > 0) {
      print(paste('chunk', n))
      print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
      n = n + 1
    }
    else {
      break
    }
  }

  # Close connection after reading all the data
  close_connection()

  # Example 2: Close loading early
  results = import_actigraph_csv_chunked(filepath, chunk_samples=2000)
  next_chunk = results[[1]]
  close_connection = results[[2]]
  # Check data as chunks, you can see chunk time is shifting forward at each iteration.
  n = 1
  repeat {
    df = next_chunk()
    if (nrow(df) > 0) {
      print(paste('chunk', n))
      print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
      n = n + 1
      close_connection()
    }
    else {
      break
    }
  }

  # Restore default options
  options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_activpal3_csv}
\alias{import_activpal3_csv}
\title{Import raw multi-channel accelerometer data stored in ActivPal3 csv format}
\usage{
import_activpal3_csv(filepath, header = FALSE)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}

\item{header}{boolean. If TRUE, the input csv file will have column names in
the first row.}
}
\value{
dataframe. The imported multi-channel accelerometer signal, with the
  first column being the timestamps in POSXlct format, and the rest columns
  being accelerometer values in \eqn{g} unit.
}
\description{
\code{import_activpal3_csv} imports the raw multi-channel accelerometer data
stored in ActivPal3 csv format by converting the accelerometer values (in
digital voltage values) to \eqn{g} unit.
}
\details{
ActivPal 3 sensors have known dynamic range to be \eqn{(-2g, +2g)}. And the
sensor stores values using 8-bit memory storage. So, the digital voltage
values may be converted to \eqn{g} unit using following equation.

\deqn{x_g = \frac{x_{voltage} - 127}{2^8} * 4}
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import data from ActivPal3 devices during
  algorithm validation.
}

\examples{
  default_ops = options()
  options(digits.secs=3)
  # Use the sample activpal3 csv file provided by the package
  filepath = system.file('extdata', 'activpal3.csv', package='MIMSunit')

  # Check the csv format
  readLines(filepath)[1:5]

  # Load the file, in our case without header
  df = import_activpal3_csv(filepath, header=FALSE)

  # Check loaded file
  head(df)

  # Check more
  summary(df)

  # Restore default options
  options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{sampling_rate}
\alias{sampling_rate}
\title{Estimate sampling rate for multi-channel signal}
\usage{
sampling_rate(df)
}
\arguments{
\item{df}{dataframe. Input dataframe of the multi-channel signal. The first
column is the timestamps in POSXlct format and the following columns are
accelerometer values.}
}
\value{
number. The estimated sampling rate in Hz.
}
\description{
\code{sampling_rate} estimates the sampling rate based on the average time
intervals between adjacent samples for the input multi-channel signal.
}
\details{
This function accepts a dataframe of multi-channel signal, computes the
duration of the sequence, and gets the sampling rate by dividing the number
of samples by it.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that was used in various part in the algorithm whenever we need to
  know the sampling rate.
}

\examples{
# Get the test data
df = sample_raw_accel_data

# Default sampling rate is 80Hz
sampling_rate(df)

# Downsample to 30Hz
output = bandlimited_interp(df, 80, 30)
sampling_rate(output)

# Upsampling to 100Hz
output = bandlimited_interp(df, 80, 100)
sampling_rate(output)
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{cut_off_signal}()},
\code{\link{interpolate_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{segment_data}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_data.R
\name{cut_off_signal}
\alias{cut_off_signal}
\title{Cut off input multi-channel signal according to a new dynamic range}
\usage{
cut_off_signal(df, range = NULL, noise_std = 0.03)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer data.}

\item{range}{numerical vector. The new dynamic ranges to cut off the signal.
Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach. Default is \code{NULL},
meaning the function will do nothing but return the input data.}

\item{noise_std}{number. The standard deviation of the added gaussian noise.}
}
\value{
dataframe. The multi-channel accelerometer data with the new dynamic
  range as specified in \code{range}.
}
\description{
\code{cut_off_signal} cuts off the input multi-channel accelerometer data
according to a new dynamic range, then adds gausian noise to the cut-off
samples.
}
\details{
This function simulates the behavior that a low dynamic range device is
trying to record high intensity movement, where recorded accelerometer signal
will be cut off at the dynamic range, but the true movement should have
higher acceleration values than the dynamic range. This function also adds
gaussian noise to the cut off samples to better simulate the real world
situation.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that is used to simulate the behaviors of low dynamic range
  devices during algorithm validation.
}

\examples{
  # Use sample data for testing
  df = sample_raw_accel_data

  # Show df
  illustrate_signal(df, range=c(-8, 8))

  # cut off the signal to c(-2, 2)
  new_df = cut_off_signal(df, range=c(-2, 2), noise_std=0.03)

  # Show new df
  illustrate_signal(new_df, range=c(-2, 2))
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{interpolate_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{sampling_rate}()},
\code{\link{segment_data}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mims_unit.R
\name{sensor_orientations}
\alias{sensor_orientations}
\title{Estimates sensor orientation}
\usage{
sensor_orientations(
  df,
  before_df = NULL,
  after_df = NULL,
  epoch = "5 sec",
  dynamic_range,
  st = NULL
)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer signal.}

\item{before_df}{dataframe. The multi-channel accelerometer signal comes
before the input signal to be prepended to the input signal during
computation. This is used to eliminate the edge effect during extrapolation
and filtering. If it is \code{NULL}, algorithm will run directly on the
input signal. Default is NULL.}

\item{after_df}{dataframe. The multi-channel accelerometer signal comes after
the input signal to be append to the input signal. This is used to
eliminate the edge effect during extrapolation and filtering. If it is
\code{NULL}, algorithm will run directly on the input signal. Default is
NULL.}

\item{epoch}{string. Any format that is acceptable by argument \code{breaks}
in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
sec", "10 min". Default is "5 sec".}

\item{dynamic_range}{numerical vector. The dynamic ranges of the input
signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach.}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the epochs generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate epochs. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}
}
\value{
dataframe. The orientation dataframe. The first column is the start
  time of each epoch in POSIXct format. The second to fourth columns are the
  orientation angles.
}
\description{
\code{sensor_orientations} estimates the orientation angles for the input
multi-channel accelerometer signal. The input signal can be from devices of
any sampling rate and dynamic range. Please refer to function
\code{\link{compute_orientation}} for the implementation of the estimation
algorithm.
}
\note{
This function interpolates and extrapolates the signal before
  estimating the orientation angles.

  \code{before_df} and \code{after_df} are often set when the accelerometer
  data are divided into files of smaller chunk.
}
\section{How is it used in MIMS-unit algorithm?}{
 This is not included in the
  official MIMS-unit algorithm nor the manuscript, but we found it is useful
  to know the sensor orientations in addition to the summary of movement.
}

\examples{
  # Use sample data for testing
  df = head(sample_raw_accel_data, 240)

  # compute sensor orientation angles
  output = sensor_orientations(df, epoch = '1 sec', dynamic_range=c(-8, 8))
  head(output)
}
\seealso{
Other Top level API functions: 
\code{\link{custom_mims_unit}()},
\code{\link{mims_unit}()},
\code{\link{shiny_app}()}
}
\concept{Top level API functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{clip_data}
\alias{clip_data}
\title{Clip dataframe to the given start and stop time}
\usage{
clip_data(df, start_time, stop_time)
}
\arguments{
\item{df}{dataframe. Input dataframe of the multi-channel signal. The first
column is the timestamps in POSXlct format and the following columns are
accelerometer values.}

\item{start_time}{POSXlct format or character. Start time for clipping.
If it is a character, it should be recognizable by as.POSXlct function.}

\item{stop_time}{POSXlct format or character. Stop time for clipping.
If it is a character, it should be recognizable by as.POSXlct function.}
}
\value{
dataframe. The same format as the input dataframe.
}
\description{
\code{clip_data} clips the input sensor dataframe according to the given
start and stop time
}
\details{
This function accepts a dataframe of multi-channel signal, clips it
according to the start_time and stop_time.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that was used in various part in the algorithm whenever we need to
  clip a dataframe.
}

\examples{
  default_ops = options()
  options(digits.secs=3)
  # Use the provided sample data
  df = sample_raw_accel_data

  # Check the start time and stop time of the dataset
  summary(df)

  # Use timestamp string to clip 1 second data
  start_time = "2016-01-15 11:01:00"
  stop_time = "2016-01-15 11:01:01"
  output = clip_data(df, start_time, stop_time)
  summary(output)

  # Use POSIXct timestamp to clip data
  start_time = as.POSIXct("2016-01-15 11:01:00")
  stop_time = as.POSIXct("2016-01-15 11:01:01")
  output = clip_data(df, start_time, stop_time)
  summary(output)

  # If start and stop time is not in the range of the input data
  # return empty data.frame
  start_time = "2016-01-15 12:01:00"
  stop_time = "2016-01-15 12:01:01"
  output = clip_data(df, start_time, stop_time)
  output

  # Restore original options
  options(default_ops)
}
\seealso{
Other utility functions: 
\code{\link{cut_off_signal}()},
\code{\link{interpolate_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{sampling_rate}()},
\code{\link{segment_data}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_mhealth_csv_chunked}
\alias{import_mhealth_csv_chunked}
\title{Import large raw multi-channel accelerometer data stored in mHealth Specification
in chunks.}
\usage{
import_mhealth_csv_chunked(filepath, chunk_samples = 180000)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}

\item{chunk_samples}{number. The number of samples in each chunk. Default is
180000, which is half hour data for 100 Hz sampling rate.}
}
\value{
list. The list contains two items. The first item is a generator
  function that each time it is called, it will
  return a dataframe with at most \code{chunk_samples} samples of imported data.
  The third item is a \code{close_connection} function which you can call at
  any moment to close the file loading.
}
\description{
\code{import_mhealth_csv_chunked} imports the raw multi-channel accelerometer
  data stored in mHealth Specification in chunks.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import data stored in mHealth Specification during
  algorithm validation.
}

\examples{
  default_ops = options()
  options(digits.secs=3)

  # Use the mhealth csv file shipped with the package
  filepath = system.file('extdata', 'mhealth.csv', package='MIMSunit')

  # Example 1
  # Load chunks every 1000 samples
  results = import_mhealth_csv_chunked(filepath, chunk_samples=100)
  next_chunk = results[[1]]
  close_connection = results[[2]]
  # Check data as chunks, you can see chunk time is shifting forward at each iteration.
  n = 1
  repeat {
    df = next_chunk()
    if (nrow(df) > 0) {
      print(paste('chunk', n))
      print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
      n = n + 1
    } else {
      break
    }
  }

  # Close connection after reading all the data
  close_connection()

  # Example 2: close loading early
  results = import_mhealth_csv_chunked(filepath, chunk_samples=1000)
  next_chunk = results[[1]]
  close_connection = results[[2]]
  # Check data as chunks, you can see chunk time is shifting forward at each iteration.
  n = 1
  repeat {
    df = next_chunk()
    if (nrow(df) > 0) {
      print(paste('chunk', n))
      print(paste("df:", df[1, 1], '-', df[nrow(df),1]))
      n = n + 1
      close_connection()
    }
    else {
      break
    }
  }

 # Restore default options
 options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrapolate.R
\name{extrapolate}
\alias{extrapolate}
\alias{extrapolate_single_col}
\title{Extrapolate input multi-channel accelerometer data}
\usage{
extrapolate(df, ...)

extrapolate_single_col(
  t,
  value,
  range,
  noise_level = 0.03,
  k = 0.05,
  spar = 0.6
)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer data. Used in
\code{\link{extrapolate}}.  The first column should be the date/time}

\item{...}{see following parameter list.}

\item{t}{POSIXct or numeric vector. Input index or timestamp sequence Used in
\code{\link{extrapolate_single_col}}.}

\item{value}{numeric vector. Value vector used in
\code{\link{extrapolate_single_col}}.}

\item{range}{numeric vector. The dynamic ranges of the input signal. Should
be a 2-element numeric vector. \code{c(low, high)}, where \code{low} is
the negative max value the device can reach and \code{high} is the positive
max value the device can reach.}

\item{noise_level}{number. The tolerable noise level in \eqn{g} unit, should
be between 0 and 1. Default is 0.03, which applies to most devices.}

\item{k}{number. Duration of neighborhood to be used in local spline
regression for each side, in seconds. Default is 0.05, as optimized by
MIMS-unit algorithm.}

\item{spar}{number. Between 0 and 1, to control how smooth we want to fit
local spline regression, 0 is linear and 1 matches all local points.
Default is 0.6, as optimized by MIMS-unit algorithm.}
}
\value{
\code{extraplate} returns a dataframe with extrapolated multi-channel
  signal. \code{extrapolate_single_col} returns a dataframe with extrapolated
  single-channel signal, the timestamp col is in numeric values instead of
  POSIXct format.
}
\description{
\code{extrapolate} applies the extrapolation algorithm to a multi-channel
accelerometer data, trying to reconstruct the true movement from the
maxed-out samples.
}
\details{
This function first linearly interpolates the input signal to 100Hz, and then
applies the extrapolation algorithm (see the manuscript) to recover the
maxed-out samples. Maxed-out samples are samples that are cut off because the
intensity of the underlying movement exceeds the dynamic range of the device.

\code{extrapolate} processes a dataframe of a multi-channel accelerometer
signal. \code{extrapolate_single_col} processes a single-channel signal with
its timestamps and values specified in the first and second arguments.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is the first
  step during MIMS-unit algorithm, applied before filtering.
}

\examples{
  # Use the maxed-out data for the conceptual diagram
  df = conceptual_diagram_data[
          conceptual_diagram_data['GRANGE'] == 4,
          c("HEADER_TIME_STAMP", "X")]

  # Plot input
  illustrate_signal(df, range=c(-4, 4))

  # Use the default parameter settings as in MIMunit algorithms
  # The dynamic range of the input data is -4g to 4g.
  output = extrapolate(df, range=c(-4, 4))

  # Plot output
  illustrate_signal(output, range=c(-4, 4))
}
\seealso{
Other extrapolation related functions: 
\code{\link{extrapolate_rate}()}
}
\concept{extrapolation related functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cv_different_algorithms}
\alias{cv_different_algorithms}
\title{Coefficient of variation values for different acceleration data summary
algorithms}
\format{
A data frame with 30 rows and 3 variables:
\describe{
  \item{TYPE}{Accelerometer summary algorithm name, in character}
  \item{HZ}{The frequency of the elliptical shaker, in number}
  \item{COEFF_OF_VARIATION}{The coefficient of variation values, in number}
}
}
\source{
\url{https://github.com/mHealthGroup/MIMSunit-dataset-shaker/}
}
\usage{
cv_different_algorithms
}
\description{
A dataset containing the coefficient of variation values at different
frequencies for the dataset that includes accelerometer measures of different
devices on a standard elliptical shaker.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_enmo_csv}
\alias{import_enmo_csv}
\title{Import ENMO data stored in csv csv}
\usage{
import_enmo_csv(filepath, enmo_col = 2)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}

\item{enmo_col}{number. The index of column of ENMO values in the csv file.}
}
\value{
dataframe. The imported ENMO data, with the first column being the
  timestamps in POSIXct format, and the second column being the ENMO values.
  Column names: \code{HEADER_TIME_STAMP}, \code{ENMO}.
}
\description{
\code{import_enmo_csv} imports ENMO data stored in a summary csv format,
which was exported by the
\href{https://github.com/OxWearables/biobankAccelerometerAnalysis}{biobank
data analysis tools}.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import ENMO data from activity monitor devices
  during algorithm validation.
}

\examples{
  # Use the enmo csv file shipped with the package
  filepath = system.file('extdata', 'enmo.csv', package='MIMSunit')

  # Check original data format
  readLines(filepath)[1:5]

  # Load file, default column for enmo values are 2
  output = import_enmo_csv(filepath, enmo_col=2)

  # Check output
  head(output)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_actigraph_meta}
\alias{import_actigraph_meta}
\title{Import The meta information stored in Actigraph RAW or summary csv file.}
\usage{
import_actigraph_meta(filepath, header = TRUE)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}

\item{header}{logical. Whether the Actigraph RAW or summary csv file includes
column names. Default is TRUE.}
}
\value{
list. A list of Actigraph device meta information.
}
\description{
\code{import_actigraph_meta} imports meta information stored in the Actigraph
summary csv file.
}
\details{
The returned meta information includes following fields.

\itemize{ \item sr: Sampling rate in Hz. \item fw: Firmware version. For
example "1.7.0". \item sw: Software version of Actilife. For example "6.13.0".
\item sn: Serial number of the device. \item st: Start time of the data, in
POSIXct format. \item dt: Download time of the data, in POSIXct format. \item
at: Type of the device. Could be "MAT","CLE", "MOS" or "TAS", corresponding
to different Actigraph devices. \item imu: Whether the file is about
Actigraph GT9X IMU data. \item gr: The dynamic range in \eqn{g} unit. \item
vs: The voltage level of the device, may be used in AD conversion. See
\code{\link{import_actigraph_csv}}. \item res: The resolution or the number of
bits used to store quantized voltage values of the device, may be used in AD
conversion. See \code{\link{import_actigraph_csv}}. }
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to get related meta information such as sampling
  rate, firmware version from Actigraph devices.
}

\examples{
  default_ops = options()
  options(digits.secs=3)

  # Use the sample actigraph csv file provided by the package
  filepath = system.file('extdata', 'actigraph_timestamped.csv', package='MIMSunit')

  # Check file format
  readLines(filepath)[1:15]

  # Load the meta headers of input file
  import_actigraph_meta(filepath, header=TRUE)

  # Restore default options
  options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{conceptual_diagram_data}
\alias{conceptual_diagram_data}
\title{The input accelerometer data used to generate the conceptual diagram
(Figure 1) in the manuscript.}
\format{
A data frame with 1704 rows and 5 variables:
\describe{
  \item{HEADER_TIME_STAMP}{The timestamp of raw accelerometer data, in POSIXct}
  \item{X}{The x axis value of raw accelerometer data, in number}
  \item{GRANGE}{The dynamic range of the simulated device in g, in number}
  \item{SR}{The sampling rate in Hz of the simulated device, in number}
  \item{NAME}{An alternative name that is friendly for plotting for different devices, in character}
}
}
\source{
\url{https://github.com/mHealthGroup/MIMSunit/}
}
\usage{
conceptual_diagram_data
}
\description{
The dataset includes accelerometer data from four devices. Device 0 is a real
 Actigraph GT9X device configured at 80Hz and 8g. Device 1 to 3 are simulated
 data from the data of device 0 using function \code{\link{simulate_new_data}}.
 Data for device 0 is a random selected nondominant wrist data from a
 participant doing Jumping jack. The data is manipulated to insert an
 artificial impulse to demonstrate the effect of the MIMS-unit algorithm when
 dealing on it.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{edge_case}
\alias{edge_case}
\title{A short snippet of raw accelerometer signal from a device that has ending data maxed out.}
\format{
A data frame with 20001 rows and 4 variables:
\describe{
  \item{HEADER_TIME_STAMP}{The timestamp of raw accelerometer data, in POSIXct}
  \item{X}{The x axis value of raw accelerometer data, in number}
  \item{Y}{The x axis value of raw accelerometer data, in number}
  \item{Z}{The x axis value of raw accelerometer data, in number}
}
}
\source{
\url{https://github.com/mHealthGroup/MIMSunit/}
}
\usage{
edge_case
}
\description{
The dataset includes accelerometer data sampled at 80Hz and 6g. This data is used to test the edge case.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_over_epoch.R
\name{aggregate_for_orientation}
\alias{aggregate_for_orientation}
\title{Aggregate over epoch to get estimated accelerometer orientation.}
\usage{
aggregate_for_orientation(
  df,
  epoch,
  estimation_window = 2,
  unit = "deg",
  st = NULL
)
}
\arguments{
\item{df}{dataframe. Input accelerometer data in mhealth format. First column
should be timestamps in POSIXt format.}

\item{epoch}{string. Any format that is acceptable by argument \code{breaks}
in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
secs", "10 mins".}

\item{estimation_window}{number. Duration in seconds to be used to estimate
orientation within each epoch. Default is 2 (seconds), as suggested by
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003}.}

\item{unit}{string. The unit of orientation angles. Can be "deg" (degree) or
"rad" (radian). Default is "deg".}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the epochs generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate epochs. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}
}
\value{
dataframe. The returned dataframe will have the same format as input
  dataframe.
}
\description{
\code{aggregate_for_orientation} returns a dataframe with accelerometer
orientations estimated by
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003} over each epoch (see \code{\link{compute_orientation}}). The epoch
start time will be used as timestamp in the first column.
}
\details{
This function accepts a dataframe (in mhealth accelerometer data format) and
computes the estimated acclerometer orientations (in x, y, and z angles) over
each fixed epoch. The returned dataframe will have the same format as input
dataframe, including four columns, and have the same datetime format as input
dataframe in the timestamp column. The orientation estimation method used in
the function is based on
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003}.
}
\note{
If \code{epoch} argument is not provided or is \code{NULL}, the
  function will treat the input dataframe as a single epoch.

  If the number of samples in an epoch is less than 90%, the output
  would be NaN (invalid) for this epoch.
}
\section{How is it used in mims-unit algorithm?}{
 This function is used in
  mims-unit algorithm after extrapolation (\code{\link{extrapolate}}). The
  extrapolated signal will be estimated to get orientation angles using this
  function.
}

\examples{
  # Use sample input data
  df = sample_raw_accel_data
  head(df)

  # set epoch to 1 second and unit to degree
  # last epoch does not have enough samples to estimate orientation angles.
  aggregate_for_orientation(df, epoch='1 sec', unit='deg')

  # set epoch to 2 seconds and unit to radian
  # last epoch does not have enough samples to estimate orientation angles.
  aggregate_for_orientation(df, epoch='2 sec', unit='rad')

  # epoch set to 2 seconds, and st set to be 1 second before the start time of the data
  # so the first segment will only include data for 1 second, therefore the resulted
  # aggregated value for the first segment will be -1 (invalid) because the
  # samples are not enough. And the second segment starts from 11:00:01, instead
  # of 11:00:01 as shown in prior example,
  aggregate_for_orientation(df, epoch = '1 sec', unit='rad', st=df[1,1] - 1)

}
\seealso{
\code{\link{aggregate_for_mims}} for aggregating to get integrated
  values for each axis for each epoch.

Other aggregate functions: 
\code{\link{aggregate_for_mims}()}
}
\concept{aggregate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combine_axes.R
\name{vector_magnitude}
\alias{vector_magnitude}
\title{Vector magnitude of multi-channel signal.}
\usage{
vector_magnitude(df, axes = NULL)
}
\arguments{
\item{df}{dataframe. multi-channel signal, with the first column being the
timestamp in POSXct format.}

\item{axes}{numerical vector. Specify the column indices for each axis. When
this value is NULL, the function assumes the axes are starting from column
2 to the end. Default is \code{NULL}.}
}
\value{
dataframe. The transformed dataframe will have the same number of
  rows as input dataframe but only two columns, with the first being
  timestamps and second being the vector magnitude values.
}
\description{
\code{vector_magnitude} computes the vector magnitude value for each sample
(row) of a multi-channel signal.
}
\details{
This function takes a dataframe of a multi-channel signal as input, and then
computes the 2-norm (vector magnitude) for each row and returns a transformed
dataframe with two columns.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is not used in
  the released version of MIMS-unit algorithm, but was used to compare the
  alternative \code{\link{sum_up}} method when combining MIMS-unit values on
  each axis into a single value.
}

\examples{
  # Use the first 10 rows of the sample data as an example
  df = sample_raw_accel_data[1:10,]
  df

  # By default, the function will assume columns starting from 2 to be axial
  # values.
  vector_magnitude(df)

  # Or, you may specify the column indices yourself
  vector_magnitude(df, axes=c(2,3,4))

  # Or, if you only want to consider x and y axes
  vector_magnitude(df, axes=c(2,3))

  # Or, just return the chosen column
  vector_magnitude(df, axes=c(2))

}
\seealso{
\code{\link{sum_up}}

Other transformation functions: 
\code{\link{compute_orientation}()},
\code{\link{sum_up}()}
}
\concept{transformation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sample_raw_accel_data}
\alias{sample_raw_accel_data}
\title{Sample raw accelerometer data}
\format{
A data frame with 480 rows and 4 variables:
\describe{
  \item{HEADER_TIME_STAMP}{Timestamp, in POSIXct}
  \item{X}{X axis values, in number}
  \item{Y}{Y axis values, in number}
  \item{Z}{Z axis values, in number}
}
}
\source{
\url{https://github.com/mHealthGroup/MIMSunit/}
}
\usage{
sample_raw_accel_data
}
\description{
A raw accelerometer data file contains treadmill data collected from a human
subject.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orientation.R
\name{compute_orientation}
\alias{compute_orientation}
\title{Estimate the accelerometer orientation}
\usage{
compute_orientation(df, estimation_window = 2, unit = "deg")
}
\arguments{
\item{df}{dataframe. Input multi-channel signal. First column should be
timestamps in POSIXt format.}

\item{estimation_window}{number. window size in seconds to be used to
estimate orientations. Default is 2 (seconds), as suggested by
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003}.}

\item{unit}{string. The unit of orientation angles. Can be "deg" (degree) or
"rad" (radian). Default is "deg".}
}
\value{
dataframe. The returned dataframe will have the same format as input
  dataframe.
}
\description{
\code{compute_orientation} returns a dataframe with accelerometer
orientations estimated by
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003} for the input dataframe.
}
\details{
This function accepts a dataframe (in mhealth accelerometer data format) and
computes the estimated acclerometer orientations (in x, y, and z angles) for
every \code{estimation_window} seconds of the entire sequence, and outputs
the mean of these angles. The returned dataframe will have the same format as
input dataframe, including four columns, and have the same datetime format as
input dataframe in the timestamp column. The orientation estimation method
used in the function is based on
\href{https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1241424}{Mizell,
2003}.
}
\section{How is it used in mims-unit algorithm?}{
 This function is used in
  function (\code{\link{aggregate_for_orientation}}).
}

\examples{
  # Use first 10 second sample data for testing
  df = sample_raw_accel_data
  df = clip_data(df, start_time = df[1,1], stop_time = df[1, 1] + 600)

  # compute orientation angles in degrees
  compute_orientation(df)

  # compute orientation angles in radian angles
  compute_orientation(df, unit='rad')
}
\seealso{
Other transformation functions: 
\code{\link{sum_up}()},
\code{\link{vector_magnitude}()}
}
\concept{transformation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{illustrate_signal}
\alias{illustrate_signal}
\title{Plot given raw signal in illustration diagram style.}
\usage{
illustrate_signal(
  data,
  point_size = 0.3,
  plot_point = TRUE,
  line_size = 0.3,
  plot_line = TRUE,
  range = c(-2, 2),
  plot_maxed_out_line = TRUE,
  plot_origin = TRUE,
  title = NULL,
  plot_title = TRUE
)
}
\arguments{
\item{data}{data.frame. The input uniaxial signal. First column should be timestamp.}

\item{point_size}{number. The size of the plotted data point.}

\item{plot_point}{Bool. Plot signal as points if TRUE.}

\item{line_size}{number. The line width of the plotted signal curve.}

\item{plot_line}{Bool. Plot signal with curve if TRUE.}

\item{range}{vector. Dynamic range of the signal.}

\item{plot_maxed_out_line}{Bool. Plot dynamic range lines if TRUE. Dynamic range is set by `range`.}

\item{plot_origin}{Bool. Plot the 0 horizontal line if TRUE.}

\item{title}{Char. The title of the plot.}

\item{plot_title}{Bool. Plot title if TRUE.}
}
\value{
ggplot2 graph object. The graph to be shown.
}
\description{
\code{illustrate_signal} plots the given uniaxial signal in illustration diagram style.
  Illustration diagram style hides axes markers, unnecessary guidelines.
}
\examples{
  # Use sample data for testing
  df = sample_raw_accel_data

  # Plot it with default settings
  illustrate_signal(df)

  # Plot with a different style
  illustrate_signal(df, point_size=1, line_size=1)

  # Turn off annotation lines
  illustrate_signal(df, plot_maxed_out_line = FALSE, plot_origin = FALSE)

  # Use title
  illustrate_signal(df, plot_title=TRUE, title = "This is a title")
}
\seealso{
Other visualization functions.: 
\code{\link{generate_interactive_plot}()},
\code{\link{illustrate_extrapolation}()}
}
\concept{visualization functions.}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrapolate_rate.R
\name{extrapolate_rate}
\alias{extrapolate_rate}
\title{Get extrapolation rate.}
\usage{
extrapolate_rate(test_df, true_df, extrap_df)
}
\arguments{
\item{test_df}{dataframe. See details for the input format.}

\item{true_df}{dataframe. See details for the input format.}

\item{extrap_df}{dataframe. See details for the input format.}
}
\value{
number. The extrapolation rate value in double format. If
  extrapolation rate is 1, it means the extrapolated signal recovers as the
  true signal. If extrapolation rate is between 0 and 1, it means the
  extrapolation helps reducing the errors caused by signal maxing out. If
  extrapolation rate is smaller than 0, it means the extrapolation increases
  the errors caused by signal maxing out (during over extrapolation).
}
\description{
\code{extrapolate_rate} computes the extrapolation rate given the test signal
(maxed out), the true complete signal (no maxed out) and the extrapolated
signal.
}
\details{
All three input dataframes will have the same format, with the first column
being timestamps in POSXlct format, and the following columns being
acceleration values in g.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is used to
  compute extrapolation rate during extrapolation parameter optimization. You
  may see results in Figure 2 of the manuscript.
}

\examples{
  # Prepare data for test, ground truth
  test_df = conceptual_diagram_data[
                conceptual_diagram_data['GRANGE'] == 4,
                c("HEADER_TIME_STAMP", "X")]
  true_df = conceptual_diagram_data[
                conceptual_diagram_data['GRANGE'] == 8,
                c("HEADER_TIME_STAMP", "X")]

  # Do extrapolation
  extrap_df = extrapolate(test_df, range=c(-4, 4))

  # Compute extrapolation rate
  extrapolate_rate(test_df, true_df, extrap_df)
}
\seealso{
Other extrapolation related functions: 
\code{\link{extrapolate}()}
}
\concept{extrapolation related functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{parse_epoch_string}
\alias{parse_epoch_string}
\title{Parse epoch string to the corresponding number of samples it represents.}
\usage{
parse_epoch_string(epoch_str, sr)
}
\arguments{
\item{epoch_str}{string. The input epoch str as accepted by \code{breaks}
argument of \code{\link[base]{cut.POSIXt}}.}

\item{sr}{number. The sampling rate in Hz used to parse the epoch string.}
}
\value{
number. The number of samples represented by the epoch string.
}
\description{
\code{parse_epoch_string} parses the epoch string (e.g. "1 min"), and outputs
the corresponding number of samples it represents.
}
\details{
This function parses the given epoch string (e.g. "5 secs") and outputs the
corresponding number of samples represented by the epoch string.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is used in
  \code{\link{aggregate_for_mims}} function and \code{\link{mims_unit}} function.
}

\examples{
# 1 min with 80 Hz = 4800 samples
parse_epoch_string('1 min', sr=80)

# 30 sec with 30 Hz = 900 samples
parse_epoch_string('30 sec', sr=30)

# 1 hour with 1 Hz = 3600 samples
parse_epoch_string('1 hour', sr=1)

# 1 day with 10 Hz = 864000 samples
parse_epoch_string('1 day', sr=10)
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{cut_off_signal}()},
\code{\link{interpolate_signal}()},
\code{\link{sampling_rate}()},
\code{\link{segment_data}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{generate_interactive_plot}
\alias{generate_interactive_plot}
\title{Plot MIMS unit values or raw signal using dygraphs interactive plotting library.}
\usage{
generate_interactive_plot(df, y_label, value_cols = c(2, 3, 4))
}
\arguments{
\item{df}{data.frame.The dataframe storing MIMS unit values or raw
accelerometer signal. The first column should be timestamps.}

\item{y_label}{str. The label name to be put on the y axis.}

\item{value_cols}{numerical vector. The indices of columns storing values,
typically starting from the second column. The default is `c(2,3,4)`.}
}
\value{
A dygraphs graph object. When showing, the graph will be plotted in
  a html widgets in an opened browser.
}
\description{
\code{generate_interactive_plot} plots MIMS unit values or raw signal
  using dygraphs interactive plotting library.
}
\examples{
  # Use sample data for testing
  df = sample_raw_accel_data

  # Plot using default settings, due to pkgdown limitation, no interactive
  # plots will be shown on the website page.
  generate_interactive_plot(df,
                            y_label="Acceleration (g)")

  # The function can be used to plot MIMS unit values as well
  mims = mims_unit(df, dynamic_range=c(-8, 8))
  generate_interactive_plot(mims,
                            y_label="MIMS-unit values",
                            value_cols=c(2))
}
\seealso{
Other visualization functions.: 
\code{\link{illustrate_extrapolation}()},
\code{\link{illustrate_signal}()}
}
\concept{visualization functions.}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{illustrate_extrapolation}
\alias{illustrate_extrapolation}
\title{Plot illustrations about extrapolation in illustration style.}
\usage{
illustrate_extrapolation(
  df,
  dynamic_range,
  title = NULL,
  show_neighbors = TRUE,
  show_extrapolated_points_and_lines = TRUE,
  ...
)
}
\arguments{
\item{df}{data.frame.The original data before extrapolation.}

\item{dynamic_range}{numerical vector. The dynamic ranges of the input
signal. Should be a 2-element numerical vector. \code{c(low, high)}, where
\code{low} is the negative max value the device can reach and \code{high}
is the positive max value the device can reach.}

\item{title}{Char. The title of the plot.}

\item{show_neighbors}{bool. Show the points used for extrapolation if TRUE.}

\item{show_extrapolated_points_and_lines}{bool. Show the extrapolated points
and curves used for extrapolation.}

\item{...}{Parameters that can be used to tune extrapolation, including
\code{spar}, \code{k}, and \code{noise_level}. See \code{\link{extrapolate}}
for explanations.}
}
\value{
ggplot2 graph object. The graph to be shown.
}
\description{
\code{illustrate_extrapolation} plots elements of extrapolations
(e.g., marked points, reference lines) in the same style as
\code{\link{illustrate_signal}}.
}
\examples{
  # Use the maxed-out data for the conceptual diagram
  df = conceptual_diagram_data[
         conceptual_diagram_data['GRANGE'] == 2,
         c("HEADER_TIME_STAMP", "X")]

  # Plot extrapolation illustration using default settings
  illustrate_extrapolation(df, dynamic_range=c(-2,2))

  # Do not show neighbor points
  illustrate_extrapolation(df, dynamic_range=c(-2,2), show_neighbors=FALSE)

  # Do not show extrapolated points and lines
  illustrate_extrapolation(df,
                           dynamic_range=c(-2,2),
                           show_extrapolated_points_and_lines=FALSE)
}
\seealso{
Other visualization functions.: 
\code{\link{generate_interactive_plot}()},
\code{\link{illustrate_signal}()}
}
\concept{visualization functions.}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_mhealth_csv}
\alias{import_mhealth_csv}
\title{Import raw multi-channel accelerometer data stored in mHealth Specification}
\usage{
import_mhealth_csv(filepath)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}
}
\value{
dataframe. The imported multi-channel accelerometer signal, with the
  first column being the timestamps in POSXlct format, and the rest columns
  being accelerometer values in \eqn{g} unit.
}
\description{
\code{import_mhealth_csv} imports the raw multi-channel accelerometer data
  stored in mHealth Specification. Note that this function will fail when
  loading data that have size too big to fit in the memory. For large data
  file, please use \code{\link{import_mhealth_csv_chunked}} to load.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import data stored in mHealth Specification during
  algorithm validation.
}

\examples{
  default_ops = options()
  options(digits.secs=3)
  # Use the sample mhealth csv file provided by the package
  filepath = system.file('extdata', 'mhealth.csv', package='MIMSunit')
  filepath

  # Load the file
  df = import_mhealth_csv(filepath)

  # Check loaded file
  head(df)

  # Check more
  summary(df)

  # Restore default options
  options(default_ops)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_count_csv}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.R
\name{bandlimited_interp}
\alias{bandlimited_interp}
\title{Apply a bandlimited interpolation filter to the signal to change the sampling
rate}
\usage{
bandlimited_interp(df, orig_sr, new_sr)
}
\arguments{
\item{df}{dataframe. The input multi-channel signal. The first column is
timestamps in POSXlct format. The rest columns are signal values.}

\item{orig_sr}{number. Sampling rate in Hz of the input signal.}

\item{new_sr}{number. The desired sampling rate in Hz of the output signal.}
}
\value{
dataframe. Filtered signal.
}
\description{
\code{bandlimited_interp} function takes a multi-channel signal and applies a
bandlimited interpolation filter to the signal to change its sampling rate.
}
\details{
This function filters the input multi-channel signal by applying a bandlimited
interpolation filter. See \code{\link[signal]{resample}} for the underlying
implementation.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is not used in
  the released version of MIMS-unit algorithm, but has once been considered
  to be used after extrapolation to harmonize sampling rate before filtering.
  But in the end, we decided to use linear interpolation before extrapolation
  to increase the sampling rate to 100Hz, so this method is no longer needed.
}

\examples{
  # Use sample data
  df = sample_raw_accel_data

  # View input
  illustrate_signal(df, plot_maxed_out_line = FALSE)

  # Apply filtering that uses the same setting as in MIMSunit algorithm
  output = bandlimited_interp(df, orig_sr=80, new_sr=30)

  # View output
  illustrate_signal(output, plot_maxed_out_line = FALSE)
}
\seealso{
Other filtering functions: 
\code{\link{iir}()}
}
\concept{filtering functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{segment_data}
\alias{segment_data}
\title{Segment input dataframe into windows as specified by breaks.
\code{segment_data} segments the input sensor dataframe into
  epoch windows with length specified in breaks.}
\usage{
segment_data(df, breaks, st = NULL)
}
\arguments{
\item{df}{dataframe. Input dataframe of the multi-channel signal. The first
column is the timestamps in POSXlct format and the following columns are
accelerometer values.}

\item{breaks}{character. An epoch length character that can be accepted by
cut.breaks function.}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the breaks generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate breaks. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}
}
\value{
dataframe. The same format as the input dataframe, but with an extra
  column "SEGMENT" in the end specifies the epoch window a sample belongs to.
}
\description{
This function accepts a dataframe of multi-channel signal, segments it
  into epoch windows with length specified in breaks.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that was used in various part in the algorithm whenever we need to
  segment a dataframe, e.g., before aggregating values over epoch windows.
}

\examples{
  # Use sample data
  df = sample_raw_accel_data

  # segment data into 1 minute segments
  output = segment_data(df, "1 min")

  # check the 3rd segment, each segment would have 1 minute data
  summary(output[output['SEGMENT'] == 3,])

  # segment data into 15 second segments
  output = segment_data(df, "15 sec")

  # check the 1st segment, each segment would have 15 second data
  summary(output[output['SEGMENT'] == 1,])

  # segment data into 1 hour segments
  output = segment_data(df, "1 hour")

  # because the input data has only 15 minute data
  # there will be only 1 segment in the output
  unique(output['SEGMENT'])
  summary(output)

  # use manually set start time
  output = segment_data(df, "15 sec", st='2016-01-15 10:59:50.000')

  # check the 1st segment, because the start time is 10 seconds before the
  # start time of the actual data, the first segment will only include 5 second
  # data.
  summary(output[output['SEGMENT'] == 1,])
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{cut_off_signal}()},
\code{\link{interpolate_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{sampling_rate}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_actigraph_count_csv}
\alias{import_actigraph_count_csv}
\title{Import Actigraph count data stored in Actigraph summary csv format}
\usage{
import_actigraph_count_csv(
  filepath,
  count_col = 2,
  count_per_axis_cols = c(2, 3, 4)
)
}
\arguments{
\item{filepath}{string. The filepath of the input data.}

\item{count_col}{number. The index of column of Actigraph count (combined
axes). If it is \code{NULL}, the function will use
\code{count_per_axis_cols} to get the combined Actigraph count values.}

\item{count_per_axis_cols}{numerical vector. The indices of columns of
Actigraph count values per axis. If \code{count_col} is not \code{NULL},
the argument will be ignored. If it is \code{NULL}, the output dataframe
will only have two columns without Actigraph count values per axis.}
}
\value{
dataframe. The imported actigraph count data, with the first column
  being the timestamps in POSIXct format, and the second column being the
  combined Actigraph count values, and the rest of columns being the
  Actigraph cont values per axis if available. Column names:
  \code{HEADER_TIME_STAMP}, \code{ACTIGRAPH_COUNT},
  \code{ACTIGRAPH_COUNT_X}....
}
\description{
\code{import_actigraph_count_csv} imports Actigraph count data stored in
Actigraph summary csv format, which was exported by Actilife.
}
\note{
If both \code{count_col} and \code{count_per_axis_cols} are
  \code{NULL}, the function will raise an error.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a File IO
  function that is used to import Actigraph count data from Actigraph devices
  during algorithm validation.
}

\examples{
  # Use the actigraph count csv file shipped with the package
  filepath = system.file('extdata', 'actigraph_count.csv', package='MIMSunit')

  # Check original data format
  readLines(filepath)[1:5]

  # Load file, default column for actigraph count values are 2, this file does not have
  # axial count values
  output = import_actigraph_count_csv(filepath, count_col=2)

  # Check output
  head(output)
}
\seealso{
Other File I/O functions: 
\code{\link{export_to_actilife}()},
\code{\link{import_actigraph_csv_chunked}()},
\code{\link{import_actigraph_csv}()},
\code{\link{import_actigraph_meta}()},
\code{\link{import_activpal3_csv}()},
\code{\link{import_enmo_csv}()},
\code{\link{import_mhealth_csv_chunked}()},
\code{\link{import_mhealth_csv}()}
}
\concept{File I/O functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_over_epoch.R
\name{aggregate_for_mims}
\alias{aggregate_for_mims}
\title{Aggregate over epoch to get numerically integrated values.}
\usage{
aggregate_for_mims(
  df,
  epoch,
  method = c("trapz", "power", "mean_by_time", "mean_by_size", "sum"),
  rectify = TRUE,
  st = NULL
)
}
\arguments{
\item{df}{dataframe of accelerometer data in mhealth format. First column
should be timestamps in POSIXt format.}

\item{epoch}{string. Any format that is acceptable by argument \code{breaks}
in method \code{\link[base]{cut.POSIXt}}.For example, "1 sec", "1 min", "5
secs", "10 mins".}

\item{method}{string. Integration methods. Supported strings include:
"trapz", "power", "sum", "meanBySecond", "meanBySize". Default is "trapz".}

\item{rectify}{logical. If TRUE, input data will be rectified before
integration. Default is TRUE.}

\item{st}{character or POSIXct timestamp. An optional start time you can set to
force the epochs generated by referencing this start time. If it is NULL, the
function will use the first timestamp in the timestamp column as start time to
generate epochs. This is useful when you are processing a stream of data and
want to use a common start time for segmenting data. Default is NULL.}
}
\value{
dataframe. The returned dataframe will have the same format as input
  dataframe.
}
\description{
\code{aggregate_for_mims} returns a dataframe with integrated values by
trapzoidal method over each epoch for each column. The epoch start time will
be used as timestamp in the first column.
}
\details{
This function accepts a dataframe (in mhealth accelerometer data format) and
computes its aggregated values over each fixed epoch using different
integration methods (default is trapzoidal method, other methods are not used
by mims unit algorithm) for each value columns. The returned dataframe will
have the same number of columns as input dataframe, and have the same
datetime format as input dataframe in the timestamp column. The trapzoidal
method used in the function is based on \code{\link[caTools]{trapz}}.
}
\note{
If \code{epoch} argument is not provided or is \code{NULL}, the
  function will treat the input dataframe as a single epoch.

  If the number of samples in one segment is less than 90% of the expected
  samples, the aggregation result will be -1 (marker of invalid value).
}
\section{How is it used in mims-unit algorithm?}{
 This function is used in
  mims-unit algorithm after filtering (\code{\link{iir}}). The filtered
  signal will be rectified and integrated to get mims unit values for each
  axis using this function.
}

\examples{
  # sample data
  df = sample_raw_accel_data
  head(df)

  # epoch set to 5 seconds, and method set to "trapz"
  aggregate_for_mims(df, epoch = '5 sec', method='trapz')

  # epoch set to 1 second, method set to "sum"
  aggregate_for_mims(df, epoch = '1 sec', method='sum')

  # epoch set to 1 second, and st set to be 1 second before the start time of the data
  # so the first segment will only include data for 1 second, therefore the resulted
  # aggregated value for the first segment will be -1 (invalid) because the
  # samples are not enough. And the second segment starts from 11:00:01, instead
  # of 11:00:02 as shown in prior example,
  aggregate_for_mims(df, epoch = '1 sec', method='sum', st=df[1,1] - 1)
}
\seealso{
\code{\link{aggregate_for_orientation}} for aggregating to get
  accelerometer orientation estimation for each epoch.

Other aggregate functions: 
\code{\link{aggregate_for_orientation}()}
}
\concept{aggregate functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{measurements_different_devices}
\alias{measurements_different_devices}
\title{The mean and standard deviation of accelerometer summary measure for
different acceleration data summary algorithms and for different devices.}
\format{
A data frame with 235 rows and 8 variables:
\describe{
  \item{DEVICE}{The name of different devices, in character}
  \item{GRANGE}{The dynamic range of the device in g, in number}
  \item{SR}{The sampling rate in Hz of the device, in number}
  \item{TYPE}{Accelerometer summary algorithm name, in character}
  \item{HZ}{The frequency of the elliptical shaker, in number}
  \item{NAME}{An alternative name that is friendly for plotting for devices, in character}
  \item{mean}{The mean values of accelerometer summary measure, in number}
  \item{sd}{The standard deviation values of accelerometer summary measure, in number}
}
}
\source{
\url{https://github.com/mHealthGroup/MIMSunit-dataset-shaker/}
}
\usage{
measurements_different_devices
}
\description{
A dataframe contains the mean and standard deviation of accelerometer summary
measured at different frequencies for the raw accelerometer signals
from different devices collected from on a standard elliptical shaker.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpolate.R
\name{interpolate_signal}
\alias{interpolate_signal}
\title{Interpolate missing points and unify sampling rate for multi-channel signal}
\usage{
interpolate_signal(
  df,
  method = "spline_natural",
  sr = 100,
  st = NULL,
  et = NULL
)
}
\arguments{
\item{df}{dataframe. Input multi-channel accelerometer signal.}

\item{method}{string. Interpolation algorithms. Could be "spline_natural",
"spline_improved" or "spline_fmm": see \code{\link[stats]{splinefun}};
and "linear": see \code{\link[stats]{approxfun}}. Default is "spline_natural".}

\item{sr}{number. Sampling rate in Hz of the output signal. Default is 100.}

\item{st}{POSIXct date. The start time for interpolation. If it is
\code{NULL}, it will use the start time of the input signal. Default is
\code{NULL}.}

\item{et}{POSIXct date. The end time for interpolation. If it is \code{NULL},
it will use the end time of the input signal. Default is \code{NULL}.}
}
\value{
dataframe. Interpolated signal.
}
\description{
\code{interpolate_signal} applies different interpolation algorithms to the
input multi-channel signal to fill in the missing samples and harmonizes the
sampling rate.
}
\section{How is it used in MIMS-unit algorithm?}{
 This function is a utility
  function that has been used in functions: \code{\link{extrapolate}}, and
  \code{\link{simulate_new_data}}.
}

\examples{
  # Use sample data
  df = sample_raw_accel_data

  # Plot input
  illustrate_signal(df, plot_maxed_out_line=FALSE)

  # Interpolate to 100 Hz
  sr = 100

  # Interpolate the entire sequence of data
  output = interpolate_signal(df, sr=sr)

  # Plot output
  illustrate_signal(output, plot_maxed_out_line=FALSE)

  # Interpolate part of the sequence
  output = interpolate_signal(df, sr=sr, st=df[10,1], et=df[100,1])

  # Plot output
  illustrate_signal(output, plot_maxed_out_line=FALSE)
}
\seealso{
Other utility functions: 
\code{\link{clip_data}()},
\code{\link{cut_off_signal}()},
\code{\link{parse_epoch_string}()},
\code{\link{sampling_rate}()},
\code{\link{segment_data}()},
\code{\link{simulate_new_data}()}
}
\concept{utility functions}
